@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2021 Taler Systems SA
+  (C) 2014-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -25,6 +25,7 @@
 #include <taler/taler_dbevents.h>
 #include <taler/taler_bank_service.h>
 #include <taler/taler_mhd_lib.h>
+#include <taler/taler_templating_lib.h>
 #include <taler/taler_exchange_service.h>
 #include "taler-merchant-httpd_auditors.h"
 #include "taler-merchant-httpd_config.h"
@@ -39,6 +40,7 @@
 #include "taler-merchant-httpd_private-delete-transfers-ID.h"
 #include "taler-merchant-httpd_private-get-instances.h"
 #include "taler-merchant-httpd_private-get-instances-ID.h"
+#include "taler-merchant-httpd_private-get-instances-ID-kyc.h"
 #include "taler-merchant-httpd_private-get-products.h"
 #include "taler-merchant-httpd_private-get-products-ID.h"
 #include "taler-merchant-httpd_private-get-orders.h"
@@ -69,7 +71,7 @@
 #include "taler-merchant-httpd_reserves.h"
 #include "taler-merchant-httpd_spa.h"
 #include "taler-merchant-httpd_statics.h"
-#include "taler-merchant-httpd_templating.h"
+
 
 /**
  * Fixme: document.
@@ -136,23 +138,6 @@ static int merchant_connection_close;
  */
 static int result;
 
-/**
- * MIN-Heap of suspended connections to resume when the timeout expires,
- * ordered by timeout. Values are of type `struct MHD_Connection`
- */
-static struct GNUNET_CONTAINER_Heap *resume_timeout_heap;
-
-/**
- * Hash map from H(order_id,merchant_pub) to `struct MHD_Connection`
- * entries to resume when a payment is made for the given order.
- */
-static struct GNUNET_CONTAINER_MultiHashMap *payment_trigger_map;
-
-/**
- * Task responsible for timeouts in the #resume_timeout_heap.
- */
-static struct GNUNET_SCHEDULER_Task *resume_timeout_task;
-
 /**
  * Our configuration.
  */
@@ -164,48 +149,39 @@ static const struct GNUNET_CONFIGURATION_Handle *cfg;
 char *TMH_default_auth;
 
 
-/**
- * Holds data needed to determine when to resume a connection for
- * GET /orders/$ORDER_ID
- */
-struct ResumeData
-{
-  /**
-   * How much of the order has been refunded.
-   */
-  const struct TALER_Amount *refund_amount;
-
-  /**
-   * Whether the refunds for the order were obtained.
-   */
-  bool obtained;
-};
-
-
-int
+enum GNUNET_GenericReturnValue
 TMH_check_auth (const char *token,
-                const struct GNUNET_ShortHashCode *salt,
-                const struct GNUNET_HashCode *hash)
+                struct TALER_MerchantAuthenticationSaltP *salt,
+                struct TALER_MerchantAuthenticationHashP *hash)
 {
   struct GNUNET_HashCode val;
+  char *dec;
+  size_t dec_len;
 
   if (GNUNET_is_zero (hash))
     return GNUNET_OK;
   if (NULL == token)
     return GNUNET_SYSERR;
+  dec_len = GNUNET_STRINGS_urldecode (token,
+                                      strlen (token),
+                                      &dec);
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Checking against token with salt %s\n",
+              TALER_B2S (salt));
   GNUNET_assert (GNUNET_YES ==
                  GNUNET_CRYPTO_kdf (&val,
                                     sizeof (val),
                                     salt,
                                     sizeof (*salt),
-                                    token,
-                                    strlen (token),
+                                    dec,
+                                    dec_len,
                                     "merchant-instance-auth",
                                     strlen ("merchant-instance-auth"),
                                     NULL,
                                     0));
+  GNUNET_free (dec);
   return (0 == GNUNET_memcmp (&val,
-                              hash))
+                              &hash->hash))
     ? GNUNET_OK
     : GNUNET_SYSERR;
 }
@@ -213,12 +189,15 @@ TMH_check_auth (const char *token,
 
 void
 TMH_compute_auth (const char *token,
-                  struct GNUNET_ShortHashCode *salt,
-                  struct GNUNET_HashCode *hash)
+                  struct TALER_MerchantAuthenticationSaltP *salt,
+                  struct TALER_MerchantAuthenticationHashP *hash)
 {
   GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_NONCE,
                               salt,
                               sizeof (*salt));
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Computing initial auth using token with salt %s\n",
+              TALER_B2S (salt));
   GNUNET_assert (GNUNET_YES ==
                  GNUNET_CRYPTO_kdf (hash,
                                     sizeof (*hash),
@@ -233,13 +212,8 @@ TMH_compute_auth (const char *token,
 }
 
 
-/**
- * Decrement reference counter of @a mi, and free if it hits zero.
- *
- * @param[in,out] mi merchant instance to update and possibly free
- */
-static void
-instance_decref (struct TMH_MerchantInstance *mi)
+void
+TMH_instance_decref (struct TMH_MerchantInstance *mi)
 {
   struct TMH_WireMethod *wm;
 
@@ -252,28 +226,23 @@ instance_decref (struct TMH_MerchantInstance *mi)
     GNUNET_CONTAINER_DLL_remove (mi->wm_head,
                                  mi->wm_tail,
                                  wm);
-    json_decref (wm->j_wire);
+    GNUNET_free (wm->payto_uri);
     GNUNET_free (wm->wire_method);
     GNUNET_free (wm);
   }
 
   GNUNET_free (mi->settings.id);
   GNUNET_free (mi->settings.name);
+  GNUNET_free (mi->settings.email);
+  GNUNET_free (mi->settings.website);
+  GNUNET_free (mi->settings.logo);
   json_decref (mi->settings.address);
   json_decref (mi->settings.jurisdiction);
   GNUNET_free (mi);
 }
 
 
-/**
- * Callback that frees an instances removing
- * it from the global hashmap.
- *
- * @param cls closure, NULL
- * @param key current key
- * @param value a `struct TMH_MerchantInstance`
- */
-int
+enum GNUNET_GenericReturnValue
 TMH_instance_free_cb (void *cls,
                       const struct GNUNET_HashCode *key,
                       void *value)
@@ -286,398 +255,11 @@ TMH_instance_free_cb (void *cls,
                  GNUNET_CONTAINER_multihashmap_remove (TMH_by_id_map,
                                                        &mi->h_instance,
                                                        mi));
-  instance_decref (mi);
+  TMH_instance_decref (mi);
   return GNUNET_YES;
 }
 
 
-/**
- * Callback that frees all the elements in the #payment_trigger_map.
- * This function should actually never be called, as by the time we
- * get to it, all payment triggers should have been cleaned up!
- *
- * @param cls closure, NULL
- * @param key current key
- * @param value a `struct TMH_SuspendedConnection`
- * @return #GNUNET_OK
- */
-static int
-payment_trigger_free (void *cls,
-                      const struct GNUNET_HashCode *key,
-                      void *value)
-{
-  struct TMH_SuspendedConnection *sc = value;
-
-  (void) cls;
-  (void) key;
-  (void) sc; /* cannot really 'clean up' */
-  GNUNET_break (0);
-  return GNUNET_OK;
-}
-
-
-/**
- * Compute @a key to use for @a order_id and @a mpub in our
- * #payment_trigger_map.
- *
- * @param order_id an order ID
- * @param mpub an instance public key
- * @param[out] key set to the hash map key to use
- */
-static void
-compute_pay_key (const char *order_id,
-                 const struct TALER_MerchantPublicKeyP *mpub,
-                 struct GNUNET_HashCode *key)
-{
-  size_t olen = strlen (order_id);
-  char buf[sizeof (*mpub) + olen];
-
-  /* sanity check for arithmetic overflow */
-  GNUNET_assert (olen < 1024 * 1024);
-  memcpy (buf,
-          mpub,
-          sizeof (*mpub));
-  memcpy (&buf[sizeof (*mpub)],
-          order_id,
-          olen);
-  GNUNET_CRYPTO_hash (buf,
-                      sizeof (buf),
-                      key);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Pay key for `%s' is %s\n",
-              order_id,
-              GNUNET_h2s (key));
-}
-
-
-/**
- * Compute @a key to use for @a session_id and @a fulfillment_url in our
- * #payment_trigger_map.
- *
- * @param session_id the session for which @a fulfillment_url matters
- * @param fulfillment_url fullfillment URL of an order
- * @param[out] key set to the hash map key to use
- */
-static void
-compute_pay_key2 (const char *session_id,
-                  const char *fulfillment_url,
-                  struct GNUNET_HashCode *key)
-{
-  size_t slen = strlen (session_id) + 1;
-  size_t ulen = strlen (fulfillment_url) + 1;
-  char buf[slen + ulen];
-
-  memcpy (buf,
-          session_id,
-          slen);
-  memcpy (&buf[slen],
-          fulfillment_url,
-          ulen);
-  GNUNET_CRYPTO_hash (buf,
-                      sizeof (buf),
-                      key);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Pay key2 for %s/%s is %s\n",
-              session_id,
-              fulfillment_url,
-              GNUNET_h2s (key));
-}
-
-
-/**
- * Resume processing all suspended connections past timeout.
- *
- * @param cls unused
- */
-static void
-do_resume (void *cls)
-{
-  struct TMH_SuspendedConnection *sc;
-
-  (void) cls;
-  resume_timeout_task = NULL;
-  while (1)
-  {
-    sc = GNUNET_CONTAINER_heap_peek (resume_timeout_heap);
-    if (NULL == sc)
-      return;
-    if (GNUNET_TIME_absolute_is_future (sc->long_poll_timeout))
-      break;
-    GNUNET_assert (sc ==
-                   GNUNET_CONTAINER_heap_remove_root (resume_timeout_heap));
-    sc->hn = NULL;
-    GNUNET_assert (GNUNET_YES ==
-                   GNUNET_CONTAINER_multihashmap_remove (payment_trigger_map,
-                                                         &sc->key,
-                                                         sc));
-    if (sc->has_key2)
-      GNUNET_assert (GNUNET_YES ==
-                     GNUNET_CONTAINER_multihashmap_remove (payment_trigger_map,
-                                                           &sc->key2,
-                                                           sc));
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Resuming long polled job due to timeout\n");
-    MHD_resume_connection (sc->con);
-    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
-  }
-  if (NULL != resume_timeout_task)
-    GNUNET_SCHEDULER_cancel (resume_timeout_task);
-  resume_timeout_task = GNUNET_SCHEDULER_add_at (sc->long_poll_timeout,
-                                                 &do_resume,
-                                                 NULL);
-}
-
-
-/**
- * Suspend connection from @a sc until payment has been received.
- *
- * @param order_id the order that we are waiting on
- * @param session_id session ID of the requester
- * @param fulfillment_url fulfillment URL of the contract
- * @param mi the merchant instance we are waiting on
- * @param sc connection to suspend
- * @param min_refund refund amount we are waiting on to be exceeded before resuming,
- *                   NULL if we are not waiting for refunds
- */
-void
-TMH_long_poll_suspend (const char *order_id,
-                       const char *session_id,
-                       const char *fulfillment_url,
-                       const struct TMH_MerchantInstance *mi,
-                       struct TMH_SuspendedConnection *sc,
-                       const struct TALER_Amount *min_refund)
-{
-  compute_pay_key (order_id,
-                   &mi->merchant_pub,
-                   &sc->key);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Suspending operation on key %s\n",
-              GNUNET_h2s (&sc->key));
-  GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CONTAINER_multihashmap_put (payment_trigger_map,
-                                                    &sc->key,
-                                                    sc,
-                                                    GNUNET_CONTAINER_MULTIHASHMAPOPTION_MULTIPLE));
-  if ( (NULL != session_id) &&
-       (NULL != fulfillment_url) )
-  {
-    sc->has_key2 = true;
-    compute_pay_key2 (session_id,
-                      fulfillment_url,
-                      &sc->key2);
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Suspending operation on %s/%s key2 %s\n",
-                session_id,
-                fulfillment_url,
-                GNUNET_h2s (&sc->key2));
-    GNUNET_assert (GNUNET_OK ==
-                   GNUNET_CONTAINER_multihashmap_put (payment_trigger_map,
-                                                      &sc->key2,
-                                                      sc,
-                                                      GNUNET_CONTAINER_MULTIHASHMAPOPTION_MULTIPLE));
-  }
-  if (NULL != min_refund)
-  {
-    sc->awaiting_refund = true;
-    sc->refund_expected = *min_refund;
-  }
-  sc->hn = GNUNET_CONTAINER_heap_insert (resume_timeout_heap,
-                                         sc,
-                                         sc->long_poll_timeout.abs_value_us);
-  MHD_suspend_connection (sc->con);
-  if (NULL != resume_timeout_task)
-  {
-    GNUNET_SCHEDULER_cancel (resume_timeout_task);
-    resume_timeout_task = NULL;
-  }
-  sc = GNUNET_CONTAINER_heap_peek (resume_timeout_heap);
-  resume_timeout_task = GNUNET_SCHEDULER_add_at (sc->long_poll_timeout,
-                                                 &do_resume,
-                                                 NULL);
-}
-
-
-/**
- * Function called to resume suspended connections.
- *
- * @param cls pointer to a `struct TALER_Amount` indicating the refund amount, or NULL
- * @param key key in the #payment_trigger_map
- * @param value a `struct TMH_SuspendedConnection` to resume
- * @return #GNUNET_OK (continue to iterate)
- */
-static int
-resume_operation (void *cls,
-                  const struct GNUNET_HashCode *key,
-                  void *value)
-{
-  const struct ResumeData *rd = cls;
-  struct TMH_SuspendedConnection *sc = value;
-
-  GNUNET_assert (0 ==
-                 GNUNET_memcmp (key,
-                                &sc->key));
-  /* If the conditions are satisfied partially, turn them off for future
-     calls. */
-  if ( (sc->awaiting_refund_obtained) &&
-       (rd->obtained))
-    sc->awaiting_refund_obtained = false;
-  if ( (sc->awaiting_refund) &&
-       ( (NULL != rd->refund_amount) &&
-         (1 == TALER_amount_cmp (rd->refund_amount,
-                                 &sc->refund_expected)) ) )
-    sc->awaiting_refund = false;
-
-  if ( (sc->awaiting_refund_obtained) &&
-       (! rd->obtained))
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Not awaking client, refunds not yet obtained\n");
-    return GNUNET_OK;
-  }
-  if ( (sc->awaiting_refund) &&
-       ( (NULL == rd->refund_amount) ||
-         (1 != TALER_amount_cmp (rd->refund_amount,
-                                 &sc->refund_expected)) ) )
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Not awaking client, refund amount of %s not yet satisfied\n",
-                TALER_amount2s (&sc->refund_expected));
-    return GNUNET_OK; /* skip */
-  }
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Resuming operation suspended pending payment on key %s\n",
-              GNUNET_h2s (key));
-  GNUNET_assert (GNUNET_YES ==
-                 GNUNET_CONTAINER_multihashmap_remove (payment_trigger_map,
-                                                       key,
-                                                       sc));
-  if (sc->has_key2)
-    GNUNET_assert (GNUNET_YES ==
-                   GNUNET_CONTAINER_multihashmap_remove (payment_trigger_map,
-                                                         &sc->key2,
-                                                         sc));
-  GNUNET_assert (sc ==
-                 GNUNET_CONTAINER_heap_remove_node (sc->hn));
-  sc->hn = NULL;
-  MHD_resume_connection (sc->con);
-  TALER_MHD_daemon_trigger ();
-  return GNUNET_OK;
-}
-
-
-/**
- * Find out if we have any clients long-polling for @a order_id to be
- * confirmed at merchant @a mpub, and if so, tell them to resume.
- *
- * @param order_id the order that was paid or refunded
- * @param mi the merchant instance where the payment or refund happened
- * @param refund_amount refunded amount, if the trigger was a refund, otherwise NULL
- * @param obtained if true, the wallet has obtained the refunds for the order
- */
-void
-TMH_long_poll_resume (const char *order_id,
-                      const struct TMH_MerchantInstance *mi,
-                      const struct TALER_Amount *refund_amount,
-                      bool obtained)
-{
-  struct GNUNET_HashCode key;
-  struct ResumeData rd = {
-    .refund_amount = refund_amount,
-    .obtained = obtained
-  };
-  int ret;
-
-  compute_pay_key (order_id,
-                   &mi->merchant_pub,
-                   &key);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Resuming operations suspended pending payment on key %s up to refund %s\n",
-              GNUNET_h2s (&key),
-              (NULL != refund_amount)
-              ? TALER_amount2s (refund_amount)
-              : "<none>");
-  ret = GNUNET_CONTAINER_multihashmap_get_multiple (payment_trigger_map,
-                                                    &key,
-                                                    &resume_operation,
-                                                    (void *) &rd);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "%u operations remain suspended pending payment (%d)\n",
-              GNUNET_CONTAINER_multihashmap_size (payment_trigger_map),
-              ret);
-}
-
-
-/**
- * Function called to resume suspended connections.
- *
- * @param cls NULL
- * @param key key in the #payment_trigger_map
- * @param value a `struct TMH_SuspendedConnection` to resume
- * @return #GNUNET_OK (continue to iterate)
- */
-static int
-resume_operation2 (void *cls,
-                   const struct GNUNET_HashCode *key,
-                   void *value)
-{
-  struct TMH_SuspendedConnection *sc = value;
-
-  GNUNET_assert (sc->has_key2);
-  GNUNET_assert (0 == GNUNET_memcmp (key,
-                                     &sc->key2));
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Resuming operation suspended pending payment on key %s\n",
-              GNUNET_h2s (key));
-  GNUNET_assert (GNUNET_YES ==
-                 GNUNET_CONTAINER_multihashmap_remove (payment_trigger_map,
-                                                       &sc->key,
-                                                       sc));
-  GNUNET_assert (GNUNET_YES ==
-                 GNUNET_CONTAINER_multihashmap_remove (payment_trigger_map,
-                                                       &sc->key2,
-                                                       sc));
-  GNUNET_assert (sc ==
-                 GNUNET_CONTAINER_heap_remove_node (sc->hn));
-  sc->hn = NULL;
-  MHD_resume_connection (sc->con);
-  TALER_MHD_daemon_trigger ();
-  return GNUNET_OK;
-}
-
-
-/**
- * Find out if we have any clients long-polling for @a order_id to be
- * confirmed at merchant @a mpub, and if so, tell them to resume.
- *
- * @param session_id the session for which @a fulfillment_url became paid
- * @param fulfillment_url fullfillment URL of which an order was paid
- */
-void
-TMH_long_poll_resume2 (const char *session_id,
-                       const char *fulfillment_url)
-{
-  struct GNUNET_HashCode key;
-  int ret;
-
-  compute_pay_key2 (session_id,
-                    fulfillment_url,
-                    &key);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Resuming operations suspended pending payment on %s/%s with key2 %s\n",
-              session_id,
-              fulfillment_url,
-              GNUNET_h2s (&key));
-  ret = GNUNET_CONTAINER_multihashmap_get_multiple (payment_trigger_map,
-                                                    &key,
-                                                    &resume_operation2,
-                                                    NULL);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "%u operations remain suspended pending payment (%d)\n",
-              GNUNET_CONTAINER_multihashmap_size (payment_trigger_map),
-              ret);
-}
-
-
 /**
  * Shutdown task (magically invoked when the application is being
  * quit)
@@ -687,38 +269,22 @@ TMH_long_poll_resume2 (const char *session_id,
 static void
 do_shutdown (void *cls)
 {
-  struct TMH_SuspendedConnection *sc;
-  struct MHD_Daemon *mhd;
-
   (void) cls;
   TMH_force_ac_resume ();
   TMH_force_pc_resume ();
+  TMH_force_kyc_resume ();
   TMH_force_rc_resume ();
+  TMH_force_gorc_resume ();
   TMH_force_post_transfers_resume ();
   TMH_force_tip_pickup_resume ();
   TMH_force_wallet_get_order_resume ();
   TMH_force_wallet_refund_order_resume ();
-  mhd = TALER_MHD_daemon_stop ();
-  /* resume all suspended connections, must be done before stopping #mhd */
-  if (NULL != resume_timeout_heap)
   {
-    while (NULL != (sc = GNUNET_CONTAINER_heap_remove_root (
-                      resume_timeout_heap)))
-    {
-      sc->hn = NULL;
-      GNUNET_assert (GNUNET_YES ==
-                     GNUNET_CONTAINER_multihashmap_remove (payment_trigger_map,
-                                                           &sc->key,
-                                                           sc));
-      MHD_resume_connection (sc->con);
-    }
-    GNUNET_CONTAINER_heap_destroy (resume_timeout_heap);
-    resume_timeout_heap = NULL;
-  }
-  if (NULL != mhd)
-  {
-    MHD_stop_daemon (mhd);
-    mhd = NULL;
+    struct MHD_Daemon *mhd;
+
+    mhd = TALER_MHD_daemon_stop ();
+    if (NULL != mhd)
+      MHD_stop_daemon (mhd);
   }
   TMH_RESERVES_done ();
   if (NULL != instance_eh)
@@ -733,19 +299,6 @@ do_shutdown (void *cls)
   }
   TMH_EXCHANGES_done ();
   TMH_AUDITORS_done ();
-  if (NULL != resume_timeout_task)
-  {
-    GNUNET_SCHEDULER_cancel (resume_timeout_task);
-    resume_timeout_task = NULL;
-  }
-  if (NULL != payment_trigger_map)
-  {
-    GNUNET_CONTAINER_multihashmap_iterate (payment_trigger_map,
-                                           &payment_trigger_free,
-                                           NULL);
-    GNUNET_CONTAINER_multihashmap_destroy (payment_trigger_map);
-    payment_trigger_map = NULL;
-  }
   if (NULL != TMH_by_id_map)
   {
     GNUNET_CONTAINER_multihashmap_iterate (TMH_by_id_map,
@@ -754,6 +307,7 @@ do_shutdown (void *cls)
     GNUNET_CONTAINER_multihashmap_destroy (TMH_by_id_map);
     TMH_by_id_map = NULL;
   }
+  TALER_TEMPLATING_done ();
 }
 
 
@@ -779,6 +333,7 @@ handle_mhd_completion_callback (void *cls,
 {
   struct TMH_HandlerContext *hc = *con_cls;
 
+  (void) cls;
   if (NULL == hc)
     return;
   GNUNET_SCHEDULER_begin_async_scope (&hc->async_scope_id);
@@ -811,7 +366,7 @@ handle_mhd_completion_callback (void *cls,
   if (NULL != hc->request_body)
     json_decref (hc->request_body);
   if (NULL != hc->instance)
-    instance_decref (hc->instance);
+    TMH_instance_decref (hc->instance);
   GNUNET_free (hc);
   *con_cls = NULL;
 }
@@ -844,7 +399,7 @@ TMH_lookup_instance (const char *instance_id)
  * @param[in,out] mi merchant instance details to define
  * @return #GNUNET_OK on success, #GNUNET_NO if the same ID is in use already
  */
-int
+enum GNUNET_GenericReturnValue
 TMH_add_instance (struct TMH_MerchantInstance *mi)
 {
   const char *id;
@@ -886,6 +441,8 @@ handle_server_options (const struct TMH_RequestHandler *rh,
                        struct MHD_Connection *connection,
                        struct TMH_HandlerContext *hc)
 {
+  (void) rh;
+  (void) hc;
   return TALER_MHD_reply_cors_preflight (connection);
 }
 
@@ -1099,8 +656,18 @@ url_handler (void *cls,
       /* Body should be pretty small. */
       .max_upload = 1024 * 1024
     },
+    /* POST /kyc: */
     {
-      NULL
+      .url_prefix = "/instances/",
+      .url_suffix = "kyc",
+      .method = MHD_HTTP_METHOD_GET,
+      .skip_instance = true,
+      .default_only = true,
+      .have_id_segment = true,
+      .handler = &TMH_private_get_instances_default_ID_kyc,
+    },
+    {
+      .url_prefix = NULL
     }
   };
 
@@ -1138,6 +705,12 @@ url_handler (void *cls,
       /* Body should be pretty small. */
       .max_upload = 1024 * 1024,
     },
+    /* GET /kyc: */
+    {
+      .url_prefix = "/kyc",
+      .method = MHD_HTTP_METHOD_GET,
+      .handler = &TMH_private_get_instances_ID_kyc,
+    },
     /* GET /products: */
     {
       .url_prefix = "/products",
@@ -1348,7 +921,7 @@ url_handler (void *cls,
       .handler = &TMH_private_get_transfers
     },
     {
-      NULL
+      .url_prefix = NULL
     }
   };
   static struct TMH_RequestHandler public_handlers[] = {
@@ -1479,7 +1052,7 @@ url_handler (void *cls,
       .handler = &handle_server_options
     },
     {
-      NULL
+      .url_prefix = NULL
     }
   };
   struct TMH_HandlerContext *hc = *con_cls;
@@ -1572,6 +1145,7 @@ url_handler (void *cls,
                        MHD_HTTP_METHOD_HEAD))
     method = MHD_HTTP_METHOD_GET; /* MHD will deal with the rest */
 
+
   /* Find out the merchant backend instance for the request.
    * If there is an instance, remove the instance specification
    * from the beginning of the request URL. */
@@ -1593,6 +1167,24 @@ url_handler (void *cls,
         instance_id = GNUNET_strndup (istart,
                                       slash - istart);
       hc->instance = TMH_lookup_instance (instance_id);
+      if ( (NULL == hc->instance) &&
+           (0 == strcmp ("default",
+                         instance_id)) )
+        hc->instance = TMH_lookup_instance (NULL);
+      if ( (0 == strcmp ("default",
+                         instance_id)) &&
+           (NULL != TMH_default_auth) &&
+           (NULL != hc->instance) )
+      {
+        /* Override default instance access control */
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Command-line override of access control\n");
+        TMH_compute_auth (TMH_default_auth,
+                          &hc->instance->auth.auth_salt,
+                          &hc->instance->auth.auth_hash);
+        hc->instance->auth_override = true;
+        GNUNET_free (TMH_default_auth);
+      }
       GNUNET_free (instance_id);
       if (NULL == slash)
         url = "";
@@ -1608,9 +1200,12 @@ url_handler (void *cls,
            (NULL != hc->instance) )
       {
         /* Override default instance access control */
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Command-line override of access control\n");
         TMH_compute_auth (TMH_default_auth,
                           &hc->instance->auth.auth_salt,
                           &hc->instance->auth.auth_hash);
+        hc->instance->auth_override = true;
         GNUNET_free (TMH_default_auth);
       }
     }
@@ -1787,10 +1382,15 @@ url_handler (void *cls,
         return ret;
       }
       if (NULL == hc->rh)
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                    "Endpoint `%s' not known\n",
+                    hc->url);
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_NOT_FOUND,
                                            TALER_EC_GENERIC_ENDPOINT_UNKNOWN,
                                            hc->url);
+      }
     }
   }
   /* At this point, we must have found a handler */
@@ -1800,6 +1400,9 @@ url_handler (void *cls,
   if ( (NULL == hc->instance) &&
        (! hc->rh->skip_instance) )
   {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Instance `%s' not known\n",
+                hc->infix);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_NOT_FOUND,
                                        TALER_EC_MERCHANT_GENERIC_INSTANCE_UNKNOWN,
@@ -1845,12 +1448,12 @@ url_handler (void *cls,
                                   &hc->instance->auth.auth_salt,
                                   &hc->instance->auth.auth_hash));
     else /* Are the credentials provided OK for CLI override? */
-      auth_ok |= ( (use_default) &&
-                   (NULL != TMH_default_auth) &&
-                   (NULL != auth) &&
-                   (! auth_malformed) &&
-                   (0 == strcmp (auth,
-                                 TMH_default_auth)) );
+      auth_ok |= (use_default &&
+                  (NULL != TMH_default_auth) &&
+                  (NULL != auth) &&
+                  (! auth_malformed) &&
+                  (0 == strcmp (auth,
+                                TMH_default_auth)) );
     if (! auth_ok)
     {
       if (auth_malformed)
@@ -1869,18 +1472,28 @@ url_handler (void *cls,
 
   if ( (NULL == hc->instance) &&
        (! hc->rh->skip_instance) )
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Instance for URL `%s' not known\n",
+                url);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_NOT_FOUND,
                                        TALER_EC_MERCHANT_GENERIC_INSTANCE_UNKNOWN,
                                        url);
+  }
   if ( (NULL != hc->instance) && /* make static analysis happy */
        (! hc->rh->skip_instance) &&
        (hc->instance->deleted) &&
        (! hc->rh->allow_deleted_instance) )
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Instance `%s' was deleted\n",
+                hc->instance->settings.id);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_NOT_FOUND,
                                        TALER_EC_MERCHANT_GENERIC_INSTANCE_DELETED,
                                        hc->instance->settings.id);
+  }
   /* parse request body */
   hc->has_body = ( (0 == strcasecmp (method,
                                      MHD_HTTP_METHOD_POST)) ||
@@ -1966,6 +1579,12 @@ add_instance_cb (void *cls,
   mi->auth = *ias;
   mi->settings.id = GNUNET_strdup (mi->settings.id);
   mi->settings.name = GNUNET_strdup (mi->settings.name);
+  if (NULL != mi->settings.email)
+    mi->settings.email = GNUNET_strdup (mi->settings.email);
+  if (NULL != mi->settings.website)
+    mi->settings.website = GNUNET_strdup (mi->settings.website);
+  if (NULL != mi->settings.logo)
+    mi->settings.logo = GNUNET_strdup (mi->settings.logo);
   mi->settings.address = json_incref (mi->settings.address);
   mi->settings.jurisdiction = json_incref (mi->settings.jurisdiction);
   if (NULL != merchant_priv)
@@ -1980,11 +1599,8 @@ add_instance_cb (void *cls,
 
     wm = GNUNET_new (struct TMH_WireMethod);
     wm->h_wire = acc->h_wire;
-    wm->j_wire = GNUNET_JSON_PACK (
-      GNUNET_JSON_pack_string ("payto_uri",
-                               acc->payto_uri),
-      GNUNET_JSON_pack_data_auto ("salt",
-                                  &acc->salt));
+    wm->payto_uri = GNUNET_strdup (acc->payto_uri);
+    wm->wire_salt = acc->salt;
     wm->wire_method = TALER_payto_get_method (acc->payto_uri);
     wm->active = acc->active;
     GNUNET_CONTAINER_DLL_insert (mi->wm_head,
@@ -2019,7 +1635,7 @@ load_instances (void *cls,
        ( (0 == extra_len) ||
          ('\0' != id[extra_len - 1]) ) )
   {
-    GNUNET_break (0); /* bogus notification */
+    GNUNET_break (0 == extra_len);
     extra = NULL;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -2054,7 +1670,9 @@ load_instances (void *cls,
   }
   if (0 > qs)
   {
-    GNUNET_break (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed initialization. Check database setup.\n");
+    result = EXIT_FAILURE;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -2072,8 +1690,8 @@ void
 TMH_reload_instances (const char *id)
 {
   struct GNUNET_DB_EventHeaderP es = {
-    es.size = ntohs (sizeof (es)),
-    es.type = ntohs (sizeof (TALER_DBEVENT_MERCHANT_INSTANCE_SETTINGS))
+    .size = ntohs (sizeof (es)),
+    .type = ntohs (TALER_DBEVENT_MERCHANT_INSTANCE_SETTINGS)
   };
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -2116,6 +1734,20 @@ run (void *cls,
   if ( (NULL != tok) &&
        (NULL == TMH_default_auth) )
     TMH_default_auth = GNUNET_strdup (tok);
+  if ( (NULL != TMH_default_auth) &&
+       (0 != strncmp (TMH_default_auth,
+                      RFC_8959_PREFIX,
+                      strlen (RFC_8959_PREFIX))) )
+  {
+    char *tmp;
+
+    GNUNET_asprintf (&tmp,
+                     "%s%s",
+                     RFC_8959_PREFIX,
+                     TMH_default_auth);
+    GNUNET_free (TMH_default_auth);
+    TMH_default_auth = tmp;
+  }
   cfg = config;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Starting taler-merchant-httpd\n");
@@ -2127,11 +1759,6 @@ run (void *cls,
   result = GNUNET_SYSERR;
   GNUNET_SCHEDULER_add_shutdown (&do_shutdown,
                                  NULL);
-  resume_timeout_heap
-    = GNUNET_CONTAINER_heap_create (GNUNET_CONTAINER_HEAP_ORDER_MIN);
-  payment_trigger_map
-    = GNUNET_CONTAINER_multihashmap_create (16,
-                                            GNUNET_YES);
   if (GNUNET_OK !=
       TALER_config_get_currency (cfg,
                                  &TMH_currency))
@@ -2156,7 +1783,7 @@ run (void *cls,
                                             "merchant",
                                             "FORCE_AUDIT"))
     TMH_force_audit = GNUNET_YES;
-  TMH_templating_init ();
+  TALER_TEMPLATING_init ("merchant");
   if (GNUNET_OK !=
       TMH_spa_init ())
   {
@@ -2208,8 +1835,8 @@ run (void *cls,
   }
   {
     struct GNUNET_DB_EventHeaderP es = {
-      es.size = ntohs (sizeof (es)),
-      es.type = ntohs (sizeof (TALER_DBEVENT_MERCHANT_INSTANCE_SETTINGS))
+      .size = ntohs (sizeof (es)),
+      .type = ntohs (TALER_DBEVENT_MERCHANT_INSTANCE_SETTINGS)
     };
 
     instance_eh = TMH_db->event_listen (TMH_db->cls,
@@ -2284,6 +1911,7 @@ main (int argc,
                                  "TOKEN",
                                  "use TOKEN to initially authenticate access to the default instance (you can also set the TALER_MERCHANT_TOKEN environment variable instead)",
                                  &TMH_default_auth),
+    GNUNET_GETOPT_option_version (PACKAGE_VERSION "-" VCS_VERSION),
     GNUNET_GETOPT_OPTION_END
   };
 
