@@ -51,19 +51,24 @@ struct TMH_WireMethod
   struct TMH_WireMethod *prev;
 
   /**
-   * Which wire method / payment target identifier is @e j_wire using?
+   * Which wire method / payment target identifier is @e payto_uri using?
    */
   char *wire_method;
 
   /**
    * Wire details for this instance
    */
-  struct json_t *j_wire;
+  char *payto_uri;
 
   /**
-   * Hash of our wire format details as given in #j_wire.
+   * Salt to use when computing @e h_wire from @e payto_uri.
    */
-  struct GNUNET_HashCode h_wire;
+  struct TALER_WireSaltP wire_salt;
+
+  /**
+   * Hash of our wire format details as given in @e payto_uri
+   */
+  struct TALER_MerchantWireHashP h_wire;
 
   /**
    * Is this wire method active (should it be included in new contracts)?
@@ -122,6 +127,12 @@ struct TMH_MerchantInstance
    */
   struct TMH_PendingOrder *po_tail;
 
+  /**
+   * Database event we are waiting on to be resuming
+   * long-polling requests from the @e po_head.
+   */
+  struct GNUNET_DB_EventHandler *po_eh;
+
   /**
    * Merchant's private key.
    */
@@ -152,13 +163,23 @@ struct TMH_MerchantInstance
    * True if this instance was deleted (but not yet purged).
    */
   bool deleted;
+
+  /**
+   * The authentication settings for this instance
+   * were changed via the command-line. Do not check
+   * against the DB value when updating the auth token.
+   */
+  bool auth_override;
 };
 
 
+GNUNET_NETWORK_STRUCT_BEGIN
+
+
 /**
  * Event triggered when an order is paid.
  */
-struct TMH_OrderPayEvent
+struct TMH_OrderPayEventP
 {
   /**
    * Type is #TALER_DBEVENT_MERCHANT_ORDER_PAID
@@ -168,7 +189,12 @@ struct TMH_OrderPayEvent
   /**
    * Always zero (for alignment).
    */
-  uint32_t reserved;
+  uint32_t reserved GNUNET_PACKED;
+
+  /**
+   * Merchant's public key
+   */
+  struct TALER_MerchantPublicKeyP merchant_pub;
 
   /**
    * Hash of the order ID.
@@ -178,19 +204,61 @@ struct TMH_OrderPayEvent
 
 
 /**
- * Event triggered when an order's refund is increased.
+ * Event triggered when a fulfillment URL is
+ * bound to a session (as paid).
  */
-struct TMH_OrderRefundEvent
+struct TMH_SessionEventP
 {
   /**
-   * Type is #TALER_DBEVENT_MERCHANT_ORDER_REFUND
+   * Type is #TALER_DBEVENT_MERCHANT_SESSION_CAPTURED
    */
   struct GNUNET_DB_EventHeaderP header;
 
   /**
    * Always zero (for alignment).
    */
-  uint32_t reserved;
+  uint32_t reserved GNUNET_PACKED;
+
+  /**
+   * Merchant's public key
+   */
+  struct TALER_MerchantPublicKeyP merchant_pub;
+
+  /**
+   * Hash of the fulfillment URL.
+   */
+  struct GNUNET_HashCode h_fulfillment_url;
+
+  /**
+   * Hash of the session ID
+   */
+  struct GNUNET_HashCode h_session_id;
+};
+
+
+/**
+ * Event triggered when an order's refund is increased
+ * or obtained by the respective wallet.
+ *
+ * Extra arguments are the amount (as a string).
+ */
+struct TMH_OrderRefundEventP
+{
+  /**
+   * Type is #TALER_DBEVENT_MERCHANT_ORDER_REFUND or
+   * #TALER_DBEVENT_MERCHANT_REFUND_OBTAINED
+   */
+  struct GNUNET_DB_EventHeaderP header;
+
+  /**
+   * Always zero (for alignment).
+   */
+  uint32_t reserved GNUNET_PACKED;
+
+  /**
+   * Merchant's public key
+   */
+  struct TALER_MerchantPublicKeyP merchant_pub;
 
   /**
    * Hash of the order ID.
@@ -199,6 +267,87 @@ struct TMH_OrderRefundEvent
 };
 
 
+/**
+ * Possible flags indicating the state of an order.
+ */
+enum TMH_OrderStateFlags
+{
+  TMH_OSF_NONE = 0,
+
+  /**
+   * Not yet used.
+   */
+  TMH_OSF_CLAIMED = 1,
+
+  /**
+   * Customer paid the order.
+   */
+  TMH_OSF_PAID = 2,
+
+  /**
+   * Merchant granted (possibly partial) refund.
+   */
+  TMH_OSF_REFUNDED = 4,
+
+  /**
+   * Merchant received the payment from the exchange.
+   * FIXME: not triggered yet!
+   */
+  TMH_OSF_WIRED = 8
+};
+
+
+/**
+ * Extra information passed for a
+ * #TALER_DBEVENT_MERCHANT_ORDERS_CHANGE.
+ */
+struct TMH_OrderChangeEventDetailsP
+{
+  /**
+   * Order ID, in NBO.
+   */
+  uint64_t order_serial_id GNUNET_PACKED;
+
+  /**
+   * Execution date of the order.
+   */
+  struct GNUNET_TIME_TimestampNBO execution_date;
+
+  /**
+   * See `enum TMH_OrderStateFlags`. In NBO.
+   */
+  uint32_t order_state GNUNET_PACKED;
+
+};
+
+
+/**
+ * Event triggered when an order's refund is increased
+ * or obtained by the respective wallet.
+ *
+ * Extra arguments are the amount (as a string).
+ */
+struct TMH_OrderChangeEventP
+{
+  /**
+   * Type is #TALER_DBEVENT_MERCHANT_ORDERS_CHANGE.
+   */
+  struct GNUNET_DB_EventHeaderP header;
+
+  /**
+   * Always zero (for alignment).
+   */
+  uint32_t reserved GNUNET_PACKED;
+
+  /**
+   * Merchant's public key
+   */
+  struct TALER_MerchantPublicKeyP merchant_pub;
+};
+
+
+GNUNET_NETWORK_STRUCT_END
+
 /**
  * @brief Struct describing an URL and the handler for it.
  *
@@ -405,7 +554,7 @@ struct TMH_HandlerContext
 
 
 /**
- * Entry in a #resume_timeout_heap.
+ * Information common for suspended requests.
  */
 struct TMH_SuspendedConnection
 {
@@ -414,25 +563,6 @@ struct TMH_SuspendedConnection
    */
   struct MHD_Connection *con;
 
-  /**
-   * Associated heap node.  Used internally by #TMH_long_poll_suspend()
-   * and TMH_long_poll_resume().
-   */
-  struct GNUNET_CONTAINER_HeapNode *hn;
-
-  /**
-   * Key of this entry in the #payment_trigger_map.  Used internally by
-   * #TMH_long_poll_suspend() and TMH_long_poll_resume().
-   */
-  struct GNUNET_HashCode key;
-
-  /**
-   * Optional session/fulfillment URI-based key
-   * of this entry in the #payment_trigger_map.  Used internally by
-   * TMH_long_poll_resume2().
-   */
-  struct GNUNET_HashCode key2;
-
   /**
    * At what time does this request expire? If set in the future, we
    * may wait this long for a payment to arrive before responding.
@@ -454,10 +584,6 @@ struct TMH_SuspendedConnection
    */
   bool awaiting_refund_obtained;
 
-  /**
-   * True if @a key2 is set.
-   */
-  bool has_key2;
 };
 
 
@@ -496,63 +622,16 @@ extern struct GNUNET_TIME_Relative TMH_legal_expiration;
 extern char *TMH_default_auth;
 
 
-/**
- * Suspend connection from @a sc until payment has been received.
- *
- * @param order_id the order that we are waiting on
- * @param session_id session ID of the requester
- * @param fulfillment_url fulfillment URL of the contract
- * @param mi the merchant instance we are waiting on
- * @param sc connection to suspend
- * @param min_refund refund amount we are waiting on to be exceeded before resuming,
- *                   NULL if we are not waiting for refunds
- */
-void
-TMH_long_poll_suspend (const char *order_id,
-                       const char *session_id,
-                       const char *fulfillment_url,
-                       const struct TMH_MerchantInstance *mi,
-                       struct TMH_SuspendedConnection *sc,
-                       const struct TALER_Amount *min_refund);
-
-
-/**
- * Find out if we have any clients long-polling for @a order_id to be
- * confirmed at merchant @a mpub, and if so, tell them to resume.
- *
- * @param order_id the order that was paid or refunded
- * @param mi the merchant instance where the payment or refund happened
- * @param refund_amount refunded amount, if the trigger was a refund, otherwise NULL
- * @param obtained if true, the wallet has obtained the refunds for the order
- */
-void
-TMH_long_poll_resume (const char *order_id,
-                      const struct TMH_MerchantInstance *mi,
-                      const struct TALER_Amount *refund_amount,
-                      bool obtained);
-
-
-/**
- * Find out if we have any clients long-polling for @a order_id to be
- * confirmed at merchant @a mpub, and if so, tell them to resume.
- *
- * @param session_id the session for which @a fulfillment_url became paid
- * @param fulfillment_url fullfillment URL of which an order was paid
- */
-void
-TMH_long_poll_resume2 (const char *session_id,
-                       const char *fulfillment_url);
-
-
 /**
  * Callback that frees an instances removing
  * it from the global hashmap.
  *
- * @param cls closure, NULL
- * @param key current key
+ * @param cls closure, pass NULL
+ * @param key current key (ignored)
  * @param value a `struct TMH_MerchantInstance`
+ * @return #GNUNET_YES (always)
  */
-int
+enum GNUNET_GenericReturnValue
 TMH_instance_free_cb (void *cls,
                       const struct GNUNET_HashCode *key,
                       void *value);
@@ -564,10 +643,19 @@ TMH_instance_free_cb (void *cls,
  * @param[in,out] mi merchant instance details to define
  * @return #GNUNET_OK on success, #GNUNET_NO if the same ID is in use already
  */
-int
+enum GNUNET_GenericReturnValue
 TMH_add_instance (struct TMH_MerchantInstance *mi);
 
 
+/**
+ * Decrement reference counter of @a mi, and free if it hits zero.
+ *
+ * @param[in,out] mi merchant instance to update and possibly free
+ */
+void
+TMH_instance_decref (struct TMH_MerchantInstance *mi);
+
+
 /**
  * Lookup a merchant instance by its instance ID.
  *
@@ -598,10 +686,10 @@ TMH_reload_instances (const char *id);
  * @param hash the hash to check against
  * @return #GNUNET_OK if the @a token matches
  */
-int
+enum GNUNET_GenericReturnValue
 TMH_check_auth (const char *token,
-                const struct GNUNET_ShortHashCode *salt,
-                const struct GNUNET_HashCode *hash);
+                struct TALER_MerchantAuthenticationSaltP *salt,
+                struct TALER_MerchantAuthenticationHashP *hash);
 
 
 /**
@@ -614,8 +702,8 @@ TMH_check_auth (const char *token,
  */
 void
 TMH_compute_auth (const char *token,
-                  struct GNUNET_ShortHashCode *salt,
-                  struct GNUNET_HashCode *hash);
+                  struct TALER_MerchantAuthenticationSaltP *salt,
+                  struct TALER_MerchantAuthenticationHashP *hash);
 
 
 #endif
