@@ -366,11 +366,15 @@ process_wire_fees (struct Exchange *exchange,
     endp = endp->next;
   while ( (NULL != endp) &&
           (NULL != fees) &&
-          (fees->start_date.abs_value_us < endp->end_date.abs_value_us) )
+          (GNUNET_TIME_timestamp_cmp (fees->start_date,
+                                      <,
+                                      endp->end_date)) )
     fees = fees->next;
   if ( (NULL != endp) &&
        (NULL != fees) &&
-       (fees->start_date.abs_value_us != endp->end_date.abs_value_us) )
+       (GNUNET_TIME_timestamp_cmp (fees->start_date,
+                                   !=,
+                                   endp->end_date)) )
   {
     /* Hole in the fee structure, not allowed! */
     GNUNET_break_op (0);
@@ -390,8 +394,8 @@ process_wire_fees (struct Exchange *exchange,
                 "Storing wire fee for `%s' and method `%s' at %s in DB; the fee is %s\n",
                 TALER_B2S (master_pub),
                 wire_method,
-                GNUNET_STRINGS_absolute_time_to_string (af->start_date),
-                TALER_amount2s (&af->wire_fee));
+                GNUNET_TIME_timestamp2s (af->start_date),
+                TALER_amount2s (&af->fees.wire));
     TMH_db->preflight (TMH_db->cls);
     if (GNUNET_OK !=
         TMH_db->start (TMH_db->cls,
@@ -406,8 +410,7 @@ process_wire_fees (struct Exchange *exchange,
     qs = TMH_db->store_wire_fee_by_exchange (TMH_db->cls,
                                              master_pub,
                                              &h_wire_method,
-                                             &af->wire_fee,
-                                             &af->closing_fee,
+                                             &af->fees,
                                              af->start_date,
                                              af->end_date,
                                              &af->master_sig);
@@ -502,9 +505,9 @@ process_wire_accounts (struct Exchange *exchange,
  * @param wire_method the wire method we want the fees for
  * @return NULL if we do not have fees for this method yet
  */
-static struct FeesByWireMethod *
+static const struct FeesByWireMethod *
 get_wire_fees (struct Exchange *exchange,
-               struct GNUNET_TIME_Absolute now,
+               struct GNUNET_TIME_Timestamp now,
                const char *wire_method)
 {
   for (struct FeesByWireMethod *fbw = exchange->wire_fees_head;
@@ -518,7 +521,9 @@ get_wire_fees (struct Exchange *exchange,
 
       /* Advance through list up to current time */
       while ( (NULL != (af = fbw->af)) &&
-              (now.abs_value_us >= af->end_date.abs_value_us) )
+              (GNUNET_TIME_timestamp_cmp (now,
+                                          >=,
+                                          af->end_date)) )
       {
         fbw->af = af->next;
         GNUNET_free (af);
@@ -545,16 +550,16 @@ static bool
 process_find_operations (struct Exchange *exchange)
 {
   struct TMH_EXCHANGES_FindOperation *fn;
-  struct GNUNET_TIME_Absolute now;
+  struct GNUNET_TIME_Timestamp now;
   bool need_wire;
 
-  now = GNUNET_TIME_absolute_get ();
+  now = GNUNET_TIME_timestamp_get ();
   need_wire = false;
   for (struct TMH_EXCHANGES_FindOperation *fo = exchange->fo_head;
        NULL != fo;
        fo = fn)
   {
-    struct FeesByWireMethod *fbw;
+    const struct FeesByWireMethod *fbw;
 
     fn = fo->next;
     if (NULL != fo->wire_method)
@@ -566,6 +571,10 @@ process_find_operations (struct Exchange *exchange)
       if (NULL == fbw)
       {
         need_wire = true;
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Missing wire fees for exchange %s and method %s\n",
+                    exchange->url,
+                    fo->wire_method);
         /* Do not warn if this is before our first attempt */
         if (! GNUNET_TIME_relative_is_zero (exchange->wire_retry_delay))
           GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
@@ -574,14 +583,26 @@ process_find_operations (struct Exchange *exchange)
         fbw = NULL;
         continue;
       }
-      if (fbw->af->start_date.abs_value_us > now.abs_value_us)
+      if (NULL == fbw->af)
       {
         /* Disagreement on the current time */
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                    "Exchange's earliest fee is %s adhead of our time. Clock skew issue?\n",
-                    GNUNET_STRINGS_relative_time_to_string (
-                      GNUNET_TIME_absolute_get_remaining (fbw->af->start_date),
-                      GNUNET_YES));
+                    "Exchange has no wire fees configured for `%s' wire method (will retry later)\n",
+                    fo->wire_method);
+        fbw = NULL;
+        continue;
+      }
+      if (GNUNET_TIME_timestamp_cmp (fbw->af->start_date,
+                                     >,
+                                     now))
+      {
+        /* Disagreement on the current time */
+        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                    "Exchange's earliest fee is %s ahead of our time. Clock skew issue?\n",
+                    GNUNET_TIME_relative2s (
+                      GNUNET_TIME_absolute_get_remaining (
+                        fbw->af->start_date.abs_time),
+                      true));
         fbw = NULL;
         continue;
       }
@@ -600,7 +621,7 @@ process_find_operations (struct Exchange *exchange)
               &hr,
               exchange->conn,
               (NULL != fbw) ? fbw->payto_uri : NULL,
-              (NULL != fbw) ? &fbw->af->wire_fee : NULL,
+              (NULL != fbw) ? &fbw->af->fees.wire : NULL,
               exchange->trusted);
     }
     TMH_EXCHANGES_find_exchange_cancel (fo);
@@ -639,6 +660,8 @@ handle_wire_data (void *cls,
   enum TALER_ErrorCode ecx;
 
   exchange->wire_request = NULL;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Received /wire response\n");
   if (MHD_HTTP_OK != hr->http_status)
   {
     struct TMH_EXCHANGES_FindOperation *fo;
@@ -735,6 +758,8 @@ wire_task_cb (void *cls)
   if (! process_find_operations (exchange))
     return; /* no more need */
   GNUNET_assert (NULL == exchange->wire_request);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Initiating /wire download\n");
   exchange->wire_request = TALER_EXCHANGE_wire (exchange->conn,
                                                 &handle_wire_data,
                                                 exchange);
@@ -860,7 +885,8 @@ fail_and_retry (struct Exchange *exchange,
               hr->http_status,
               GNUNET_STRINGS_relative_time_to_string (exchange->retry_delay,
                                                       GNUNET_YES));
-  GNUNET_assert (NULL == exchange->retry_task);
+  if (NULL != exchange->retry_task)
+    GNUNET_SCHEDULER_cancel (exchange->retry_task);
   exchange->first_retry = GNUNET_TIME_relative_to_absolute (
     exchange->retry_delay);
   exchange->retry_task = GNUNET_SCHEDULER_add_delayed (exchange->retry_delay,
@@ -892,7 +918,7 @@ keys_mgmt_cb (void *cls,
               enum TALER_EXCHANGE_VersionCompatibility compat)
 {
   struct Exchange *exchange = cls;
-  struct GNUNET_TIME_Absolute expire;
+  struct GNUNET_TIME_Timestamp expire;
   struct GNUNET_TIME_Relative delay;
 
   if ( (MHD_HTTP_OK != hr->http_status) ||
@@ -969,13 +995,13 @@ keys_mgmt_cb (void *cls,
     }
   }
 
+  exchange->first_retry = GNUNET_TIME_relative_to_absolute (RELOAD_DELAY);
   expire = TALER_EXCHANGE_check_keys_current (exchange->conn,
                                               TALER_EXCHANGE_CKF_NONE);
-  exchange->first_retry = GNUNET_TIME_relative_to_absolute (RELOAD_DELAY);
-  if (0 == expire.abs_value_us)
+  if (0 == GNUNET_TIME_absolute_is_zero (expire.abs_time))
     delay = RELOAD_DELAY;
   else
-    delay = GNUNET_TIME_absolute_get_remaining (expire);
+    delay = GNUNET_TIME_absolute_get_remaining (expire.abs_time);
   if (GNUNET_TIME_relative_is_zero (delay))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -1039,7 +1065,7 @@ TMH_EXCHANGES_find_exchange (const char *chosen_exchange,
 {
   struct Exchange *exchange;
   struct TMH_EXCHANGES_FindOperation *fo;
-  struct GNUNET_TIME_Absolute now;
+  struct GNUNET_TIME_Timestamp now;
 
   if (NULL == merchant_curl_ctx)
   {
@@ -1100,13 +1126,17 @@ TMH_EXCHANGES_find_exchange (const char *chosen_exchange,
       = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_relative_max (
                                             exchange->retry_delay,
                                             FORCED_RELOAD_DELAY));
-    TALER_EXCHANGE_check_keys_current (exchange->conn,
-                                       TALER_EXCHANGE_CKF_FORCE_DOWNLOAD);
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "/keys retry forced, waiting until %s\n",
+                GNUNET_TIME_absolute2s (exchange->first_retry));
+    /* NOTE: return value tells us how long /keys should still
+       be valid. */
+    (void) TALER_EXCHANGE_check_keys_current (exchange->conn,
+                                              TALER_EXCHANGE_CKF_FORCE_DOWNLOAD);
     return fo;
   }
 
-  now = GNUNET_TIME_absolute_get ();
-  (void) GNUNET_TIME_round_abs (&now);
+  now = GNUNET_TIME_timestamp_get ();
   if ( (! exchange->pending) &&
        ( (NULL == fo->wire_method) ||
          (NULL != get_wire_fees (exchange,
@@ -1115,6 +1145,9 @@ TMH_EXCHANGES_find_exchange (const char *chosen_exchange,
   {
     /* We are not currently waiting for a reply, immediately
        return result */
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "The exchange `%s' is ready\n",
+                chosen_exchange);
     GNUNET_assert (NULL == fo->at);
     fo->at = GNUNET_SCHEDULER_add_now (&return_result,
                                        fo);
@@ -1127,7 +1160,8 @@ TMH_EXCHANGES_find_exchange (const char *chosen_exchange,
        (exchange->pending) )
   {
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Do not have current key data. Will request /keys now\n");
+                "Do not have current /keys data for `%s'. Will request /keys now\n",
+                chosen_exchange);
     exchange->retry_task = GNUNET_SCHEDULER_add_now (&retry_exchange,
                                                      exchange);
   }
@@ -1135,7 +1169,7 @@ TMH_EXCHANGES_find_exchange (const char *chosen_exchange,
             (NULL == exchange->wire_task) &&
             (NULL == exchange->wire_request) )
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Do not have required wire data. Will re-request /wire now\n");
     exchange->wire_task = GNUNET_SCHEDULER_add_now (&wire_task_cb,
                                                     exchange);
@@ -1258,7 +1292,7 @@ accept_exchanges (void *cls,
 }
 
 
-int
+enum GNUNET_GenericReturnValue
 TMH_EXCHANGES_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
 {
   merchant_curl_ctx
@@ -1269,9 +1303,9 @@ TMH_EXCHANGES_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
+  merchant_curl_rc = GNUNET_CURL_gnunet_rc_create (merchant_curl_ctx);
   GNUNET_CURL_enable_async_scope_header (merchant_curl_ctx,
                                          "Taler-Correlation-Id");
-  merchant_curl_rc = GNUNET_CURL_gnunet_rc_create (merchant_curl_ctx);
   /* get exchanges from the merchant configuration and try to connect to them */
   GNUNET_CONFIGURATION_iterate_sections (cfg,
                                          &accept_exchanges,
