@@ -43,7 +43,7 @@ extern json_t *TMH_trusted_exchanges;
  * @return number of trusted exchanges on success; #GNUNET_SYSERR upon error in
  *          parsing.
  */
-int
+enum GNUNET_GenericReturnValue
 TMH_EXCHANGES_init (const struct GNUNET_CONFIGURATION_Handle *cfg);
 
 
