@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2021 Taler Systems SA
+  (C) 2014-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -25,12 +25,12 @@
 #include <taler/taler_signatures.h>
 #include <taler/taler_dbevents.h>
 #include <taler/taler_json_lib.h>
+#include <taler/taler_templating_lib.h>
 #include <taler/taler_exchange_service.h>
 #include "taler-merchant-httpd_exchanges.h"
 #include "taler-merchant-httpd_get-orders-ID.h"
 #include "taler-merchant-httpd_mhd.h"
 #include "taler-merchant-httpd_qr.h"
-#include "taler-merchant-httpd_templating.h"
 
 /**
  * How often do we retry DB transactions on serialization failures?
@@ -47,7 +47,7 @@ struct GetOrderData
   /**
    * Hashed version of contract terms. All zeros if not provided.
    */
-  struct GNUNET_HashCode h_contract_terms;
+  struct TALER_PrivateContractHashP h_contract_terms;
 
   /**
    * Claim token used for access control. All zeros if not provided.
@@ -78,7 +78,12 @@ struct GetOrderData
   /**
    * Database event we are waiting on to be resuming.
    */
-  struct GNUNET_DB_EventHandler *eh;
+  struct GNUNET_DB_EventHandler *pay_eh;
+
+  /**
+   * Database event we are waiting on to be resuming.
+   */
+  struct GNUNET_DB_EventHandler *refund_eh;
 
   /**
    * Which merchant instance is this for?
@@ -119,6 +124,12 @@ struct GetOrderData
    */
   struct TALER_Amount refund_amount;
 
+  /**
+   * Total refunds already collected.
+   * if @e refunded is set to true.
+   */
+  struct TALER_Amount refund_taken;
+
   /**
    * Return code: #TALER_EC_NONE if successful.
    */
@@ -126,9 +137,12 @@ struct GetOrderData
 
   /**
    * Did we suspend @a connection and are thus in
-   * the #god_head DLL?
+   * the #god_head DLL (#GNUNET_YES). Set to
+   * #GNUNET_NO if we are not suspended, and to
+   * #GNUNET_SYSERR if we should close the connection
+   * without a response due to shutdown.
    */
-  bool suspended;
+  enum GNUNET_GenericReturnValue suspended;
 
   /**
    * Set to true if we are dealing with a claimed order
@@ -146,8 +160,9 @@ struct GetOrderData
   /**
    * Set to true if a refund is still available for the
    * wallet for this payment.
+   * @deprecated: true if refund_taken < refund_amount
    */
-  bool refund_available;
+  bool refund_pending;
 
   /**
    * Set to true if the client requested HTML, otherwise we generate JSON.
@@ -179,7 +194,7 @@ TMH_force_wallet_get_order_resume (void)
                                  god_tail,
                                  god);
     GNUNET_assert (god->suspended);
-    god->suspended = false;
+    god->suspended = GNUNET_SYSERR;
     MHD_resume_connection (god->sc.con);
     TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
   }
@@ -200,17 +215,75 @@ resume_by_event (void *cls,
                  size_t extra_size)
 {
   struct GetOrderData *god = cls;
+  struct GNUNET_AsyncScopeSave old;
 
-  (void) extra;
-  (void) extra_size;
+  GNUNET_async_scope_enter (&god->hc->async_scope_id,
+                            &old);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Received event for %s with argument `%.*s`\n",
+              god->order_id,
+              (int) extra_size,
+              (const char *) extra);
   if (! god->suspended)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Not suspended, ignoring event\n");
+    GNUNET_async_scope_restore (&old);
     return; /* duplicate event is possible */
-  god->suspended = false;
-  GNUNET_CONTAINER_DLL_insert (god_head,
+  }
+  if (GNUNET_TIME_absolute_is_future (god->sc.long_poll_timeout) &&
+      god->sc.awaiting_refund)
+  {
+    char *as;
+    struct TALER_Amount a;
+
+    if (0 == extra_size)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                  "No amount given, but need refund above threshold\n");
+      GNUNET_async_scope_restore (&old);
+      return; /* not relevant */
+    }
+    as = GNUNET_strndup (extra,
+                         extra_size);
+    if (GNUNET_OK !=
+        TALER_string_to_amount (as,
+                                &a))
+    {
+      GNUNET_break (0);
+      GNUNET_async_scope_restore (&old);
+      return;
+    }
+    if (GNUNET_OK !=
+        TALER_amount_cmp_currency (&god->sc.refund_expected,
+                                   &a))
+    {
+      GNUNET_break (0);
+      GNUNET_async_scope_restore (&old);
+      return; /* bad currency!? */
+    }
+    if (1 == TALER_amount_cmp (&god->sc.refund_expected,
+                               &a))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                  "Amount too small to trigger resuming\n");
+      GNUNET_async_scope_restore (&old);
+      return; /* refund too small */
+    }
+  }
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Resuming (%d/%d) by event with argument `%.*s`\n",
+              (int) GNUNET_TIME_absolute_is_future (god->sc.long_poll_timeout),
+              god->sc.awaiting_refund,
+              (int) extra_size,
+              (const char *) extra);
+  god->suspended = GNUNET_NO;
+  GNUNET_CONTAINER_DLL_remove (god_head,
                                god_tail,
                                god);
   MHD_resume_connection (god->sc.con);
   TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
+  GNUNET_async_scope_restore (&old);
 }
 
 
@@ -232,18 +305,11 @@ suspend_god (struct GetOrderData *god)
     god->contract_terms = NULL;
   }
   GNUNET_assert (! god->suspended);
-  god->suspended = true;
+  god->suspended = GNUNET_YES;
   GNUNET_CONTAINER_DLL_insert (god_head,
                                god_tail,
                                god);
-  TMH_long_poll_suspend (god->order_id,
-                         god->session_id,
-                         god->fulfillment_url,
-                         god->hc->instance,
-                         &god->sc,
-                         god->sc.awaiting_refund
-                         ? &god->sc.refund_expected
-                         : NULL);
+  MHD_suspend_connection (god->sc.con);
 }
 
 
@@ -307,7 +373,7 @@ TMH_make_order_status_url (struct MHD_Connection *con,
                            const char *session_id,
                            const char *instance_id,
                            struct TALER_ClaimTokenP *claim_token,
-                           struct GNUNET_HashCode *h_contract)
+                           struct TALER_PrivateContractHashP *h_contract)
 {
   const char *host;
   const char *forwarded_host;
@@ -371,7 +437,7 @@ TMH_make_order_status_url (struct MHD_Connection *con,
                              "?token=");
     GNUNET_buffer_write_data_encoded (&buf,
                                       (char *) claim_token,
-                                      sizeof (struct TALER_ClaimTokenP));
+                                      sizeof (*claim_token));
     num_qp++;
   }
 
@@ -398,7 +464,7 @@ TMH_make_order_status_url (struct MHD_Connection *con,
                                "?h_contract=");
     GNUNET_buffer_write_data_encoded (&buf,
                                       (char *) h_contract,
-                                      sizeof (struct GNUNET_HashCode));
+                                      sizeof (*h_contract));
   }
 
   return GNUNET_buffer_reap_str (&buf);
@@ -536,7 +602,7 @@ send_pay_request (struct GetOrderData *god,
        (NULL == already_paid_order_id) )
   {
     /* long polling: do not queue a response, suspend connection instead */
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Suspending request: long polling for payment\n");
     suspend_god (god);
     return MHD_YES;
@@ -625,12 +691,12 @@ send_pay_request (struct GetOrderData *god,
                                    qr),
           GNUNET_JSON_pack_string ("order_summary",
                                    get_order_summary (god)));
-        res = TMH_return_from_template (god->sc.con,
-                                        MHD_HTTP_PAYMENT_REQUIRED,
-                                        "request_payment",
-                                        god->hc->instance->settings.id,
-                                        taler_pay_uri,
-                                        context);
+        res = TALER_TEMPLATING_reply (god->sc.con,
+                                      MHD_HTTP_PAYMENT_REQUIRED,
+                                      "request_payment",
+                                      god->hc->instance->settings.id,
+                                      taler_pay_uri,
+                                      context);
         if (GNUNET_SYSERR == res)
         {
           GNUNET_break (0);
@@ -682,7 +748,7 @@ send_pay_request (struct GetOrderData *god,
 static void
 process_refunds_cb (void *cls,
                     uint64_t refund_serial,
-                    struct GNUNET_TIME_Absolute timestamp,
+                    struct GNUNET_TIME_Timestamp timestamp,
                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
                     const char *exchange_url,
                     uint64_t rtransaction_id,
@@ -692,21 +758,27 @@ process_refunds_cb (void *cls,
 {
   struct GetOrderData *god = cls;
 
+  (void) refund_serial;
+  (void) timestamp;
+  (void) exchange_url;
+  (void) rtransaction_id;
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Found refund of %s for coin %s with reason `%s' in database\n",
               TALER_amount2s (refund_amount),
               TALER_B2S (coin_pub),
               reason);
-  god->refund_available |= pending;
-  if (god->refunded)
+  god->refund_pending |= pending;
+  if (! pending)
   {
     GNUNET_assert (0 <=
-                   TALER_amount_add (&god->refund_amount,
-                                     &god->refund_amount,
+                   TALER_amount_add (&god->refund_taken,
+                                     &god->refund_taken,
                                      refund_amount));
-    return;
   }
-  god->refund_amount = *refund_amount;
+  GNUNET_assert (0 <=
+                 TALER_amount_add (&god->refund_amount,
+                                   &god->refund_amount,
+                                   refund_amount));
   god->refunded = true;
 }
 
@@ -726,10 +798,15 @@ god_cleanup (void *cls)
     json_decref (god->contract_terms);
     god->contract_terms = NULL;
   }
-  if (NULL != god->eh)
+  if (NULL != god->refund_eh)
   {
-    TMH_db->event_listen_cancel (god->eh);
-    god->eh = NULL;
+    TMH_db->event_listen_cancel (god->refund_eh);
+    god->refund_eh = NULL;
+  }
+  if (NULL != god->pay_eh)
+  {
+    TMH_db->event_listen_cancel (god->pay_eh);
+    god->pay_eh = NULL;
   }
   GNUNET_free (god);
 }
@@ -750,6 +827,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   bool contract_available = false;
   const char *merchant_base_url;
 
+  (void) rh;
   if (NULL == god)
   {
     god = GNUNET_new (struct GetOrderData);
@@ -771,7 +849,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
       if ( (NULL != cts) &&
            (GNUNET_OK !=
             GNUNET_CRYPTO_hash_from_string (cts,
-                                            &god->h_contract_terms)) )
+                                            &god->h_contract_terms.hash)) )
       {
         /* cts has wrong encoding */
         GNUNET_break_op (0);
@@ -824,6 +902,9 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
         ? 0 == strcasecmp (await_refund_obtained_s,
                            "yes")
         : false;
+      if (god->sc.awaiting_refund_obtained)
+        GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                    "Awaiting refund obtained\n");
     }
 
     {
@@ -847,6 +928,9 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                              "refund");
         }
         god->sc.awaiting_refund = true;
+        GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                    "Awaiting minimum refund of %s\n",
+                    min_refund);
       }
     }
 
@@ -886,37 +970,48 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
             = GNUNET_TIME_relative_to_absolute (timeout);
           if (! GNUNET_TIME_relative_is_zero (timeout))
           {
-            if (god->sc.awaiting_refund)
+            if (god->sc.awaiting_refund ||
+                god->sc.awaiting_refund_obtained)
             {
-              struct TMH_OrderPayEvent refund_eh = {
+              struct TMH_OrderPayEventP refund_eh = {
                 .header.size = htons (sizeof (refund_eh)),
-                .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_REFUND)
+                .header.type = htons (god->sc.awaiting_refund_obtained
+                                      ? TALER_DBEVENT_MERCHANT_REFUND_OBTAINED
+                                      : TALER_DBEVENT_MERCHANT_ORDER_REFUND),
+                .merchant_pub = hc->instance->merchant_pub
               };
 
               GNUNET_CRYPTO_hash (god->order_id,
                                   strlen (god->order_id),
                                   &refund_eh.h_order_id);
-              god->eh = TMH_db->event_listen (TMH_db->cls,
-                                              &refund_eh.header,
-                                              timeout,
-                                              &resume_by_event,
-                                              god);
+              GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                          "Subscribing %p to refunds on %s\n",
+                          god,
+                          god->order_id);
+              god->refund_eh = TMH_db->event_listen (TMH_db->cls,
+                                                     &refund_eh.header,
+                                                     timeout,
+                                                     &resume_by_event,
+                                                     god);
             }
-            else /* ! waiting for refund */
             {
-              struct TMH_OrderPayEvent pay_eh = {
+              struct TMH_OrderPayEventP pay_eh = {
                 .header.size = htons (sizeof (pay_eh)),
-                .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID)
+                .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID),
+                .merchant_pub = hc->instance->merchant_pub
               };
 
+              GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                          "Subscribing to payments on %s\n",
+                          god->order_id);
               GNUNET_CRYPTO_hash (god->order_id,
                                   strlen (god->order_id),
                                   &pay_eh.h_order_id);
-              god->eh = TMH_db->event_listen (TMH_db->cls,
-                                              &pay_eh.header,
-                                              timeout,
-                                              &resume_by_event,
-                                              god);
+              god->pay_eh = TMH_db->event_listen (TMH_db->cls,
+                                                  &pay_eh.header,
+                                                  timeout,
+                                                  &resume_by_event,
+                                                  god);
             }
           } /* end of timeout non-zero */
         } /* end of HTML generation NOT requested */
@@ -925,9 +1020,11 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
 
   } /* end of first-time initialization / sanity checks */
 
-  if (god->suspended)
+  if (GNUNET_SYSERR == god->suspended)
+    return MHD_NO; /* we are in shutdown */
+  if (GNUNET_YES == god->suspended)
   {
-    god->suspended = false;
+    god->suspended = GNUNET_NO;
     GNUNET_CONTAINER_DLL_remove (god_head,
                                  god_tail,
                                  god);
@@ -938,6 +1035,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   if (NULL == god->contract_terms)
   {
     uint64_t order_serial;
+    bool paid = false;
     struct TALER_ClaimTokenP db_claim_token;
 
     qs = TMH_db->lookup_contract_terms (TMH_db->cls,
@@ -945,6 +1043,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                         order_id,
                                         &god->contract_terms,
                                         &order_serial,
+                                        &paid,
                                         &db_claim_token);
     if (0 > qs)
     {
@@ -969,33 +1068,53 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   /* Check if client provided the right hash code of the contract terms */
   if (NULL != god->contract_terms)
   {
-    struct GNUNET_HashCode h;
-
     contract_available = true;
-    if (GNUNET_OK !=
-        TALER_JSON_contract_hash (god->contract_terms,
-                                  &h))
+
+    if (GNUNET_YES == GNUNET_is_zero (&god->h_contract_terms))
     {
-      GNUNET_break (0);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                         TALER_EC_GENERIC_FAILED_COMPUTE_JSON_HASH,
-                                         "contract terms");
+
+      if (GNUNET_OK !=
+          TALER_JSON_contract_hash (god->contract_terms,
+                                    &god->h_contract_terms))
+      {
+        GNUNET_break (0);
+        return TALER_MHD_reply_with_error (connection,
+                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                           TALER_EC_GENERIC_FAILED_COMPUTE_JSON_HASH,
+                                           "contract terms");
+      }
+
     }
-    contract_match = (0 ==
-                      GNUNET_memcmp (&h,
-                                     &god->h_contract_terms));
-    if ( (GNUNET_NO ==
-          GNUNET_is_zero (&god->h_contract_terms)) &&
-         (! contract_match) )
+    else
     {
-      GNUNET_break_op (0);
-      return TALER_MHD_reply_with_error (
-        connection,
-        MHD_HTTP_FORBIDDEN,
-        TALER_EC_MERCHANT_GENERIC_CONTRACT_HASH_DOES_NOT_MATCH_ORDER,
-        NULL);
+
+      struct TALER_PrivateContractHashP h;
+
+      if (GNUNET_OK !=
+          TALER_JSON_contract_hash (god->contract_terms,
+                                    &h))
+      {
+        GNUNET_break (0);
+        return TALER_MHD_reply_with_error (connection,
+                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                           TALER_EC_GENERIC_FAILED_COMPUTE_JSON_HASH,
+                                           "contract terms");
+      }
+      contract_match = (0 ==
+                        GNUNET_memcmp (&h,
+                                       &god->h_contract_terms));
+      if (! contract_match)
+      {
+        GNUNET_break_op (0);
+        return TALER_MHD_reply_with_error (
+          connection,
+          MHD_HTTP_FORBIDDEN,
+          TALER_EC_MERCHANT_GENERIC_CONTRACT_HASH_DOES_NOT_MATCH_ORDER,
+          NULL);
+      }
+
     }
+
   }
 
   if (contract_available)
@@ -1005,7 +1124,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   else
   {
     struct TALER_ClaimTokenP db_claim_token;
-    struct GNUNET_HashCode unused;
+    struct TALER_MerchantPostDataHashP unused;
 
     qs = TMH_db->lookup_order (TMH_db->cls,
                                hc->instance->settings.id,
@@ -1081,11 +1200,10 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   {
     /* Authentication provided but wrong. */
     GNUNET_break_op (0);
-    /* FIXME: use better error code */
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_FORBIDDEN,
-                                       TALER_EC_MERCHANT_GET_ORDERS_ID_INVALID_TOKEN,
-                                       "authentication with h_contract provided but wrong");
+                                       TALER_EC_MERCHANT_GET_ORDERS_ID_INVALID_CONTRACT_HASH,
+                                       NULL);
   }
 
   if (! (token_match ||
@@ -1156,7 +1274,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                public_reorder_url));
   }
 
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Claim token or contract matched\n");
 
   if ( (NULL != god->session_id) &&
@@ -1225,7 +1343,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
 
   {
     /* Check if paid. */
-    struct GNUNET_HashCode h_contract;
+    struct TALER_PrivateContractHashP h_contract;
     bool paid;
 
     qs = TMH_db->lookup_order_status (TMH_db->cls,
@@ -1258,6 +1376,9 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_set_zero (TMH_currency,
                                         &god->refund_amount));
+  GNUNET_assert (GNUNET_OK ==
+                 TALER_amount_set_zero (TMH_currency,
+                                        &god->refund_taken));
   qs = TMH_db->lookup_refunds_detailed (TMH_db->cls,
                                         hc->instance->settings.id,
                                         &god->h_contract_terms,
@@ -1276,8 +1397,8 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
         ( (! god->refunded) ||
           (1 != TALER_amount_cmp (&god->refund_amount,
                                   &god->sc.refund_expected)) )) ||
-       ((god->sc.awaiting_refund_obtained) &&
-        (god->refund_available)) )
+       ( (god->sc.awaiting_refund_obtained) &&
+         (god->refund_pending) ) )
   {
     /* Client is waiting for a refund larger than what we have, suspend
        until timeout */
@@ -1292,7 +1413,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                     "Awaiting refund exceeding %s\n",
                     TALER_amount2s (&god->sc.refund_expected));
       if (god->sc.awaiting_refund_obtained)
-        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+        GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                     "Awaiting pending refunds\n");
       suspend_god (god);
       return MHD_YES;
@@ -1304,7 +1425,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   {
     enum GNUNET_GenericReturnValue res;
 
-    if (god->refund_available)
+    if (god->refund_pending)
     {
       char *qr;
       char *uri;
@@ -1338,16 +1459,18 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                    get_order_summary (god)),
           TALER_JSON_pack_amount ("refund_amount",
                                   &god->refund_amount),
+          TALER_JSON_pack_amount ("refund_taken",
+                                  &god->refund_taken),
           GNUNET_JSON_pack_string ("taler_refund_uri",
                                    uri),
           GNUNET_JSON_pack_string ("taler_refund_qrcode_svg",
                                    qr));
-        res = TMH_return_from_template (god->sc.con,
-                                        MHD_HTTP_OK,
-                                        "offer_refund",
-                                        hc->instance->settings.id,
-                                        uri,
-                                        context);
+        res = TALER_TEMPLATING_reply (god->sc.con,
+                                      MHD_HTTP_OK,
+                                      "offer_refund",
+                                      hc->instance->settings.id,
+                                      uri,
+                                      context);
         json_decref (context);
       }
       GNUNET_free (uri);
@@ -1363,13 +1486,15 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
         GNUNET_JSON_pack_string ("order_summary",
                                  get_order_summary (god)),
         TALER_JSON_pack_amount ("refund_amount",
-                                &god->refund_amount));
-      res = TMH_return_from_template (god->sc.con,
-                                      MHD_HTTP_OK,
-                                      "show_order_details",
-                                      hc->instance->settings.id,
-                                      NULL,
-                                      context);
+                                &god->refund_amount),
+        TALER_JSON_pack_amount ("refund_taken",
+                                &god->refund_taken));
+      res = TALER_TEMPLATING_reply (god->sc.con,
+                                    MHD_HTTP_OK,
+                                    "show_order_details",
+                                    hc->instance->settings.id,
+                                    NULL,
+                                    context);
       json_decref (context);
     }
     if (GNUNET_SYSERR == res)
@@ -1388,7 +1513,9 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
     GNUNET_JSON_pack_bool ("refunded",
                            god->refunded),
     GNUNET_JSON_pack_bool ("refund_pending",
-                           god->refund_available),
+                           god->refund_pending),
+    TALER_JSON_pack_amount ("refund_taken",
+                            &god->refund_taken),
     TALER_JSON_pack_amount ("refund_amount",
                             &god->refund_amount));
 }
