@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2020 Taler Systems SA
+  (C) 2014-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as
@@ -23,24 +23,54 @@
  * @author Christian Grothoff
  */
 #include "platform.h"
+#include <taler/taler_dbevents.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_json_lib.h>
 #include <taler/taler_exchange_service.h>
 #include "taler-merchant-httpd_post-orders-ID-paid.h"
 
 
+/**
+ * Use database to notify other clients about the
+ * session being captured.
+ *
+ * @param hc http context
+ * @param session_id the captured session
+ * @param fulfillment_url the URL that is now paid for by @a session_id
+ */
+static void
+trigger_session_notification (struct TMH_HandlerContext *hc,
+                              const char *session_id,
+                              const char *fulfillment_url)
+{
+  struct TMH_SessionEventP session_eh = {
+    .header.size = htons (sizeof (session_eh)),
+    .header.type = htons (TALER_DBEVENT_MERCHANT_SESSION_CAPTURED),
+    .merchant_pub = hc->instance->merchant_pub
+  };
+
+  GNUNET_CRYPTO_hash (session_id,
+                      strlen (session_id),
+                      &session_eh.h_session_id);
+  GNUNET_CRYPTO_hash (fulfillment_url,
+                      strlen (fulfillment_url),
+                      &session_eh.h_fulfillment_url);
+  TMH_db->event_notify (TMH_db->cls,
+                        &session_eh.header,
+                        NULL,
+                        0);
+}
+
+
 MHD_RESULT
 TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
                          struct MHD_Connection *connection,
                          struct TMH_HandlerContext *hc)
 {
-  struct TALER_PaymentResponsePS pr = {
-    .purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_PAYMENT_OK),
-    .purpose.size = htonl (sizeof (pr))
-  };
   const char *order_id = hc->infix;
   struct TALER_MerchantSignatureP merchant_sig;
   const char *session_id;
+  struct TALER_PrivateContractHashP hct;
   json_t *contract_terms;
   const char *fulfillment_url;
   enum GNUNET_DB_QueryStatus qs;
@@ -50,7 +80,7 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
       GNUNET_JSON_spec_fixed_auto ("sig",
                                    &merchant_sig),
       GNUNET_JSON_spec_fixed_auto ("h_contract",
-                                   &pr.h_contract_terms),
+                                   &hct),
       GNUNET_JSON_spec_string ("session_id",
                                &session_id),
       GNUNET_JSON_spec_end ()
@@ -71,10 +101,9 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
 
 
   if (GNUNET_OK !=
-      GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_PAYMENT_OK,
-                                  &pr,
-                                  &merchant_sig.eddsa_sig,
-                                  &hc->instance->merchant_pub.eddsa_pub))
+      TALER_merchant_pay_verify (&hct,
+                                 &hc->instance->merchant_pub,
+                                 &merchant_sig))
   {
     GNUNET_break_op (0);
     return TALER_MHD_reply_with_error (connection,
@@ -86,12 +115,14 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
   TMH_db->preflight (TMH_db->cls);
   {
     uint64_t order_serial;
+    bool paid;
 
     qs = TMH_db->lookup_contract_terms (TMH_db->cls,
                                         hc->instance->settings.id,
                                         order_id,
                                         &contract_terms,
                                         &order_serial,
+                                        &paid,
                                         NULL);
   }
   if (0 > qs)
@@ -118,7 +149,7 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
   }
 
   {
-    struct GNUNET_HashCode h_contract_terms;
+    struct TALER_PrivateContractHashP h_contract_terms;
 
     if (GNUNET_OK !=
         TALER_JSON_contract_hash (contract_terms,
@@ -131,7 +162,7 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
                                          TALER_EC_GENERIC_FAILED_COMPUTE_JSON_HASH,
                                          NULL);
     }
-    if (0 != GNUNET_memcmp (&pr.h_contract_terms,
+    if (0 != GNUNET_memcmp (&hct,
                             &h_contract_terms))
     {
       json_decref (contract_terms);
@@ -145,7 +176,7 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
 
   fulfillment_url
     = json_string_value (json_object_get (contract_terms,
-                                            "fulfillment_url"));
+                                          "fulfillment_url"));
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Marking contract %s with %s/%s as paid\n",
               order_id,
@@ -153,7 +184,7 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
               fulfillment_url);
   qs = TMH_db->mark_contract_paid (TMH_db->cls,
                                    hc->instance->settings.id,
-                                   &pr.h_contract_terms,
+                                   &hct,
                                    session_id);
   /* If the order was paid already, we get qs == 0. */
   if (0 > qs)
@@ -165,19 +196,15 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
                                        TALER_EC_GENERIC_DB_STORE_FAILED,
                                        "mark_contract_paid");
   }
- 
-  /* Wake everybody up who waits for this fulfillment_url and session_id */ 
-  if (NULL != fulfillment_url)
-    TMH_long_poll_resume2 (session_id,
-                           fulfillment_url);
 
+  /* Wake everybody up who waits for this fulfillment_url and session_id */
+  if ( (NULL != fulfillment_url) &&
+       (NULL != session_id) )
+    trigger_session_notification (hc,
+                                  session_id,
+                                  fulfillment_url);
   /* fulfillment_url is part of the contract_terms */
   json_decref (contract_terms);
-  /* Resume clients waiting on the order */
-  TMH_long_poll_resume (order_id,
-                        hc->instance,
-                        NULL,
-                        false);
   return TALER_MHD_reply_static (connection,
                                  MHD_HTTP_NO_CONTENT,
                                  NULL,
