@@ -51,21 +51,19 @@ get_instances_ID (struct TMH_MerchantInstance *mi,
         GNUNET_JSON_PACK (
           GNUNET_JSON_pack_string (
             "payto_uri",
-            json_string_value (json_object_get (wm->j_wire,
-                                                "payto_uri"))),
+            wm->payto_uri),
           GNUNET_JSON_pack_data_auto ("h_wire",
                                       &wm->h_wire),
-          GNUNET_JSON_pack_string (
+          GNUNET_JSON_pack_data_auto (
             "salt",
-            json_string_value (json_object_get (wm->j_wire,
-                                                "salt"))),
+            &wm->wire_salt),
           GNUNET_JSON_pack_bool ("active",
                                  wm->active))));
   }
 
   auth = GNUNET_JSON_PACK (
     GNUNET_JSON_pack_string ("method",
-                             GNUNET_is_zero (mi->auth.auth_hash.bits)
+                             GNUNET_is_zero (&mi->auth.auth_hash)
                                                     ? "external"
                                                     : "token"));
   return TALER_MHD_REPLY_JSON_PACK (
@@ -75,6 +73,15 @@ get_instances_ID (struct TMH_MerchantInstance *mi,
                                   ja),
     GNUNET_JSON_pack_string ("name",
                              mi->settings.name),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("website",
+                               mi->settings.website)),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("email",
+                               mi->settings.email)),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("logo",
+                               mi->settings.logo)),
     GNUNET_JSON_pack_data_auto ("merchant_pub",
                                 &mi->merchant_pub),
     GNUNET_JSON_pack_object_incref ("address",
