@@ -21,6 +21,7 @@
 #include "platform.h"
 #include "taler-merchant-httpd_private-get-orders.h"
 #include <taler/taler_json_lib.h>
+#include <taler/taler_dbevents.h>
 
 
 /**
@@ -127,6 +128,11 @@ TMH_force_get_orders_resume (struct TMH_MerchantInstance *mi)
     MHD_resume_connection (po->con);
     po->in_dll = false;
   }
+  if (NULL != mi->po_eh)
+  {
+    TMH_db->event_listen_cancel (mi->po_eh);
+    mi->po_eh = NULL;
+  }
   if (NULL != order_timeout_task)
   {
     GNUNET_SCHEDULER_cancel (order_timeout_task);
@@ -175,6 +181,12 @@ order_timeout (void *cls)
     GNUNET_CONTAINER_DLL_remove (mi->po_head,
                                  mi->po_tail,
                                  po);
+    if ( (NULL == mi->po_head) &&
+         (NULL != mi->po_eh) )
+    {
+      TMH_db->event_listen_cancel (mi->po_eh);
+      mi->po_eh = NULL;
+    }
     po->in_dll = false;
     MHD_resume_connection (po->con);
     TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
@@ -229,7 +241,7 @@ cleanup (void *ctx)
 static void
 process_refunds_cb (void *cls,
                     uint64_t refund_serial,
-                    struct GNUNET_TIME_Absolute timestamp,
+                    struct GNUNET_TIME_Timestamp timestamp,
                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
                     const char *exchange_url,
                     uint64_t rtransaction_id,
@@ -249,57 +261,77 @@ process_refunds_cb (void *cls,
 /**
  * Add order details to our JSON array.
  *
- * @param[in,out] cls a `json_t *` JSON array to build
- * @param order_id ID of the order
+ * @param cls some closure
+ * @param orig_order_id the order this is about
  * @param order_serial serial ID of the order
  * @param creation_time when was the order created
  */
 static void
 add_order (void *cls,
-           const char *order_id,
+           const char *orig_order_id,
            uint64_t order_serial,
-           struct GNUNET_TIME_Absolute creation_time)
+           struct GNUNET_TIME_Timestamp creation_time)
 {
   struct TMH_PendingOrder *po = cls;
-  json_t *contract_terms;
-  struct GNUNET_HashCode h_contract_terms;
+  json_t *contract_terms = NULL;
+  struct TALER_PrivateContractHashP h_contract_terms;
   enum GNUNET_DB_QueryStatus qs;
   const char *summary;
+  char *order_id = NULL;
   bool refundable = false;
   bool paid;
   struct TALER_Amount order_amount;
 
-  qs = TMH_db->lookup_order_status (TMH_db->cls,
-                                    po->instance_id,
-                                    order_id,
-                                    &h_contract_terms,
-                                    &paid);
-  /* qs == 0: contract terms don't exist, so the order cannot be paid. */
-  if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
-    paid = false;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Adding order `%s' (%llu) to result set\n",
+              orig_order_id,
+              (unsigned long long) order_serial);
+  qs = TMH_db->lookup_order_status_by_serial (TMH_db->cls,
+                                              po->instance_id,
+                                              order_serial,
+                                              &order_id,
+                                              &h_contract_terms,
+                                              &paid);
   if (qs < 0)
   {
     GNUNET_break (0);
     po->result = TALER_EC_GENERIC_DB_FETCH_FAILED;
     return;
   }
-
-  if (paid)
+  if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
   {
-    /* if the order was paid, it must have been claimed, so use
-       lookup_contract_terms to avoid the order being deleted in the db. */
+    /* Contract terms don't exist, so the order cannot be paid. */
+    paid = false;
+    if (NULL == orig_order_id)
+    {
+      /* cannot be via DB trigger, and the other code
+         path should have passed an orig_order_id */
+      GNUNET_break (0);
+      po->result = TALER_EC_GENERIC_DB_FETCH_FAILED;
+      return;
+    }
+    order_id = GNUNET_strdup (orig_order_id);
+  }
+
+  {
+    /* First try to find the order in the contracts */
     uint64_t os;
+    bool paid = false;
 
     qs = TMH_db->lookup_contract_terms (TMH_db->cls,
                                         po->instance_id,
                                         order_id,
                                         &contract_terms,
                                         &os,
+                                        &paid,
                                         NULL);
+    if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
+      GNUNET_break (os == order_serial);
   }
-  else
+  if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
   {
-    struct GNUNET_HashCode unused;
+    /* Might still be unclaimed, so try order table */
+    struct TALER_MerchantPostDataHashP unused;
 
     qs = TMH_db->lookup_order (TMH_db->cls,
                                po->instance_id,
@@ -308,23 +340,32 @@ add_order (void *cls,
                                &unused,
                                &contract_terms);
   }
-
+  if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Order %llu disappeared during iteration. Skipping.\n",
+                (unsigned long long) order_serial);
+    json_decref (contract_terms); /* should still be NULL */
+    GNUNET_free (order_id);
+    return;
+  }
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
   {
     GNUNET_break (0);
     po->result = TALER_EC_GENERIC_DB_FETCH_FAILED;
     json_decref (contract_terms);
+    GNUNET_free (order_id);
     return;
   }
 
   {
-    struct GNUNET_TIME_Absolute rd;
+    struct GNUNET_TIME_Timestamp rd;
     struct GNUNET_JSON_Specification spec[] = {
       TALER_JSON_spec_amount ("amount",
                               TMH_currency,
                               &order_amount),
-      GNUNET_JSON_spec_absolute_time ("refund_deadline",
-                                      &rd),
+      GNUNET_JSON_spec_timestamp ("refund_deadline",
+                                  &rd),
       GNUNET_JSON_spec_string ("summary",
                                &summary),
       GNUNET_JSON_spec_end ()
@@ -338,10 +379,11 @@ add_order (void *cls,
       GNUNET_break (0);
       po->result = TALER_EC_MERCHANT_GENERIC_DB_CONTRACT_CONTENT_INVALID;
       json_decref (contract_terms);
+      GNUNET_free (order_id);
       return;
     }
 
-    if (GNUNET_TIME_absolute_is_future (rd) &&
+    if (GNUNET_TIME_absolute_is_future (rd.abs_time) &&
         paid)
     {
       struct TALER_Amount refund_amount;
@@ -359,6 +401,7 @@ add_order (void *cls,
         GNUNET_break (0);
         po->result = TALER_EC_GENERIC_DB_FETCH_FAILED;
         json_decref (contract_terms);
+        GNUNET_free (order_id);
         return;
       }
       if (0 > TALER_amount_cmp (&refund_amount,
@@ -375,8 +418,8 @@ add_order (void *cls,
                                               order_id),
                      GNUNET_JSON_pack_uint64 ("row_id",
                                               order_serial),
-                     GNUNET_JSON_pack_time_abs ("timestamp",
-                                                creation_time),
+                     GNUNET_JSON_pack_timestamp ("timestamp",
+                                                 creation_time),
                      TALER_JSON_pack_amount ("amount",
                                              &order_amount),
                      GNUNET_JSON_pack_string ("summary",
@@ -386,62 +429,106 @@ add_order (void *cls,
                      GNUNET_JSON_pack_bool ("paid",
                                             paid))));
   json_decref (contract_terms);
+  GNUNET_free (order_id);
 }
 
 
 /**
- * There has been a change or addition of a new @a order_id.  Wake up
- * long-polling clients that may have been waiting for this event.
+ * We have received a trigger from the database
+ * that we should (possibly) resume some requests.
  *
- * @param mi the instance where the order changed
- * @param order_id the order that changed
- * @param paid is the order paid by the customer?
- * @param refunded was the order refunded?
- * @param wired was the merchant paid via wire transfer?
- * @param date execution date of the order
- * @param order_serial_id serial ID of the order in the database
+ * @param cls a `struct TMH_MerchantInstance`
+ * @param extra a `struct TMH_OrderChangeEventP`
+ * @param extra_size number of bytes in @a extra
  */
-void
-TMH_notify_order_change (struct TMH_MerchantInstance *mi,
-                         const char *order_id,
-                         bool paid,
-                         bool refunded,
-                         bool wired,
-                         struct GNUNET_TIME_Absolute date,
-                         uint64_t order_serial_id)
+static void
+resume_by_event (void *cls,
+                 const void *extra,
+                 size_t extra_size)
 {
+  struct TMH_MerchantInstance *mi = cls;
+  const struct TMH_OrderChangeEventDetailsP *oce = extra;
   struct TMH_PendingOrder *pn;
+  enum TMH_OrderStateFlags osf;
+  uint64_t order_serial_id;
+  struct GNUNET_TIME_Timestamp date;
 
+  if (sizeof (*oce) != extra_size)
+  {
+    GNUNET_break (0);
+    return;
+  }
+  osf = (enum TMH_OrderStateFlags) ntohl (oce->order_state);
+  order_serial_id = GNUNET_ntohll (oce->order_serial_id);
+  date = GNUNET_TIME_timestamp_ntoh (oce->execution_date);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Received notification about new order %llu\n",
+              (unsigned long long) order_serial_id);
   for (struct TMH_PendingOrder *po = mi->po_head;
        NULL != po;
        po = pn)
   {
     pn = po->next;
-    if (! ( ( ((TALER_EXCHANGE_YNA_YES == po->of.paid) == paid) ||
+    if (! ( ( ((TALER_EXCHANGE_YNA_YES == po->of.paid) ==
+               (0 != (osf & TMH_OSF_PAID))) ||
               (TALER_EXCHANGE_YNA_ALL == po->of.paid) ) &&
-            ( ((TALER_EXCHANGE_YNA_YES == po->of.refunded) == refunded) ||
+            ( ((TALER_EXCHANGE_YNA_YES == po->of.refunded) ==
+               (0 != (osf & TMH_OSF_REFUNDED))) ||
               (TALER_EXCHANGE_YNA_ALL == po->of.refunded) ) &&
-            ( ((TALER_EXCHANGE_YNA_YES == po->of.wired) == wired) ||
+            ( ((TALER_EXCHANGE_YNA_YES == po->of.wired) ==
+               (0 != (osf & TMH_OSF_WIRED))) ||
               (TALER_EXCHANGE_YNA_ALL == po->of.wired) ) ) )
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Client %p waits on different order type\n",
+                  po);
       continue;
+    }
     if (po->of.delta > 0)
     {
       if (order_serial_id < po->of.start_row)
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Client %p waits on different order row\n",
+                    po);
         continue;
-      if (date.abs_value_us < po->of.date.abs_value_us)
+      }
+      if (GNUNET_TIME_timestamp_cmp (date,
+                                     <,
+                                     po->of.date))
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Client %p waits on different order date\n",
+                    po);
         continue;
+      }
       po->of.delta--;
     }
     else
     {
       if (order_serial_id > po->of.start_row)
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Client %p waits on different order row\n",
+                    po);
         continue;
-      if (date.abs_value_us > po->of.date.abs_value_us)
+      }
+      if (GNUNET_TIME_timestamp_cmp (date,
+                                     >,
+                                     po->of.date))
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Client %p waits on different order date\n",
+                    po);
         continue;
+      }
       po->of.delta++;
     }
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Waking up client %p!\n",
+                po);
     add_order (po,
-               order_id,
+               NULL,
                order_serial_id,
                date);
     GNUNET_assert (po->in_dll);
@@ -451,9 +538,52 @@ TMH_notify_order_change (struct TMH_MerchantInstance *mi,
     po->in_dll = false;
     GNUNET_assert (po ==
                    GNUNET_CONTAINER_heap_remove_node (po->hn));
+    po->hn = NULL;
     MHD_resume_connection (po->con);
     TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
   }
+  if (NULL == mi->po_head)
+  {
+    TMH_db->event_listen_cancel (mi->po_eh);
+    mi->po_eh = NULL;
+  }
+}
+
+
+/**
+ * There has been a change or addition of a new @a order_id.  Wake up
+ * long-polling clients that may have been waiting for this event.
+ *
+ * @param mi the instance where the order changed
+ * @param osf order state flags
+ * @param date execution date of the order
+ * @param order_serial_id serial ID of the order in the database
+ */
+void
+TMH_notify_order_change (struct TMH_MerchantInstance *mi,
+                         enum TMH_OrderStateFlags osf,
+                         struct GNUNET_TIME_Timestamp date,
+                         uint64_t order_serial_id)
+{
+  struct TMH_OrderChangeEventDetailsP oce = {
+    .order_serial_id = GNUNET_htonll (order_serial_id),
+    .execution_date = GNUNET_TIME_timestamp_hton (date),
+    .order_state = htonl (osf)
+  };
+  struct TMH_OrderChangeEventP eh = {
+    .header.type = htons (TALER_DBEVENT_MERCHANT_ORDERS_CHANGE),
+    .header.size = htons (sizeof (eh)),
+    .merchant_pub = mi->merchant_pub
+  };
+
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Notifying clients of new order %llu at %s\n",
+              (unsigned long long) order_serial_id,
+              TALER_B2S (&mi->merchant_pub));
+  TMH_db->event_notify (TMH_db->cls,
+                        &eh.header,
+                        &oce,
+                        sizeof (oce));
 }
 
 
@@ -557,17 +687,17 @@ TMH_private_get_orders (const struct TMH_RequestHandler *rh,
     }
   }
   {
-    const char *date_ms_str;
+    const char *date_s_str;
 
-    date_ms_str = MHD_lookup_connection_value (connection,
+    date_s_str = MHD_lookup_connection_value (connection,
                                                MHD_GET_ARGUMENT_KIND,
-                                               "date_ms");
-    if (NULL == date_ms_str)
+                                               "date_s");
+    if (NULL == date_s_str)
     {
       if (of.delta > 0)
-        of.date = GNUNET_TIME_UNIT_ZERO_ABS;
+        of.date = GNUNET_TIME_UNIT_ZERO_TS;
       else
-        of.date = GNUNET_TIME_UNIT_FOREVER_ABS;
+        of.date = GNUNET_TIME_UNIT_FOREVER_TS;
     }
     else
     {
@@ -575,7 +705,7 @@ TMH_private_get_orders (const struct TMH_RequestHandler *rh,
       unsigned long long ll;
 
       if (1 !=
-          sscanf (date_ms_str,
+          sscanf (date_s_str,
                   "%llu%c",
                   &ll,
                   &dummy))
@@ -587,14 +717,15 @@ TMH_private_get_orders (const struct TMH_RequestHandler *rh,
                                            "date_ms");
       }
 
-      of.date = GNUNET_TIME_absolute_from_ms (ll);
-      if (GNUNET_TIME_absolute_is_never (of.date))
+      of.date = GNUNET_TIME_absolute_to_timestamp (
+        GNUNET_TIME_absolute_from_s (ll));
+      if (GNUNET_TIME_absolute_is_never (of.date.abs_time))
       {
         GNUNET_break_op (0);
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_BAD_REQUEST,
                                            TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                           "date_ms");
+                                           "date_s");
       }
     }
   }
@@ -688,7 +819,6 @@ TMH_private_get_orders (const struct TMH_RequestHandler *rh,
   GNUNET_assert (NULL != po->pa);
   po->instance_id = hc->instance->settings.id;
   po->mi = hc->instance;
-
   qs = TMH_db->lookup_orders (TMH_db->cls,
                               po->instance_id,
                               &of,
@@ -696,6 +826,7 @@ TMH_private_get_orders (const struct TMH_RequestHandler *rh,
                               po);
   if (0 > qs)
   {
+    GNUNET_break (0);
     po->result = TALER_EC_GENERIC_DB_FETCH_FAILED;
   }
   if (TALER_EC_NONE != po->result)
@@ -724,6 +855,20 @@ TMH_private_get_orders (const struct TMH_RequestHandler *rh,
                                  mi->po_tail,
                                  po);
     po->in_dll = true;
+    if (NULL == mi->po_eh)
+    {
+      struct TMH_OrderChangeEventP change_eh = {
+        .header.type = htons (TALER_DBEVENT_MERCHANT_ORDERS_CHANGE),
+        .header.size = htons (sizeof (change_eh)),
+        .merchant_pub = mi->merchant_pub
+      };
+
+      mi->po_eh = TMH_db->event_listen (TMH_db->cls,
+                                        &change_eh.header,
+                                        GNUNET_TIME_UNIT_FOREVER_REL,
+                                        &resume_by_event,
+                                        mi);
+    }
     MHD_suspend_connection (connection);
     {
       struct TMH_PendingOrder *pot;
