@@ -70,8 +70,8 @@ struct GetReserveContext
  */
 static void
 handle_reserve_details (void *cls,
-                        struct GNUNET_TIME_Absolute creation_time,
-                        struct GNUNET_TIME_Absolute expiration_time,
+                        struct GNUNET_TIME_Timestamp creation_time,
+                        struct GNUNET_TIME_Timestamp expiration_time,
                         const struct TALER_Amount *merchant_initial_amount,
                         const struct TALER_Amount *exchange_initial_amount,
                         const struct TALER_Amount *picked_up_amount,
@@ -84,11 +84,6 @@ handle_reserve_details (void *cls,
 {
   struct GetReserveContext *ctx = cls;
   json_t *tips_json;
-  struct GNUNET_TIME_Absolute creation_time_round = creation_time;
-  struct GNUNET_TIME_Absolute expiration_time_round = expiration_time;
-
-  GNUNET_TIME_round_abs (&creation_time_round);
-  GNUNET_TIME_round_abs (&expiration_time_round);
 
   if (NULL != tips)
   {
@@ -115,10 +110,10 @@ handle_reserve_details (void *cls,
   ctx->res = TALER_MHD_REPLY_JSON_PACK (
     ctx->connection,
     MHD_HTTP_OK,
-    GNUNET_JSON_pack_time_abs ("creation_time",
-                               creation_time_round),
-    GNUNET_JSON_pack_time_abs ("expiration_time",
-                               expiration_time_round),
+    GNUNET_JSON_pack_timestamp ("creation_time",
+                                creation_time),
+    GNUNET_JSON_pack_timestamp ("expiration_time",
+                                expiration_time),
     TALER_JSON_pack_amount ("merchant_initial_amount",
                             merchant_initial_amount),
     TALER_JSON_pack_amount ("exchange_initial_amount",
@@ -157,6 +152,7 @@ TMH_private_get_reserves_ID (const struct TMH_RequestHandler *rh,
   struct TALER_ReservePublicKeyP reserve_pub;
   bool tips;
 
+  (void) rh;
   if (GNUNET_OK !=
       GNUNET_STRINGS_string_to_data (hc->infix,
                                      strlen (hc->infix),
