@@ -42,7 +42,7 @@
 static void
 free_wm (struct TMH_WireMethod *wm)
 {
-  json_decref (wm->j_wire);
+  GNUNET_free (wm->payto_uri);
   GNUNET_free (wm->wire_method);
   GNUNET_free (wm);
 }
@@ -71,6 +71,18 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
                            &payto_uris),
     GNUNET_JSON_spec_string ("name",
                              &name),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("website",
+                               (const char **) &is.website),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("email",
+                               (const char **) &is.email),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("logo",
+                               (const char **) &is.logo),
+      NULL),
     GNUNET_JSON_spec_json ("address",
                            &is.address),
     GNUNET_JSON_spec_json ("jurisdiction",
@@ -83,10 +95,10 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
     TALER_JSON_spec_amount ("default_max_deposit_fee",
                             TMH_currency,
                             &is.default_max_deposit_fee),
-    TALER_JSON_spec_relative_time ("default_wire_transfer_delay",
-                                   &is.default_wire_transfer_delay),
-    TALER_JSON_spec_relative_time ("default_pay_delay",
-                                   &is.default_pay_delay),
+    GNUNET_JSON_spec_relative_time ("default_wire_transfer_delay",
+                                    &is.default_wire_transfer_delay),
+    GNUNET_JSON_spec_relative_time ("default_pay_delay",
+                                    &is.default_pay_delay),
     GNUNET_JSON_spec_end ()
   };
   enum GNUNET_DB_QueryStatus qs;
@@ -116,6 +128,16 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
                                        TALER_EC_GENERIC_PARAMETER_MALFORMED,
                                        "address");
   }
+  if ( (NULL != is.logo) &&
+       (! TMH_image_data_url_valid (is.logo)) )
+  {
+    GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                       "logo");
+  }
 
   if (! TMH_location_object_valid (is.jurisdiction))
   {
@@ -159,6 +181,18 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
     /* Check for equality of settings */
     if (! ( (0 == strcmp (mi->settings.name,
                           name)) &&
+            ((mi->settings.email == is.email) ||
+             (NULL != is.email && NULL != mi->settings.email &&
+              0 == strcmp (mi->settings.email,
+                           is.email))) &&
+            ((mi->settings.website == is.website) ||
+             (NULL != is.website && NULL != mi->settings.website &&
+              0 == strcmp (mi->settings.website,
+                           is.website))) &&
+            ((mi->settings.logo == is.logo) ||
+             (NULL != is.logo && NULL != mi->settings.logo &&
+              0 == strcmp (mi->settings.logo,
+                           is.logo))) &&
             (1 == json_equal (mi->settings.address,
                               is.address)) &&
             (1 == json_equal (mi->settings.jurisdiction,
@@ -175,10 +209,12 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
                                     &is.default_max_wire_fee)) &&
             (mi->settings.default_wire_fee_amortization ==
              is.default_wire_fee_amortization) &&
-            (mi->settings.default_wire_transfer_delay.rel_value_us ==
-             is.default_wire_transfer_delay.rel_value_us) &&
-            (mi->settings.default_pay_delay.rel_value_us ==
-             is.default_pay_delay.rel_value_us) ) )
+            (GNUNET_TIME_relative_cmp (mi->settings.default_wire_transfer_delay,
+                                       ==,
+                                       is.default_wire_transfer_delay)) &&
+            (GNUNET_TIME_relative_cmp (mi->settings.default_pay_delay,
+                                       ==,
+                                       is.default_pay_delay)) ) )
     {
       is.id = mi->settings.id;
       is.name = GNUNET_strdup (name);
@@ -208,8 +244,7 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
            NULL != wm;
            wm = wm->next)
       {
-        const char *uri = json_string_value (json_object_get (wm->j_wire,
-                                                              "payto_uri"));
+        const char *uri = wm->payto_uri;
 
         GNUNET_assert (NULL != uri);
         matched = false;
@@ -287,6 +322,7 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
                                      wm_tail,
                                      wm);
         ad.h_wire = wm->h_wire;
+        ad.salt = wm->wire_salt;
         ad.active = true;
         qs = TMH_db->insert_account (TMH_db->cls,
                                      mi->settings.id,
@@ -350,6 +386,9 @@ giveup:
 
   /* Update our 'settings' */
   GNUNET_free (mi->settings.name);
+  GNUNET_free (mi->settings.email);
+  GNUNET_free (mi->settings.website);
+  GNUNET_free (mi->settings.logo);
   json_decref (mi->settings.address);
   json_decref (mi->settings.jurisdiction);
   is.id = mi->settings.id;
@@ -357,6 +396,12 @@ giveup:
   mi->settings.address = json_incref (mi->settings.address);
   mi->settings.jurisdiction = json_incref (mi->settings.jurisdiction);
   mi->settings.name = GNUNET_strdup (name);
+  if (NULL != is.email)
+    mi->settings.email = GNUNET_strdup (is.email);
+  if (NULL != is.website)
+    mi->settings.website = GNUNET_strdup (is.website);
+  if (NULL != is.logo)
+    mi->settings.logo = GNUNET_strdup (is.logo);
 
   /* Add 'new' wire methods to our list */
   {
