@@ -39,8 +39,8 @@ static struct MHD_Response *spa;
 
 MHD_RESULT
 TMH_return_spa (const struct TMH_RequestHandler *rh,
-                   struct MHD_Connection *connection,
-                   struct TMH_HandlerContext *hc)
+                struct MHD_Connection *connection,
+                struct TMH_HandlerContext *hc)
 {
   if ( (MHD_YES ==
         TALER_MHD_can_compress (connection)) &&
@@ -152,6 +152,10 @@ TMH_spa_init ()
                                                "text/html"));
       }
     }
+    else
+    {
+      GNUNET_free (in);
+    }
   }
 
   spa = MHD_create_response_from_fd (sb.st_size,
@@ -170,6 +174,7 @@ TMH_spa_init ()
     }
     return GNUNET_SYSERR;
   }
+  GNUNET_free (dn);
   GNUNET_break (MHD_YES ==
                 MHD_add_response_header (spa,
                                          MHD_HTTP_HEADER_CONTENT_TYPE,
