@@ -339,7 +339,7 @@ TALER_TESTING_cmd_merchant_post_products2 (
   json_t *taxes,
   int64_t total_stock,
   json_t *address,
-  struct GNUNET_TIME_Absolute next_restock,
+  struct GNUNET_TIME_Timestamp next_restock,
   unsigned int http_status);
 
 
@@ -404,7 +404,7 @@ TALER_TESTING_cmd_merchant_patch_product (
   int64_t total_stock,
   uint64_t total_lost,
   json_t *address,
-  struct GNUNET_TIME_Absolute next_restock,
+  struct GNUNET_TIME_Timestamp next_restock,
   unsigned int http_status);
 
 
@@ -502,15 +502,14 @@ TALER_TESTING_cmd_merchant_delete_product (const char *label,
  * @return the command
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_post_orders (const char *label,
-                                        const char *merchant_url,
-                                        unsigned int http_status,
-                                        const char *order_id,
-                                        struct GNUNET_TIME_Absolute
-                                        refund_deadline,
-                                        struct GNUNET_TIME_Absolute
-                                        pay_deadline,
-                                        const char *amount);
+TALER_TESTING_cmd_merchant_post_orders (
+  const char *label,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *order_id,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  struct GNUNET_TIME_Timestamp pay_deadline,
+  const char *amount);
 
 /**
  * Make the "proposal" command AVOIDING claiming the order.
@@ -526,15 +525,14 @@ TALER_TESTING_cmd_merchant_post_orders (const char *label,
  * @return the command
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_post_orders_no_claim (const char *label,
-                                                 const char *merchant_url,
-                                                 unsigned int http_status,
-                                                 const char *order_id,
-                                                 struct GNUNET_TIME_Absolute
-                                                 refund_deadline,
-                                                 struct GNUNET_TIME_Absolute
-                                                 pay_deadline,
-                                                 const char *amount);
+TALER_TESTING_cmd_merchant_post_orders_no_claim (
+  const char *label,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *order_id,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  struct GNUNET_TIME_Timestamp pay_deadline,
+  const char *amount);
 
 /**
  * Make the "proposal" command.
@@ -564,8 +562,8 @@ TALER_TESTING_cmd_merchant_post_orders2 (
   const char *merchant_url,
   unsigned int http_status,
   const char *order_id,
-  struct GNUNET_TIME_Absolute refund_deadline,
-  struct GNUNET_TIME_Absolute pay_deadline,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  struct GNUNET_TIME_Timestamp pay_deadline,
   bool claim_token,
   const char *amount,
   const char *payment_target,
@@ -1437,6 +1435,7 @@ TALER_TESTING_cmd_tip_authorize_fake (const char *label);
  * @param amounts array of string-defined amounts that specifies
  *        which denominations will be accepted for tipping.
  * @param ec expected Taler error code.
+ * @return the command
  */
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_tip_pickup_with_ec (const char *label,
@@ -1457,6 +1456,7 @@ TALER_TESTING_cmd_tip_pickup_with_ec (const char *label,
  *        that offers a tip id to pick up.
  * @param amounts array of string-defined amounts that specifies
  *        which denominations will be accepted for tipping.
+ * @return the command
  */
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_tip_pickup (const char *label,
@@ -1466,249 +1466,85 @@ TALER_TESTING_cmd_tip_pickup (const char *label,
                               const char **amounts);
 
 
+/**
+ * Run a command to fetch the KYC status of a merchant.
+ *
+ * @param label the command label
+ * @param merchant_url base URL of the merchant
+ * @param instance_id instance to use, NULL if instance is part of @a merchant_url
+ * @param h_wire_ref label of command with a merchant wire hash trait
+ *        of the bank account to check KYC for; NULL to check all accounts
+ * @param exchange_url base URL of the exchange to check KYC status for
+ * @param expected_http_status expected HTTP status
+ * @return the command
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_kyc_get (const char *label,
+                                    const char *merchant_url,
+                                    const char *instance_id,
+                                    const char *h_wire_ref,
+                                    const char *exchange_url,
+                                    unsigned int expected_http_status);
+
+
 /* ****** Specific traits supported by this component ******* */
 
+
 /**
- * Offer a merchant signature over a contract.
+ * Call @a op on all simple traits.
  *
- * @param index which signature to offer if there are multiple
- *        on offer
- * @param merchant_sig set to the offered signature.
- * @return the trait
+ * @param op macro to call
  */
-struct TALER_TESTING_Trait
-TALER_TESTING_make_trait_merchant_sig (
-  unsigned int index,
-  const struct TALER_MerchantSignatureP *merchant_sig);
+// FIXME: rename: refund_entry->refund_detail
+#define TALER_MERCHANT_TESTING_SIMPLE_TRAITS(op) \
+  op (claim_nonce, const struct GNUNET_CRYPTO_EddsaPublicKey) \
+  op (tip_id, const struct TALER_TipIdentifierP) \
+  op (pickup_id, const struct TALER_PickupIdentifierP) \
+  op (instance_name, const char *) \
+  op (instance_id, const char *) \
+  op (address, const json_t) \
+  op (product_description, const char *) \
+  op (product_image, const char *) \
+  op (product_stock, const int64_t) \
+  op (product_unit, const char *) \
+  op (product_id, const char *) \
+  op (reason, const char *) \
+  op (lock_uuid, const char *) \
+  op (auth_token, const char *) \
+  op (paths_length, const uint32_t) \
+  op (payto_length, const uint32_t) \
+  op (num_planchets, const uint32_t) \
+  op (i18n_description, const json_t) \
+  op (taxes, const json_t)      \
+  op (fee, const struct TALER_Amount) \
+  op (max_wire_fee, const struct TALER_Amount) \
+  op (max_deposit_fee, const struct TALER_Amount) \
+  op (wire_fee_amortization, const uint32_t) \
+  op (jurisdiction, const json_t) \
+  op (wire_delay, const struct GNUNET_TIME_Relative) \
+  op (pay_delay, const struct GNUNET_TIME_Relative) \
+  op (refund_entry, const struct TALER_MERCHANT_RefundDetail) \
+  op (order_terms, const json_t) \
+  op (h_contract_terms, const struct TALER_PrivateContractHashP) \
+  op (h_wire, const struct TALER_MerchantWireHashP) \
+  op (proposal_reference, const char *)
 
 
 /**
- * Obtain a merchant signature over a contract from a @a cmd.
+ * Call @a op on all indexed traits.
  *
- * @param cmd command to extract trait from
- * @param index which signature to pick if @a cmd has multiple
- *        on offer
- * @param[out] merchant_sig set to the wanted signature.
- *
- * @return #GNUNET_OK on success
+ * @param op macro to call
  */
-int
-TALER_TESTING_get_trait_merchant_sig (
-  const struct TALER_TESTING_Command *cmd,
-  unsigned int index,
-  struct TALER_MerchantSignatureP **merchant_sig);
+#define TALER_MERCHANT_TESTING_INDEXED_TRAITS(op) \
+  op (coin_reference, const char *) \
+  op (paths, const char *) \
+  op (payto_uris, const char *) \
+  op (amounts, const struct TALER_Amount) \
+  op (planchet_secrets, const struct TALER_PlanchetMasterSecretP)
 
 
-/**
- * Offer an order claim nonce.
- *
- * @param index which nonce to offer if there are
- *        multiple on offer.
- * @param nonce set to the offered nonce.
- * @return the trait
- */
-struct TALER_TESTING_Trait
-TALER_TESTING_make_trait_claim_nonce (
-  unsigned int index,
-  const struct GNUNET_CRYPTO_EddsaPublicKey *nonce);
-
-
-/**
- * Obtain an order claim nonce from a @a cmd.
- *
- * @param cmd command to extract the trait from.
- * @param index which nonce to pick if @a
- *        cmd has multiple on offer
- * @param[out] nonce set to the wanted data.
- * @return #GNUNET_OK on success
- */
-int
-TALER_TESTING_get_trait_claim_nonce (
-  const struct TALER_TESTING_Command *cmd,
-  unsigned int index,
-  const struct GNUNET_CRYPTO_EddsaPublicKey **nonce);
-
-
-/**
- * Obtain a reference to a proposal command.  Any command that
- * works with proposals, might need to offer their reference to
- * it.  Notably, the "pay" command, offers its proposal reference
- * to the "pay abort" command as the latter needs to reconstruct
- * the same data needed by the former in order to use the "pay
- * abort" API.
- *
- * @param cmd command to extract the trait from.
- * @param index which reference to pick if @a cmd has multiple
- *        on offer.
- * @param[out] proposal_reference set to the wanted reference.
- * @return #GNUNET_OK on success
- */
-int
-TALER_TESTING_get_trait_proposal_reference (
-  const struct TALER_TESTING_Command *cmd,
-  unsigned int index,
-  const char **proposal_reference);
-
-/**
- * Offer a proposal reference.
- *
- * @param index which reference to offer if there are
- *        multiple on offer.
- * @param proposal_reference pointer to the reference to offer.
- *
- * @return the trait
- */
-struct TALER_TESTING_Trait
-TALER_TESTING_make_trait_proposal_reference (unsigned int index,
-                                             const char *proposal_reference);
-
-/**
- * Offer a coin reference.
- *
- * @param index which reference to offer if there are
- *        multiple on offer.
- * @param coin_reference set to the offered reference.
- *
- * @return the trait
- */
-struct TALER_TESTING_Trait
-TALER_TESTING_make_trait_coin_reference (unsigned int index,
-                                         const char *coin_reference);
-
-/**
- * Obtain a reference to any command that can provide coins as
- * traits.
- *
- * @param cmd command to extract trait from
- * @param index which reference to pick if @a cmd has multiple
- *        on offer
- * @param[out] coin_reference set to the wanted reference.
- *        NOTE: a _single_ reference can contain
- *        _multiple_ instances, using semi-colon as separator.
- *        For example, a _single_ reference can be this:
- *        "coin-ref-1", or even this: "coin-ref-1;coin-ref-2".
- *        The "pay" command contains functions that can parse
- *        such format.
- *
- * @return #GNUNET_OK on success
- */
-int
-TALER_TESTING_get_trait_coin_reference (const struct TALER_TESTING_Command *cmd,
-                                        unsigned int index,
-                                        const char **coin_reference);
-
-
-/**
- * Obtain planchet secrets from a @a cmd.
- *
- * @param cmd command to extract trait from.
- * @param index index of the trait.
- * @param[out] planchet_secrets set to the wanted secrets.
- * @return #GNUNET_OK on success
- */
-int
-TALER_TESTING_get_trait_planchet_secrets (
-  const struct TALER_TESTING_Command *cmd,
-  unsigned int index,
-  struct TALER_PlanchetSecretsP **planchet_secrets);
-
-
-/**
- * Offer planchet secrets.
- *
- * @param index of the trait.
- * @param planchet_secrets set to the offered secrets.
- * @return the trait
- */
-struct TALER_TESTING_Trait
-TALER_TESTING_make_trait_planchet_secrets (
-  unsigned int index,
-  const struct TALER_PlanchetSecretsP *planchet_secrets);
-
-
-/**
- * Offer tip id.
- *
- * @param index which tip id to offer if there are
- *        multiple on offer.
- * @param tip_id set to the offered tip id.
- * @return the trait
- */
-struct TALER_TESTING_Trait
-TALER_TESTING_make_trait_tip_id (unsigned int index,
-                                 const struct GNUNET_HashCode *tip_id);
-
-
-/**
- * Obtain tip id from a @a cmd.
- *
- * @param cmd command to extract the trait from.
- * @param index which tip id to pick if @a
- *        cmd has multiple on offer
- * @param[out] tip_id set to the wanted data.
- * @return #GNUNET_OK on success
- */
-int
-TALER_TESTING_get_trait_tip_id (const struct TALER_TESTING_Command *cmd,
-                                unsigned int index,
-                                const struct GNUNET_HashCode **tip_id);
-
-
-/**
- * Offer contract terms hash code.
- *
- * @param index which hashed contract terms to
- *        offer if there are multiple on offer
- * @param h_contract_terms set to the offered hashed
- *        contract terms.
- * @return the trait
- */
-struct TALER_TESTING_Trait
-TALER_TESTING_make_trait_h_contract_terms (
-  unsigned int index,
-  const struct GNUNET_HashCode *h_contract_terms);
-
-
-/**
- * Obtain contract terms hash from a @a cmd.
- *
- * @param cmd command to extract the trait from.
- * @param index index number of the trait to fetch.
- * @param[out] h_contract_terms set to the wanted data.
- * @return #GNUNET_OK on success
- */
-int
-TALER_TESTING_get_trait_h_contract_terms (
-  const struct TALER_TESTING_Command *cmd,
-  unsigned int index,
-  const struct GNUNET_HashCode **h_contract_terms);
-
-/**
- * Offer refund entry.
- *
- * @param index index number of the trait to offer.
- * @param refund_entry set to the offered refund entry.
- * @return the trait
- */
-struct TALER_TESTING_Trait
-TALER_TESTING_make_trait_refund_entry ( // FIXME: rename: entry->detail
-  unsigned int index,
-  const struct TALER_MERCHANT_RefundDetail *refund_entry);
-
-
-/**
- * Obtain refund entry from a @a cmd.
- *
- * @param cmd command to extract the trait from.
- * @param index the trait index.
- * @param[out] refund_entry set to the wanted data.
- * @return #GNUNET_OK on success
- */
-int
-TALER_TESTING_get_trait_refund_entry ( // FIXME: rename: entry->detail
-  const struct TALER_TESTING_Command *cmd,
-  unsigned int index,
-  const struct TALER_MERCHANT_RefundDetail **refund_entry);
+TALER_MERCHANT_TESTING_SIMPLE_TRAITS (TALER_TESTING_MAKE_DECL_SIMPLE_TRAIT)
+TALER_MERCHANT_TESTING_INDEXED_TRAITS (TALER_TESTING_MAKE_DECL_INDEXED_TRAIT)
 
 
 #endif
