@@ -33,6 +33,50 @@
 struct TALER_MERCHANTDB_Plugin;
 
 
+GNUNET_NETWORK_STRUCT_BEGIN
+
+
+/**
+ * @brief Hash over an order request, used for the idempotency check.
+ */
+struct TALER_MerchantPostDataHashP
+{
+  /**
+   * The authentication hash is a SHA-512 hash code.
+   */
+  struct GNUNET_HashCode hash;
+};
+
+
+/**
+ * @brief Hash used for client authenticiation. Computed with a
+ * `struct TALER_MerchantAuthenticationSaltP`.
+ */
+struct TALER_MerchantAuthenticationHashP
+{
+  /**
+   * The authentication hash is a SHA-512 hash code.
+   * All zeros if authentication is off.
+   */
+  struct GNUNET_HashCode hash;
+};
+
+
+/**
+ * @brief Salt used for client authenticiation.
+ */
+struct TALER_MerchantAuthenticationSaltP
+{
+  /**
+   * The authentication salt is a 256-bit value.
+   */
+  uint32_t salt[256 / 8 / sizeof(uint32_t)];  /* = 8 */
+};
+
+
+GNUNET_NETWORK_STRUCT_END
+
+
 /**
  * Details about a wire account of the merchant.
  */
@@ -41,12 +85,12 @@ struct TALER_MERCHANTDB_AccountDetails
   /**
    * Hash of the wire details (@e payto_uri and @e salt).
    */
-  struct GNUNET_HashCode h_wire;
+  struct TALER_MerchantWireHashP h_wire;
 
   /**
    * Salt value used for hashing @e payto_uri.
    */
-  struct GNUNET_HashCode salt;
+  struct TALER_WireSaltP salt;
 
   /**
    * Actual account address as a payto://-URI.
@@ -68,13 +112,13 @@ struct TALER_MERCHANTDB_InstanceAuthSettings
   /**
    * Hash used for authentication.  All zero if authentication is off.
    */
-  struct GNUNET_HashCode auth_hash;
+  struct TALER_MerchantAuthenticationHashP auth_hash;
 
   /**
    * Salt used to hash the "Authentication" header, the result must then
    * match the @e auth_hash.
    */
-  struct GNUNET_ShortHashCode auth_salt;
+  struct TALER_MerchantAuthenticationSaltP auth_salt;
 };
 
 /**
@@ -92,6 +136,20 @@ struct TALER_MERCHANTDB_InstanceSettings
    */
   char *name;
 
+  /**
+   * merchant's site url
+   */
+  char *website;
+
+  /**
+   * email contact for customers
+   */
+  char *email;
+
+  /**
+   * merchant's logo data uri
+   */
+  char *logo;
   /**
    * Address of the business
    */
@@ -233,7 +291,14 @@ struct TALER_MERCHANTDB_ProductDetails
    * Identifies when the product will be restocked. 0 for unknown,
    * #GNUNET_TIME_UNIT_FOREVER_ABS for never.
    */
-  struct GNUNET_TIME_Absolute next_restock;
+  struct GNUNET_TIME_Timestamp next_restock;
+
+  /**
+   * Minimum required age for consumers buying this product.
+   * Default is 0. Only enforced of an exchange supports age
+   * restrictions.
+   */
+  uint32_t minimum_age;
 };
 
 
@@ -260,7 +325,7 @@ struct TALER_MERCHANTDB_OrderFilter
   /**
    * Filter orders by date, exact meaning depends on @e delta.
    */
-  struct GNUNET_TIME_Absolute date;
+  struct GNUNET_TIME_Timestamp date;
 
   /**
    * Filter orders by order serial number, exact meaning depends on @e delta.
@@ -293,7 +358,7 @@ typedef void
 (*TALER_MERCHANTDB_OrdersCallback)(void *cls,
                                    const char *order_id,
                                    uint64_t order_serial,
-                                   struct GNUNET_TIME_Absolute timestamp);
+                                   struct GNUNET_TIME_Timestamp timestamp);
 
 
 /**
@@ -349,7 +414,7 @@ typedef void
   void *cls,
   const struct TALER_WireTransferIdentifierRawP *wtid,
   const char *exchange_url,
-  struct GNUNET_TIME_Absolute execution_time,
+  struct GNUNET_TIME_Timestamp execution_time,
   const struct TALER_Amount *deposit_value,
   const struct TALER_Amount *deposit_fee,
   bool transfer_confirmed);
@@ -372,7 +437,7 @@ typedef void
 (*TALER_MERCHANTDB_RefundDetailCallback)(
   void *cls,
   uint64_t refund_serial,
-  struct GNUNET_TIME_Absolute timestamp,
+  struct GNUNET_TIME_Timestamp timestamp,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
   const char *exchange_url,
   uint64_t rtransaction_id,
@@ -381,6 +446,29 @@ typedef void
   bool pending);
 
 
+/**
+ * Function called from ``account_kyc_get_status``
+ * with KYC status information for this merchant.
+ *
+ * @param cls closure
+ * @param h_wire hash of the wire account
+ * @param exchange_kyc_serial serial number for the KYC process at the exchange, 0 if unknown
+ * @param payto_uri payto:// URI of the merchant's bank account
+ * @param exchange_url base URL of the exchange for which this is a status
+ * @param last_check when did we last get an update on our KYC status from the exchange
+ * @param kyc_ok true if we satisfied the KYC requirements
+ */
+typedef void
+(*TALER_MERCHANTDB_KycCallback)(
+  void *cls,
+  const struct TALER_MerchantWireHashP *h_wire,
+  uint64_t exchange_kyc_serial,
+  const char *payto_uri,
+  const char *exchange_url,
+  struct GNUNET_TIME_Timestamp last_check,
+  bool kyc_ok);
+
+
 /**
  * Results from trying to increase a refund.
  */
@@ -470,7 +558,7 @@ typedef void
   const char *payto_uri,
   const char *exchange_url,
   uint64_t transfer_serial_id,
-  struct GNUNET_TIME_Absolute execution_time,
+  struct GNUNET_TIME_Timestamp execution_time,
   bool verified,
   bool confirmed);
 
@@ -494,8 +582,8 @@ typedef void
 (*TALER_MERCHANTDB_ReservesCallback)(
   void *cls,
   const struct TALER_ReservePublicKeyP *reserve_pub,
-  struct GNUNET_TIME_Absolute creation_time,
-  struct GNUNET_TIME_Absolute expiration_time,
+  struct GNUNET_TIME_Timestamp creation_time,
+  struct GNUNET_TIME_Timestamp expiration_time,
   const struct TALER_Amount *merchant_initial_amount,
   const struct TALER_Amount *exchange_initial_amount,
   const struct TALER_Amount *pickup_amount,
@@ -529,7 +617,7 @@ struct TALER_MERCHANTDB_TipDetails
   /**
    * ID of the tip.
    */
-  struct GNUNET_HashCode tip_id;
+  struct TALER_TipIdentifierP tip_id;
 
   /**
    * Total amount of the tip.
@@ -559,7 +647,7 @@ typedef void
   void *cls,
   uint64_t deposit_serial,
   const char *exchange_url,
-  const struct GNUNET_HashCode *h_wire,
+  const struct TALER_MerchantWireHashP *h_wire,
   const struct TALER_Amount *amount_with_fee,
   const struct TALER_Amount *deposit_fee,
   const struct TALER_CoinSpendPublicKeyP *coin_pub);
@@ -586,8 +674,8 @@ typedef void
 typedef void
 (*TALER_MERCHANTDB_ReserveDetailsCallback)(
   void *cls,
-  struct GNUNET_TIME_Absolute creation_time,
-  struct GNUNET_TIME_Absolute expiration_time,
+  struct GNUNET_TIME_Timestamp creation_time,
+  struct GNUNET_TIME_Timestamp expiration_time,
   const struct TALER_Amount *merchant_initial_amount,
   const struct TALER_Amount *exchange_initial_amount,
   const struct TALER_Amount *picked_up_amount,
@@ -610,7 +698,7 @@ typedef void
 typedef void
 (*TALER_MERCHANTDB_TipsCallback)(void *cls,
                                  uint64_t row_id,
-                                 struct GNUNET_HashCode tip_id,
+                                 struct TALER_TipIdentifierP tip_id,
                                  struct TALER_Amount amount);
 
 
@@ -637,9 +725,9 @@ typedef void
   const struct TALER_Amount *deposit_fee,
   const struct TALER_Amount *refund_fee,
   const struct TALER_Amount *wire_fee,
-  const struct GNUNET_HashCode *h_wire,
-  struct GNUNET_TIME_Absolute deposit_timestamp,
-  struct GNUNET_TIME_Absolute refund_deadline,
+  const struct TALER_MerchantWireHashP *h_wire,
+  struct GNUNET_TIME_Timestamp deposit_timestamp,
+  struct GNUNET_TIME_Timestamp refund_deadline,
   const struct TALER_ExchangeSignatureP *exchange_sig,
   const struct TALER_ExchangePublicKeyP *exchange_pub);
 
@@ -652,7 +740,7 @@ struct TALER_MERCHANTDB_PickupDetails
   /**
    * Identifier for the pickup operation.
    */
-  struct GNUNET_HashCode pickup_id;
+  struct TALER_PickupIdentifierP pickup_id;
 
   /**
    * Total amount requested for this @e pickup_id.
@@ -695,8 +783,8 @@ struct TALER_MERCHANTDB_Plugin
    *
    * @param cls closure
    */
-  int
-  (*connect) (void *cls);
+  enum GNUNET_GenericReturnValue
+  (*connect)(void *cls);
 
   /**
    * Drop merchant tables. Used for testcases and to reset the DB.
@@ -704,8 +792,8 @@ struct TALER_MERCHANTDB_Plugin
    * @param cls closure
    * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure
    */
-  int
-  (*drop_tables) (void *cls);
+  enum GNUNET_GenericReturnValue
+  (*drop_tables)(void *cls);
 
   /**
    * Initialize merchant tables
@@ -713,8 +801,8 @@ struct TALER_MERCHANTDB_Plugin
    * @param cls closure
    * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure
    */
-  int
-  (*create_tables) (void *cls);
+  enum GNUNET_GenericReturnValue
+  (*create_tables)(void *cls);
 
   /**
    * Register callback to be invoked on events of type @a es.
@@ -776,9 +864,9 @@ struct TALER_MERCHANTDB_Plugin
    *             must point to a constant
    * @return #GNUNET_OK on success
    */
-  int
-  (*start) (void *cls,
-            const char *name);
+  enum GNUNET_GenericReturnValue
+  (*start)(void *cls,
+           const char *name);
 
   /**
    * Start a transaction with isolation level 'read committed'.
@@ -788,9 +876,9 @@ struct TALER_MERCHANTDB_Plugin
    *             must point to a constant
    * @return #GNUNET_OK on success
    */
-  int
-  (*start_read_committed) (void *cls,
-                           const char *name);
+  enum GNUNET_GenericReturnValue
+  (*start_read_committed)(void *cls,
+                          const char *name);
 
   /**
    * Roll back the current transaction of a database connection.
@@ -928,10 +1016,10 @@ struct TALER_MERCHANTDB_Plugin
    * @return database result code
    */
   enum GNUNET_DB_QueryStatus
-  (*update_instance_auth)(void *cls,
-                          const char *merchant_id,
-                          const struct
-                          TALER_MERCHANTDB_InstanceAuthSettings *ias);
+  (*update_instance_auth)(
+    void *cls,
+    const char *merchant_id,
+    const struct TALER_MERCHANTDB_InstanceAuthSettings *ias);
 
   /**
    * Set an instance's account in our database to "inactive".
@@ -944,7 +1032,7 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*inactivate_account)(void *cls,
                         const char *merchant_id,
-                        const struct GNUNET_HashCode *h_wire);
+                        const struct TALER_MerchantWireHashP *h_wire);
 
 
   /**
@@ -958,7 +1046,55 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*activate_account)(void *cls,
                       const char *merchant_id,
-                      const struct GNUNET_HashCode *h_wire);
+                      const struct TALER_MerchantWireHashP *h_wire);
+
+
+  /**
+   * Check an instance's account's KYC status.
+   *
+   * @param cls closure
+   * @param merchant_id merchant backend instance ID
+   * @param h_wire hash of the wire account to check,
+   *        NULL to check all accounts of the merchant
+   * @param exchange_url base URL of the exchange to check,
+   *        NULL to check all exchanges
+   * @param kyc_cb KYC status callback to invoke
+   * @param kyc_cb_cls closure for @a kyc_cb
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*account_kyc_get_status)(void *cls,
+                            const char *merchant_id,
+                            const struct TALER_MerchantWireHashP *h_wire,
+                            const char *exchange_url,
+                            TALER_MERCHANTDB_KycCallback kyc_cb,
+                            void *kyc_cb_cls);
+
+  /**
+   * Update an instance's account's KYC status.
+   *
+   * @param cls closure
+   * @param merchant_id merchant backend instance ID
+   * @param h_wire hash of the wire account to check
+   * @param exchange_url base URL of the exchange to check
+   * @param exchange_kyc_serial serial number for our account at the exchange (0 if unknown)
+   * @param exchange_sig signature of the exchange, or NULL for none
+   * @param exchange_pub public key of the exchange, or NULL for none
+   * @param timestamp timestamp to store
+   * @param kyc_ok current KYC status (true for satisfied)
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*account_kyc_set_status)(void *cls,
+                            const char *merchant_id,
+                            const struct TALER_MerchantWireHashP *h_wire,
+                            const char *exchange_url,
+                            uint64_t exchange_kyc_serial,
+                            const struct TALER_ExchangeSignatureP *exchange_sig,
+                            const struct TALER_ExchangePublicKeyP *exchange_pub,
+                            struct GNUNET_TIME_Timestamp timestamp,
+                            bool kyc_ok);
+
 
   /**
    * Lookup all of the products the given instance has configured.
@@ -1065,7 +1201,7 @@ struct TALER_MERCHANTDB_Plugin
                   const char *product_id,
                   const struct GNUNET_Uuid *uuid,
                   uint64_t quantity,
-                  struct GNUNET_TIME_Absolute expiration_time);
+                  struct GNUNET_TIME_Timestamp expiration_time);
 
 
   /**
@@ -1086,13 +1222,15 @@ struct TALER_MERCHANTDB_Plugin
    * @param cls closure
    * @param instance_id instance to delete order of
    * @param order_id order to delete
+   * @param force force deletion of claimed but unpaid orders
    * @return DB status code, #GNUNET_DB_STATUS_SUCCESS_NO_RESULTS
    *           if locks prevent deletion OR order unknown
    */
   enum GNUNET_DB_QueryStatus
   (*delete_order)(void *cls,
                   const char *instance_id,
-                  const char *order_id);
+                  const char *order_id,
+                  bool force);
 
 
   /**
@@ -1113,7 +1251,7 @@ struct TALER_MERCHANTDB_Plugin
                   const char *instance_id,
                   const char *order_id,
                   struct TALER_ClaimTokenP *claim_token,
-                  struct GNUNET_HashCode *h_post_data,
+                  struct TALER_MerchantPostDataHashP *h_post_data,
                   json_t **contract_terms);
 
 
@@ -1131,7 +1269,7 @@ struct TALER_MERCHANTDB_Plugin
   (*lookup_order_summary)(void *cls,
                           const char *instance_id,
                           const char *order_id,
-                          struct GNUNET_TIME_Absolute *timestamp,
+                          struct GNUNET_TIME_Timestamp *timestamp,
                           uint64_t *order_serial);
 
 
@@ -1169,8 +1307,8 @@ struct TALER_MERCHANTDB_Plugin
   (*insert_order)(void *cls,
                   const char *instance_id,
                   const char *order_id,
-                  const struct GNUNET_HashCode *h_post_data,
-                  struct GNUNET_TIME_Absolute pay_deadline,
+                  const struct TALER_MerchantPostDataHashP *h_post_data,
+                  struct GNUNET_TIME_Timestamp pay_deadline,
                   const struct TALER_ClaimTokenP *claim_token,
                   const json_t *contract_terms);
 
@@ -1218,6 +1356,7 @@ struct TALER_MERCHANTDB_Plugin
    * @param order_id order_id used to lookup.
    * @param[out] contract_terms where to store the result, NULL to only check for existence
    * @param[out] order_serial set to the order's serial number
+   * @param[out] paid set to true if the order is fully paid
    * @param[out] claim_token set to the claim token, NULL to only check for existence
    * @return transaction status
    */
@@ -1227,6 +1366,7 @@ struct TALER_MERCHANTDB_Plugin
                            const char *order_id,
                            json_t **contract_terms,
                            uint64_t *order_serial,
+                           bool *paid,
                            struct TALER_ClaimTokenP *claim_token);
 
 
@@ -1242,6 +1382,7 @@ struct TALER_MERCHANTDB_Plugin
    * @param instance_id instance's identifier
    * @param order_id order_id used to store
    * @param claim_token the token belonging to the order
+   * @param[out] order_serial set to the serial of the order
    * @return transaction status, #GNUNET_DB_STATUS_HARD_ERROR if @a contract_terms
    *          is malformed
    */
@@ -1249,7 +1390,8 @@ struct TALER_MERCHANTDB_Plugin
   (*insert_contract_terms)(void *cls,
                            const char *instance_id,
                            const char *order_id,
-                           json_t *contract_terms);
+                           json_t *contract_terms,
+                           uint64_t *order_serial);
 
 
   /**
@@ -1308,7 +1450,7 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*lookup_deposits)(void *cls,
                      const char *instance_id,
-                     const struct GNUNET_HashCode *h_contract_terms,
+                     const struct TALER_PrivateContractHashP *h_contract_terms,
                      TALER_MERCHANTDB_DepositsCallback cb,
                      void *cb_cls);
 
@@ -1329,9 +1471,9 @@ struct TALER_MERCHANTDB_Plugin
     void *cls,
     const struct TALER_MasterPublicKeyP *master_pub,
     const struct TALER_ExchangePublicKeyP *exchange_pub,
-    struct GNUNET_TIME_Absolute start_date,
-    struct GNUNET_TIME_Absolute expire_date,
-    struct GNUNET_TIME_Absolute end_date,
+    struct GNUNET_TIME_Timestamp start_date,
+    struct GNUNET_TIME_Timestamp expire_date,
+    struct GNUNET_TIME_Timestamp end_date,
     const struct TALER_MasterSignatureP *master_sig);
 
 
@@ -1349,21 +1491,21 @@ struct TALER_MERCHANTDB_Plugin
    * @param wire_fee wire fee the exchange charges
    * @param h_wire hash of the wire details of the target account of the merchant
    * @param exchange_sig signature from exchange that coin was accepted
-   * @param exchange_pub signgin key that was used for @a exchange_sig
+   * @param exchange_pub signing key that was used for @a exchange_sig
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
   (*insert_deposit)(void *cls,
                     const char *instance_id,
-                    struct GNUNET_TIME_Absolute deposit_timestamp,
-                    const struct GNUNET_HashCode *h_contract_terms,
+                    struct GNUNET_TIME_Timestamp deposit_timestamp,
+                    const struct TALER_PrivateContractHashP *h_contract_terms,
                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
                     const char *exchange_url,
                     const struct TALER_Amount *amount_with_fee,
                     const struct TALER_Amount *deposit_fee,
                     const struct TALER_Amount *refund_fee,
                     const struct TALER_Amount *wire_fee,
-                    const struct GNUNET_HashCode *h_wire,
+                    const struct TALER_MerchantWireHashP *h_wire,
                     const struct TALER_ExchangeSignatureP *exchange_sig,
                     const struct TALER_ExchangePublicKeyP *exchange_pub);
 
@@ -1381,7 +1523,7 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*lookup_refunds)(void *cls,
                     const char *instance_id,
-                    const struct GNUNET_HashCode *h_contract_terms,
+                    const struct TALER_PrivateContractHashP *h_contract_terms,
                     TALER_MERCHANTDB_RefundCallback rc,
                     void *rc_cls);
 
@@ -1398,10 +1540,11 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*mark_contract_paid)(void *cls,
-                        const char *instance_id,
-                        const struct GNUNET_HashCode *h_contract_terms,
-                        const char *session_id);
+  (*mark_contract_paid)(
+    void *cls,
+    const char *instance_id,
+    const struct TALER_PrivateContractHashP *h_contract_terms,
+    const char *session_id);
 
 
   /**
@@ -1422,8 +1565,8 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*refund_coin)(void *cls,
                  const char *instance_id,
-                 const struct GNUNET_HashCode *h_contract_terms,
-                 struct GNUNET_TIME_Absolute refund_timestamp,
+                 const struct TALER_PrivateContractHashP *h_contract_terms,
+                 struct GNUNET_TIME_Timestamp refund_timestamp,
                  const struct TALER_CoinSpendPublicKeyP *coin_pub,
                  const char *reason);
 
@@ -1442,9 +1585,29 @@ struct TALER_MERCHANTDB_Plugin
   (*lookup_order_status)(void *cls,
                          const char *instance_id,
                          const char *order_id,
-                         struct GNUNET_HashCode *h_contract_terms,
+                         struct TALER_PrivateContractHashP *h_contract_terms,
                          bool *paid);
 
+  /**
+   * Retrieve contract terms given its @a order_serial
+   *
+   * @param cls closure
+   * @param instance_id instance's identifier
+   * @param order_serial serial ID of the order to look up
+   * @param[out] order_id set to ID of the order
+   * @param[out] h_contract_terms set to the hash of the contract.
+   * @param[out] paid set to the payment status of the contract
+   * @return transaction status
+   */
+  enum GNUNET_DB_QueryStatus
+  (*lookup_order_status_by_serial)(void *cls,
+                                   const char *instance_id,
+                                   uint64_t order_serial,
+                                   char **order_id,
+                                   struct TALER_PrivateContractHashP *
+                                   h_contract_terms,
+                                   bool *paid);
+
 
   /**
    * Retrieve payment and wire status for a given @a order_serial and
@@ -1563,11 +1726,12 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*lookup_refunds_detailed)(void *cls,
-                             const char *instance_id,
-                             const struct GNUNET_HashCode *h_contract_terms,
-                             TALER_MERCHANTDB_RefundDetailCallback rc,
-                             void *rc_cls);
+  (*lookup_refunds_detailed)(
+    void *cls,
+    const char *instance_id,
+    const struct TALER_PrivateContractHashP *h_contract_terms,
+    TALER_MERCHANTDB_RefundDetailCallback rc,
+    void *rc_cls);
 
   /**
    * Insert refund proof data from the exchange into the database.
@@ -1723,9 +1887,7 @@ struct TALER_MERCHANTDB_Plugin
    * @param master_pub master public key of the exchange
    * @param h_wire_method hash of wire method
    * @param contract_date date of the contract to use for the lookup
-   * @param[out] wire_fee wire fee charged
-   * @param[out] closing_fee closing fee charged (irrelevant for us,
-   *              but needed to check signature)
+   * @param[out] fees set to wire fees charged
    * @param[out] start_date start of fee being used
    * @param[out] end_date end of fee being used
    * @param[out] master_sig signature of exchange over fee structure
@@ -1735,11 +1897,10 @@ struct TALER_MERCHANTDB_Plugin
   (*lookup_wire_fee)(void *cls,
                      const struct TALER_MasterPublicKeyP *master_pub,
                      const char *wire_method,
-                     struct GNUNET_TIME_Absolute contract_date,
-                     struct TALER_Amount *wire_fee,
-                     struct TALER_Amount *closing_fee,
-                     struct GNUNET_TIME_Absolute *start_date,
-                     struct GNUNET_TIME_Absolute *end_date,
+                     struct GNUNET_TIME_Timestamp contract_date,
+                     struct TALER_WireFeeSet *fees,
+                     struct GNUNET_TIME_Timestamp *start_date,
+                     struct GNUNET_TIME_Timestamp *end_date,
                      struct TALER_MasterSignatureP *master_sig);
 
 
@@ -1759,7 +1920,7 @@ struct TALER_MERCHANTDB_Plugin
   (*lookup_deposits_by_contract_and_coin)(
     void *cls,
     const char *instance_id,
-    const struct GNUNET_HashCode *h_contract_terms,
+    const struct TALER_PrivateContractHashP *h_contract_terms,
     const struct TALER_CoinSpendPublicKeyP *coin_pub,
     TALER_MERCHANTDB_CoinDepositCallback cb,
     void *cb_cls);
@@ -1792,7 +1953,7 @@ struct TALER_MERCHANTDB_Plugin
     struct TALER_Amount *total_amount,
     struct TALER_Amount *wire_fee,
     struct TALER_Amount *exchange_amount,
-    struct GNUNET_TIME_Absolute *execution_time,
+    struct GNUNET_TIME_Timestamp *execution_time,
     bool *have_exchange_sig,
     bool *verified);
 
@@ -1876,8 +2037,8 @@ struct TALER_MERCHANTDB_Plugin
   (*lookup_transfers)(void *cls,
                       const char *instance_id,
                       const char *payto_uri,
-                      struct GNUNET_TIME_Absolute before,
-                      struct GNUNET_TIME_Absolute after,
+                      struct GNUNET_TIME_Timestamp before,
+                      struct GNUNET_TIME_Timestamp after,
                       int64_t limit,
                       uint64_t offset,
                       enum TALER_EXCHANGE_YesNoAll yna,
@@ -1892,9 +2053,7 @@ struct TALER_MERCHANTDB_Plugin
    * @param cls closure
    * @param exchange_pub public key of the exchange
    * @param h_wire_method hash of wire method
-   * @param wire_fee wire fee charged
-   * @param closing_fee closing fee charged (irrelevant for us,
-   *              but needed to check signature)
+   * @param fees wire fees charged
    * @param start_date start of fee being used
    * @param end_date end of fee being used
    * @param exchange_sig signature of exchange over fee structure
@@ -1905,10 +2064,9 @@ struct TALER_MERCHANTDB_Plugin
     void *cls,
     const struct TALER_MasterPublicKeyP *exchange_pub,
     const struct GNUNET_HashCode *h_wire_method,
-    const struct TALER_Amount *wire_fee,
-    const struct TALER_Amount *closing_fee,
-    struct GNUNET_TIME_Absolute start_date,
-    struct GNUNET_TIME_Absolute end_date,
+    const struct TALER_WireFeeSet *fees,
+    struct GNUNET_TIME_Timestamp start_date,
+    struct GNUNET_TIME_Timestamp end_date,
     const struct TALER_MasterSignatureP *exchange_sig);
 
 
@@ -1938,7 +2096,7 @@ struct TALER_MERCHANTDB_Plugin
                     const char *exchange_url,
                     const char *payto_uri,
                     const struct TALER_Amount *initial_balance,
-                    struct GNUNET_TIME_Absolute expiration);
+                    struct GNUNET_TIME_Timestamp expiration);
 
 
   /**
@@ -1976,7 +2134,7 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*lookup_reserves)(void *cls,
                      const char *instance_id,
-                     struct GNUNET_TIME_Absolute created_after,
+                     struct GNUNET_TIME_Timestamp created_after,
                      enum TALER_EXCHANGE_YesNoAll active,
                      enum TALER_EXCHANGE_YesNoAll failures,
                      TALER_MERCHANTDB_ReservesCallback cb,
@@ -2075,8 +2233,8 @@ struct TALER_MERCHANTDB_Plugin
                    const struct TALER_Amount *amount,
                    const char *justification,
                    const char *next_url,
-                   struct GNUNET_HashCode *tip_id,
-                   struct GNUNET_TIME_Absolute *expiration);
+                   struct TALER_TipIdentifierP *tip_id,
+                   struct GNUNET_TIME_Timestamp *expiration);
 
 
   /**
@@ -2096,12 +2254,12 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*lookup_pickup)(void *cls,
                    const char *instance_id,
-                   const struct GNUNET_HashCode *tip_id,
-                   const struct GNUNET_HashCode *pickup_id,
+                   const struct TALER_TipIdentifierP *tip_id,
+                   const struct TALER_PickupIdentifierP *pickup_id,
                    char **exchange_url,
                    struct TALER_ReservePrivateKeyP *reserve_priv,
                    unsigned int sigs_length,
-                   struct GNUNET_CRYPTO_RsaSignature *sigs[]);
+                   struct TALER_BlindedDenominationSignature sigs[]);
 
 
   /**
@@ -2120,10 +2278,10 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*lookup_tip)(void *cls,
                 const char *instance_id,
-                const struct GNUNET_HashCode *tip_id,
+                const struct TALER_TipIdentifierP *tip_id,
                 struct TALER_Amount *total_authorized,
                 struct TALER_Amount *total_picked_up,
-                struct GNUNET_TIME_Absolute *expiration,
+                struct GNUNET_TIME_Timestamp *expiration,
                 char **exchange_url,
                 struct TALER_ReservePrivateKeyP *reserve_priv);
 
@@ -2170,12 +2328,12 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*lookup_tip_details)(void *cls,
                         const char *instance_id,
-                        const struct GNUNET_HashCode *tip_id,
+                        const struct TALER_TipIdentifierP *tip_id,
                         bool fpu,
                         struct TALER_Amount *total_authorized,
                         struct TALER_Amount *total_picked_up,
                         char **justification,
-                        struct GNUNET_TIME_Absolute *expiration,
+                        struct GNUNET_TIME_Timestamp *expiration,
                         struct TALER_ReservePublicKeyP *reserve_pub,
                         unsigned int *pickups_length,
                         struct TALER_MERCHANTDB_PickupDetails **pickups);
@@ -2202,9 +2360,9 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*insert_pickup)(void *cls,
                    const char *instance_id,
-                   const struct GNUNET_HashCode *tip_id,
+                   const struct TALER_TipIdentifierP *tip_id,
                    const struct TALER_Amount *total_picked_up,
-                   const struct GNUNET_HashCode *pickup_id,
+                   const struct TALER_PickupIdentifierP *pickup_id,
                    const struct TALER_Amount *total_requested);
 
 
@@ -2223,9 +2381,9 @@ struct TALER_MERCHANTDB_Plugin
   enum GNUNET_DB_QueryStatus
   (*insert_pickup_blind_signature)(
     void *cls,
-    const struct GNUNET_HashCode *pickup_id,
+    const struct TALER_PickupIdentifierP *pickup_id,
     uint32_t offset,
-    const struct GNUNET_CRYPTO_RsaSignature *blind_sig);
+    const struct TALER_BlindedDenominationSignature *blind_sig);
 
 };
 
