@@ -21,12 +21,8 @@
  */
 #include "platform.h"
 #include <curl/curl.h>
-#include <jansson.h>
-#include <microhttpd.h> /* just for HTTP status codes */
-#include <gnunet/gnunet_util_lib.h>
-#include <gnunet/gnunet_uri_lib.h>
-#include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include <gnunet/gnunet_uri_lib.h>
 #include <taler/taler_json_lib.h>
 
 
