@@ -27,6 +27,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 
@@ -98,26 +99,28 @@ handle_reserve_get_finished (void *cls,
       const char *exchange_url = NULL;
       const char *payto_uri = NULL;
       struct GNUNET_JSON_Specification spec[] = {
-        TALER_JSON_spec_absolute_time ("creation_time",
-                                       &rs.creation_time),
-        TALER_JSON_spec_absolute_time ("expiration_time",
-                                       &rs.expiration_time),
+        GNUNET_JSON_spec_timestamp ("creation_time",
+                                    &rs.creation_time),
+        GNUNET_JSON_spec_timestamp ("expiration_time",
+                                    &rs.expiration_time),
         GNUNET_JSON_spec_bool ("active",
                                &active),
         GNUNET_JSON_spec_mark_optional (
           GNUNET_JSON_spec_string ("exchange_url",
-                                   &exchange_url)),
+                                   &exchange_url),
+          NULL),
         GNUNET_JSON_spec_mark_optional (
           GNUNET_JSON_spec_string ("payto_uri",
-                                   &payto_uri)),
+                                   &payto_uri),
+          NULL),
         TALER_JSON_spec_amount_any ("merchant_initial_amount",
-                                &rs.merchant_initial_amount),
+                                    &rs.merchant_initial_amount),
         TALER_JSON_spec_amount_any ("exchange_initial_amount",
-                                &rs.exchange_initial_amount),
+                                    &rs.exchange_initial_amount),
         TALER_JSON_spec_amount_any ("pickup_amount",
-                                &rs.pickup_amount),
+                                    &rs.pickup_amount),
         TALER_JSON_spec_amount_any ("committed_amount",
-                                &rs.committed_amount),
+                                    &rs.committed_amount),
         GNUNET_JSON_spec_end ()
       };
 
@@ -172,7 +175,7 @@ handle_reserve_get_finished (void *cls,
             GNUNET_JSON_spec_fixed_auto ("tip_id",
                                          &td->tip_id),
             TALER_JSON_spec_amount_any ("total_amount",
-                                    &td->amount),
+                                        &td->amount),
             GNUNET_JSON_spec_string ("reason",
                                      &td->reason),
             GNUNET_JSON_spec_end ()
@@ -288,11 +291,7 @@ TALER_MERCHANT_reserve_get (struct GNUNET_CURL_Context *ctx,
     GNUNET_free (rgh);
     return NULL;
   }
-  eh = curl_easy_init ();
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   rgh->url));
+  eh = TALER_MERCHANT_curl_easy_get_ (rgh->url);
   rgh->job = GNUNET_CURL_job_add (ctx,
                                   eh,
                                   &handle_reserve_get_finished,
