@@ -27,6 +27,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 
@@ -134,18 +135,18 @@ handle_reserves_get_finished (void *cls,
           struct GNUNET_JSON_Specification ispec[] = {
             GNUNET_JSON_spec_fixed_auto ("reserve_pub",
                                          &rd->reserve_pub),
-            TALER_JSON_spec_absolute_time ("creation_time",
-                                           &rd->creation_time),
-            TALER_JSON_spec_absolute_time ("expiration_time",
-                                           &rd->expiration_time),
+            GNUNET_JSON_spec_timestamp ("creation_time",
+                                        &rd->creation_time),
+            GNUNET_JSON_spec_timestamp ("expiration_time",
+                                        &rd->expiration_time),
             TALER_JSON_spec_amount_any ("merchant_initial_amount",
-                                    &rd->merchant_initial_amount),
+                                        &rd->merchant_initial_amount),
             TALER_JSON_spec_amount_any ("exchange_initial_amount",
-                                    &rd->exchange_initial_amount),
+                                        &rd->exchange_initial_amount),
             TALER_JSON_spec_amount_any ("pickup_amount",
-                                    &rd->pickup_amount),
+                                        &rd->pickup_amount),
             TALER_JSON_spec_amount_any ("committed_amount",
-                                    &rd->committed_amount),
+                                        &rd->committed_amount),
             GNUNET_JSON_spec_bool ("active",
                                    &rd->active),
             GNUNET_JSON_spec_end ()
@@ -216,7 +217,7 @@ handle_reserves_get_finished (void *cls,
 struct TALER_MERCHANT_ReservesGetHandle *
 TALER_MERCHANT_reserves_get (struct GNUNET_CURL_Context *ctx,
                              const char *backend_url,
-                             struct GNUNET_TIME_Absolute after,
+                             struct GNUNET_TIME_Timestamp after,
                              enum TALER_EXCHANGE_YesNoAll active,
                              enum TALER_EXCHANGE_YesNoAll failures,
                              TALER_MERCHANT_ReservesGetCallback cb,
@@ -234,8 +235,7 @@ TALER_MERCHANT_reserves_get (struct GNUNET_CURL_Context *ctx,
   rgh->cb_cls = cb_cls;
   active_s = TALER_yna_to_string (active);
   failures_s = TALER_yna_to_string (failures);
-  after_s = GNUNET_strdup (GNUNET_STRINGS_absolute_time_to_string (
-                             after));
+  after_s = GNUNET_strdup (GNUNET_TIME_timestamp2s (after));
   rgh->url = TALER_url_join (backend_url,
                              "private/reserves",
                              "active",
@@ -243,9 +243,9 @@ TALER_MERCHANT_reserves_get (struct GNUNET_CURL_Context *ctx,
                              "failures",
                              failures_s,
                              "after",
-                             after.abs_value_us != 0
-                             ? after_s
-                             : NULL,
+                             GNUNET_TIME_absolute_is_zero (after.abs_time)
+                             ? NULL
+                             : after_s,
                              NULL);
   GNUNET_free (after_s);
   if (NULL == rgh->url)
@@ -255,11 +255,7 @@ TALER_MERCHANT_reserves_get (struct GNUNET_CURL_Context *ctx,
     GNUNET_free (rgh);
     return NULL;
   }
-  eh = curl_easy_init ();
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   rgh->url));
+  eh = TALER_MERCHANT_curl_easy_get_ (rgh->url);
   rgh->job = GNUNET_CURL_job_add (ctx,
                                   eh,
                                   &handle_reserves_get_finished,
