@@ -27,6 +27,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 
@@ -133,7 +134,7 @@ handle_transfers_get_finished (void *cls,
           struct TALER_MERCHANT_TransferData *td = &tds[i];
           struct GNUNET_JSON_Specification ispec[] = {
             TALER_JSON_spec_amount_any ("credit_amount",
-                                    &td->credit_amount),
+                                        &td->credit_amount),
             GNUNET_JSON_spec_fixed_auto ("wtid",
                                          &td->wtid),
             GNUNET_JSON_spec_string ("payto_uri",
@@ -143,14 +144,17 @@ handle_transfers_get_finished (void *cls,
             GNUNET_JSON_spec_uint64 ("transfer_serial_id",
                                      &td->credit_serial),
             GNUNET_JSON_spec_mark_optional (
-              TALER_JSON_spec_absolute_time ("execution_time",
-                                             &td->execution_time)),
+              GNUNET_JSON_spec_timestamp ("execution_time",
+                                          &td->execution_time),
+              NULL),
             GNUNET_JSON_spec_mark_optional (
               GNUNET_JSON_spec_bool ("verified",
-                                     &td->verified)),
+                                     &td->verified),
+              NULL),
             GNUNET_JSON_spec_mark_optional (
               GNUNET_JSON_spec_bool ("confirmed",
-                                     &td->confirmed)),
+                                     &td->confirmed),
+              NULL),
             GNUNET_JSON_spec_end ()
           };
 
@@ -227,8 +231,8 @@ TALER_MERCHANT_transfers_get (
   struct GNUNET_CURL_Context *ctx,
   const char *backend_url,
   const char *payto_uri,
-  const struct GNUNET_TIME_Absolute before,
-  const struct GNUNET_TIME_Absolute after,
+  const struct GNUNET_TIME_Timestamp before,
+  const struct GNUNET_TIME_Timestamp after,
   int64_t limit,
   uint64_t offset,
   enum TALER_EXCHANGE_YesNoAll verified,
@@ -256,8 +260,8 @@ TALER_MERCHANT_transfers_get (
                    sizeof (offset_s),
                    "%lld",
                    (unsigned long long) offset);
-  before_s = GNUNET_strdup (GNUNET_STRINGS_absolute_time_to_string (before));
-  after_s = GNUNET_strdup (GNUNET_STRINGS_absolute_time_to_string (after));
+  before_s = GNUNET_strdup (GNUNET_TIME_timestamp2s (before));
+  after_s = GNUNET_strdup (GNUNET_TIME_timestamp2s (after));
   gth->url = TALER_url_join (backend_url,
                              "private/transfers",
                              "payto_uri",
@@ -275,14 +279,13 @@ TALER_MERCHANT_transfers_get (
                              ? offset_s
                              : NULL,
                              "before",
-                             before.abs_value_us !=
-                             GNUNET_TIME_UNIT_FOREVER_ABS.abs_value_us
-                             ? before_s
-                             : NULL,
+                             GNUNET_TIME_absolute_is_never (before.abs_time)
+                             ? NULL
+                             : before_s,
                              "after",
-                             after.abs_value_us != 0
-                             ? after_s
-                             : NULL,
+                             GNUNET_TIME_absolute_is_zero (after.abs_time)
+                             ? NULL
+                             : after_s,
                              NULL);
   GNUNET_free (before_s);
   GNUNET_free (after_s);
@@ -293,11 +296,7 @@ TALER_MERCHANT_transfers_get (
     GNUNET_free (gth);
     return NULL;
   }
-  eh = curl_easy_init ();
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   gth->url));
+  eh = TALER_MERCHANT_curl_easy_get_ (gth->url);
   gth->job = GNUNET_CURL_job_add (ctx,
                                   eh,
                                   &handle_transfers_get_finished,
