@@ -28,6 +28,7 @@
 #include <microhttpd.h> /* just for HTTP status codes */
 #include <gnunet/gnunet_util_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_curl_lib.h>
 
@@ -242,23 +243,20 @@ TALER_MERCHANT_instance_patch (
   {
     CURL *eh;
 
-    eh = curl_easy_init ();
+    eh = TALER_MERCHANT_curl_easy_get_ (iph->url);
     if (GNUNET_OK !=
         TALER_curl_easy_post (&iph->post_ctx,
                               eh,
                               req_obj))
     {
       GNUNET_break (0);
+      curl_easy_cleanup (eh);
       json_decref (req_obj);
       GNUNET_free (iph->url);
       GNUNET_free (iph);
       return NULL;
     }
-
     json_decref (req_obj);
-    GNUNET_assert (CURLE_OK == curl_easy_setopt (eh,
-                                                 CURLOPT_URL,
-                                                 iph->url));
     GNUNET_assert (CURLE_OK ==
                    curl_easy_setopt (eh,
                                      CURLOPT_CUSTOMREQUEST,
