@@ -28,6 +28,7 @@
 #include <microhttpd.h> /* just for HTTP status codes */
 #include <gnunet/gnunet_util_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_curl_lib.h>
 
@@ -168,14 +169,13 @@ TALER_MERCHANT_product_patch (
   int64_t total_stock,
   uint64_t total_lost,
   const json_t *address,
-  struct GNUNET_TIME_Absolute next_restock,
+  struct GNUNET_TIME_Timestamp next_restock,
   TALER_MERCHANT_ProductPatchCallback cb,
   void *cb_cls)
 {
   struct TALER_MERCHANT_ProductPatchHandle *pph;
   json_t *req_obj;
 
-  (void) GNUNET_TIME_round_abs (&next_restock);
   req_obj = GNUNET_JSON_PACK (
     GNUNET_JSON_pack_string ("description",
                              description),
@@ -195,8 +195,8 @@ TALER_MERCHANT_product_patch (
                              total_lost),
     GNUNET_JSON_pack_object_incref ("address",
                                     (json_t *) address),
-    GNUNET_JSON_pack_time_abs ("next_restock",
-                               next_restock));
+    GNUNET_JSON_pack_timestamp ("next_restock",
+                                next_restock));
   pph = GNUNET_new (struct TALER_MERCHANT_ProductPatchHandle);
   pph->ctx = ctx;
   pph->cb = cb;
@@ -223,22 +223,19 @@ TALER_MERCHANT_product_patch (
   {
     CURL *eh;
 
-    eh = curl_easy_init ();
+    eh = TALER_MERCHANT_curl_easy_get_ (pph->url);
     if (GNUNET_OK !=
         TALER_curl_easy_post (&pph->post_ctx,
                               eh,
                               req_obj))
     {
       GNUNET_break (0);
+      curl_easy_cleanup (eh);
       json_decref (req_obj);
       GNUNET_free (pph);
       return NULL;
     }
-
     json_decref (req_obj);
-    GNUNET_assert (CURLE_OK == curl_easy_setopt (eh,
-                                                 CURLOPT_URL,
-                                                 pph->url));
     GNUNET_assert (CURLE_OK ==
                    curl_easy_setopt (eh,
                                      CURLOPT_CUSTOMREQUEST,
