@@ -30,6 +30,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_exchange_service.h>
@@ -44,7 +45,7 @@ struct TALER_MERCHANT_OrderAbortHandle
   /**
    * Hash of the contract.
    */
-  struct GNUNET_HashCode h_contract_terms;
+  struct TALER_PrivateContractHashP h_contract_terms;
 
   /**
    * Public key of the merchant.
@@ -171,29 +172,19 @@ check_abort_refund (struct TALER_MERCHANT_OrderAbortHandle *oah,
           return GNUNET_SYSERR;
         }
 
+        if (GNUNET_OK !=
+            TALER_exchange_online_refund_confirmation_verify (
+              &oah->h_contract_terms,
+              &oah->coins[i].coin_pub,
+              &oah->merchant_pub,
+              0,                                                   /* transaction id */
+              &oah->coins[i].amount_with_fee,
+              &res[i].exchange_pub,
+              &res[i].exchange_sig))
         {
-          struct TALER_RefundConfirmationPS rr = {
-            .purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_REFUND),
-            .purpose.size = htonl (sizeof (rr)),
-            .h_contract_terms = oah->h_contract_terms,
-            .coin_pub = oah->coins[i].coin_pub,
-            .merchant = oah->merchant_pub,
-            .rtransaction_id = GNUNET_htonll (0)
-          };
-
-          TALER_amount_hton (&rr.refund_amount,
-                             &oah->coins[i].amount_with_fee);
-          if (GNUNET_OK !=
-              GNUNET_CRYPTO_eddsa_verify (
-                TALER_SIGNATURE_EXCHANGE_CONFIRM_REFUND,
-                &rr,
-                &res[i].exchange_sig.eddsa_signature,
-                &res[i].exchange_pub.eddsa_pub))
-          {
-            GNUNET_break_op (0);
-            GNUNET_JSON_parse_free (spec);
-            return GNUNET_SYSERR;
-          }
+          GNUNET_break_op (0);
+          GNUNET_JSON_parse_free (spec);
+          return GNUNET_SYSERR;
         }
       }
     }
@@ -326,7 +317,7 @@ TALER_MERCHANT_order_abort (struct GNUNET_CURL_Context *ctx,
                             const char *merchant_url,
                             const char *order_id,
                             const struct TALER_MerchantPublicKeyP *merchant_pub,
-                            const struct GNUNET_HashCode *h_contract,
+                            const struct TALER_PrivateContractHashP *h_contract,
                             unsigned int num_coins,
                             const struct TALER_MERCHANT_AbortCoin coins[],
                             TALER_MERCHANT_AbortCallback cb,
@@ -403,23 +394,19 @@ TALER_MERCHANT_order_abort (struct GNUNET_CURL_Context *ctx,
   {
     CURL *eh;
 
-    eh = curl_easy_init ();
-    GNUNET_assert (NULL != eh);
+    eh = TALER_MERCHANT_curl_easy_get_ (oah->url);
     if (GNUNET_OK !=
         TALER_curl_easy_post (&oah->post_ctx,
                               eh,
                               abort_obj))
     {
       GNUNET_break (0);
+      curl_easy_cleanup (eh);
       json_decref (abort_obj);
       GNUNET_free (oah);
       return NULL;
     }
     json_decref (abort_obj);
-    GNUNET_assert (CURLE_OK ==
-                   curl_easy_setopt (eh,
-                                     CURLOPT_URL,
-                                     oah->url));
     oah->job = GNUNET_CURL_job_add2 (ctx,
                                      eh,
                                      oah->post_ctx.headers,
