@@ -29,6 +29,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_exchange_service.h>
@@ -165,7 +166,7 @@ TALER_MERCHANT_order_paid (
   const char *merchant_url,
   const char *order_id,
   const char *session_id,
-  const struct GNUNET_HashCode *h_contract_terms,
+  const struct TALER_PrivateContractHashP *h_contract_terms,
   const struct TALER_MerchantSignatureP *merchant_sig,
   TALER_MERCHANT_OrderPaidCallback paid_cb,
   void *paid_cb_cls)
@@ -206,22 +207,19 @@ TALER_MERCHANT_order_paid (
   {
     CURL *eh;
 
-    eh = curl_easy_init ();
+    eh = TALER_MERCHANT_curl_easy_get_ (oph->url);
     if (GNUNET_OK !=
         TALER_curl_easy_post (&oph->post_ctx,
                               eh,
                               req_obj))
     {
       GNUNET_break (0);
+      curl_easy_cleanup (eh);
       json_decref (req_obj);
       GNUNET_free (oph);
       return NULL;
     }
-
     json_decref (req_obj);
-    GNUNET_assert (CURLE_OK == curl_easy_setopt (eh,
-                                                 CURLOPT_URL,
-                                                 oph->url));
     oph->job = GNUNET_CURL_job_add2 (ctx,
                                      eh,
                                      oph->post_ctx.headers,
