@@ -30,6 +30,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_exchange_service.h>
@@ -81,7 +82,7 @@ struct TALER_MERCHANT_OrderPayHandle
    * Hash of the contract we are paying, set
    * if @e am_wallet is true.
    */
-  struct GNUNET_HashCode h_contract_terms;
+  struct TALER_PrivateContractHashP h_contract_terms;
 
   /**
    * Public key of the merchant (instance) being paid, set
@@ -89,6 +90,29 @@ struct TALER_MERCHANT_OrderPayHandle
    */
   struct TALER_MerchantPublicKeyP merchant_pub;
 
+  /**
+   * JSON with the full reply, used during async
+   * processing.
+   */
+  json_t *full_reply;
+
+  /**
+   * Pointer into @e coins array for the coin that
+   * created a conflict (that we are checking).
+   */
+  const struct TALER_MERCHANT_PaidCoin *error_pc;
+
+  /**
+   * Coin history that proves a conflict.
+   */
+  json_t *error_history;
+
+  /**
+   * Handle to the exchange that issued a problematic
+   * coin (if any).
+   */
+  struct TALER_EXCHANGE_Handle *exchange;
+
   /**
    * Number of @e coins we are paying with.
    */
@@ -105,29 +129,31 @@ struct TALER_MERCHANT_OrderPayHandle
 
 /**
  * We got a 409 response back from the exchange (or the merchant).
- * Now we need to check the provided cryptograophic proof that the
+ * Now we need to check the provided cryptographic proof that the
  * coin was actually already spent!
  *
- * @param pc handle of the original coin we paid with
- * @param json cryptograophic proof of coin's transaction
- *        history as was returned by the exchange/merchant
- * @return #GNUNET_OK if proof checks out
+ * @param oph operation handle
+ * @param keys key data from the exchange
+ * @return #GNUNET_OK if conflict is valid
  */
-static int
-check_coin_history (const struct TALER_MERCHANT_PaidCoin *pc,
-                    json_t *json)
+static enum GNUNET_GenericReturnValue
+check_conflict (struct TALER_MERCHANT_OrderPayHandle *oph,
+                const struct TALER_EXCHANGE_Keys *keys)
 {
   struct TALER_Amount spent;
   struct TALER_Amount spent_plus_contrib;
-  struct GNUNET_HashCode h_denom_pub;
-  struct GNUNET_HashCode h_denom_pub_pc;
+  struct TALER_DenominationHashP h_denom_pub_pc;
+  const struct TALER_EXCHANGE_DenomPublicKey *dpk;
 
+  TALER_denom_pub_hash (&oph->error_pc->denom_pub,
+                        &h_denom_pub_pc);
+  dpk = TALER_EXCHANGE_get_denomination_key_by_hash (
+    keys,
+    &h_denom_pub_pc);
   if (GNUNET_OK !=
-      TALER_EXCHANGE_verify_coin_history (NULL, /* do not verify fees */
-                                          pc->amount_with_fee.currency,
-                                          &pc->coin_pub,
-                                          json,
-                                          &h_denom_pub,
+      TALER_EXCHANGE_verify_coin_history (dpk,
+                                          &oph->error_pc->coin_pub,
+                                          oph->error_history,
                                           &spent))
   {
     /* Exchange's history fails to verify */
@@ -137,18 +163,14 @@ check_coin_history (const struct TALER_MERCHANT_PaidCoin *pc,
   if (0 >
       TALER_amount_add (&spent_plus_contrib,
                         &spent,
-                        &pc->amount_with_fee))
+                        &oph->error_pc->amount_with_fee))
   {
     /* We got an integer overflow? Bad application! */
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  GNUNET_CRYPTO_rsa_public_key_hash (pc->denom_pub.rsa_public_key,
-                                     &h_denom_pub_pc);
-  if ( (-1 != TALER_amount_cmp (&pc->denom_value,
-                                &spent_plus_contrib)) &&
-       (0 != GNUNET_memcmp (&h_denom_pub,
-                            &h_denom_pub_pc)) )
+  if (-1 != TALER_amount_cmp (&oph->error_pc->denom_value,
+                              &spent_plus_contrib))
   {
     /* according to our calculations, the transaction should
        have still worked, AND we did not get any proof of
@@ -162,32 +184,148 @@ check_coin_history (const struct TALER_MERCHANT_PaidCoin *pc,
 }
 
 
+/**
+ * We got the fee structure from the exchange. Now
+ * validate the conflict error.
+ *
+ * @param cls a `struct TALER_MERCHANT_OrderPayHandle`
+ * @param ehr reply from the exchange
+ * @param keys the key structure
+ * @param compat protocol compatibility indication
+ */
+static void
+cert_cb (void *cls,
+         const struct TALER_EXCHANGE_HttpResponse *ehr,
+         const struct TALER_EXCHANGE_Keys *keys,
+         enum TALER_EXCHANGE_VersionCompatibility compat)
+{
+  struct TALER_MERCHANT_OrderPayHandle *oph = cls;
+
+  if (TALER_EXCHANGE_VC_INCOMPATIBLE & compat)
+  {
+    struct TALER_MERCHANT_PayResponse pr = {
+      .hr.http_status = MHD_HTTP_CONFLICT,
+      .hr.exchange_http_status = 0,
+      .hr.ec = TALER_EC_WALLET_EXCHANGE_PROTOCOL_VERSION_INCOMPATIBLE,
+      .hr.reply = oph->full_reply,
+      .hr.exchange_reply = ehr->reply,
+      .hr.hint = "could not check error: incompatible exchange version"
+    };
+
+    oph->pay_cb (oph->pay_cb_cls,
+                 &pr);
+    TALER_MERCHANT_order_pay_cancel (oph);
+    return;
+  }
+  if ( (MHD_HTTP_OK != ehr->http_status) ||
+       (NULL == keys) )
+  {
+    struct TALER_MERCHANT_PayResponse pr = {
+      .hr.http_status = MHD_HTTP_CONFLICT,
+      .hr.exchange_http_status = ehr->http_status,
+      .hr.ec = TALER_EC_EXCHANGE_GENERIC_KEYS_MISSING,
+      .hr.reply = oph->full_reply,
+      .hr.exchange_reply = ehr->reply,
+      .hr.hint = "failed to download /keys from the exchange"
+    };
+
+    oph->pay_cb (oph->pay_cb_cls,
+                 &pr);
+    TALER_MERCHANT_order_pay_cancel (oph);
+    return;
+  }
+
+  if (GNUNET_OK !=
+      check_conflict (oph,
+                      keys))
+  {
+    struct TALER_MERCHANT_PayResponse pr = {
+      .hr.http_status = 0,
+      .hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE,
+      .hr.reply = oph->full_reply
+    };
+
+    oph->pay_cb (oph->pay_cb_cls,
+                 &pr);
+    TALER_MERCHANT_order_pay_cancel (oph);
+    return;
+  }
+
+  {
+    struct TALER_MERCHANT_PayResponse pr = {
+      .hr.http_status = MHD_HTTP_CONFLICT,
+      .hr.ec = TALER_JSON_get_error_code (oph->full_reply),
+      .hr.reply = oph->full_reply
+    };
+
+    oph->pay_cb (oph->pay_cb_cls,
+                 &pr);
+    TALER_MERCHANT_order_pay_cancel (oph);
+  }
+}
+
+
 /**
  * We got a 409 response back from the exchange (or the merchant).
  * Now we need to check the provided cryptograophic proof that the
  * coin was actually already spent!
  *
- * @param oph handle of the original pay operation
+ * @param[in,out] oph handle of the original pay operation
+ * @param[in,out] pr response to modify if #GNUNET_OK is returned
  * @param json cryptograophic proof returned by the
  *        exchange/merchant
- * @return #GNUNET_OK if proof checks out
+ * @return #GNUNET_OK if proof checks out,
+ *         #GNUNET_SYSERR if it is wrong,
+ *         #GNUNET_NO if checking continues asynchronously
  */
-static int
-check_conflict (struct TALER_MERCHANT_OrderPayHandle *oph,
+static enum GNUNET_GenericReturnValue
+parse_conflict (struct TALER_MERCHANT_OrderPayHandle *oph,
+                struct TALER_MERCHANT_PayResponse *pr,
                 const json_t *json)
 {
-  json_t *history;
   json_t *ereply;
-  struct TALER_CoinSpendPublicKeyP coin_pub;
+  const char *exchange_url;
   struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_json ("exchange_reply", &ereply),
-    GNUNET_JSON_spec_fixed_auto ("coin_pub", &coin_pub),
+    GNUNET_JSON_spec_json ("exchange_reply",
+                           &ereply),
+    GNUNET_JSON_spec_string ("exchange_url",
+                             &exchange_url),
     GNUNET_JSON_spec_end ()
   };
+  struct TALER_CoinSpendPublicKeyP coin_pub;
   struct GNUNET_JSON_Specification hspec[] = {
-    GNUNET_JSON_spec_json ("history", &history),
+    GNUNET_JSON_spec_json ("history",
+                           &oph->error_history),
+    GNUNET_JSON_spec_fixed_auto ("coin_pub",
+                                 &coin_pub),
     GNUNET_JSON_spec_end ()
   };
+  enum TALER_ErrorCode ec = TALER_JSON_get_error_code (json);
+
+  switch (ec)
+  {
+  case TALER_EC_GENERIC_CURRENCY_MISMATCH:
+    /* no proof to check, still very strange, as we
+       should have checked that the currency matches */
+    GNUNET_break_op (0);
+    TALER_MERCHANT_parse_error_details_ (json,
+                                         MHD_HTTP_CONFLICT,
+                                         &pr->hr);
+    return GNUNET_OK;
+  case TALER_EC_MERCHANT_POST_ORDERS_ID_PAY_ALREADY_PAID:
+    /* We can only be happy and accept the result;
+       FIXME: parse the refunds... */
+    TALER_MERCHANT_parse_error_details_ (json,
+                                         MHD_HTTP_CONFLICT,
+                                         &pr->hr);
+    return GNUNET_OK;
+  case TALER_EC_MERCHANT_POST_ORDERS_ID_PAY_INSUFFICIENT_FUNDS:
+    /* main case, handled below */
+    break;
+  default:
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
 
   if (GNUNET_OK !=
       GNUNET_JSON_parse (json,
@@ -210,16 +348,18 @@ check_conflict (struct TALER_MERCHANT_OrderPayHandle *oph,
 
   for (unsigned int i = 0; i<oph->num_coins; i++)
   {
-    if (0 == memcmp (&oph->coins[i].coin_pub,
-                     &coin_pub,
-                     sizeof (struct TALER_CoinSpendPublicKeyP)))
+    if (0 ==
+        GNUNET_memcmp (&oph->coins[i].coin_pub,
+                       &coin_pub))
     {
-      int ret;
-
-      ret = check_coin_history (&oph->coins[i],
-                                history);
-      GNUNET_JSON_parse_free (hspec);
-      return ret;
+      oph->error_pc = &oph->coins[i];
+      oph->full_reply = json_incref ((json_t *) json);
+      oph->exchange = TALER_EXCHANGE_connect (oph->ctx,
+                                              oph->error_pc->exchange_url,
+                                              &cert_cb,
+                                              oph,
+                                              TALER_EXCHANGE_OPTION_END);
+      return GNUNET_NO;
     }
   }
   GNUNET_break_op (0); /* complaint is not about any of the coins
@@ -244,12 +384,10 @@ handle_pay_finished (void *cls,
 {
   struct TALER_MERCHANT_OrderPayHandle *oph = cls;
   const json_t *json = response;
-  struct TALER_MERCHANT_HttpResponse hr = {
-    .http_status = (unsigned int) response_code,
-    .reply = json
+  struct TALER_MERCHANT_PayResponse pr = {
+    .hr.http_status = (unsigned int) response_code,
+    .hr.reply = json
   };
-  struct TALER_MerchantSignatureP *merchant_sigp = NULL;
-  struct TALER_MerchantSignatureP merchant_sig;
 
   oph->job = NULL;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -258,20 +396,16 @@ handle_pay_finished (void *cls,
   switch (response_code)
   {
   case 0:
-    hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+    pr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     if (oph->am_wallet)
     {
       /* Here we can (and should) verify the merchant's signature */
-      struct TALER_PaymentResponsePS pr = {
-        .purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_PAYMENT_OK),
-        .purpose.size = htonl (sizeof (pr)),
-        .h_contract_terms = oph->h_contract_terms
-      };
       struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_fixed_auto ("sig",
-                                     &merchant_sig),
+        GNUNET_JSON_spec_fixed_auto (
+          "sig",
+          &pr.details.success.merchant_sig),
         GNUNET_JSON_spec_end ()
       };
 
@@ -281,95 +415,107 @@ handle_pay_finished (void *cls,
                              NULL, NULL))
       {
         GNUNET_break_op (0);
-        hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
-        hr.http_status = 0;
-        hr.hint = "sig field missing in response";
+        pr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+        pr.hr.http_status = 0;
+        pr.hr.hint = "sig field missing in response";
         break;
       }
 
       if (GNUNET_OK !=
-          GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_PAYMENT_OK,
-                                      &pr,
-                                      &merchant_sig.eddsa_sig,
-                                      &oph->merchant_pub.eddsa_pub))
+          TALER_merchant_pay_verify (&oph->h_contract_terms,
+                                     &oph->merchant_pub,
+                                     &pr.details.success.merchant_sig))
       {
         GNUNET_break_op (0);
-        hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
-        hr.http_status = 0;
-        hr.hint = "signature invalid";
+        pr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+        pr.hr.http_status = 0;
+        pr.hr.hint = "signature invalid";
       }
-      merchant_sigp = &merchant_sig;
     }
     break;
   /* Tolerating Not Acceptable because sometimes
    * - especially in tests - we might want to POST
    * coins one at a time.  */
   case MHD_HTTP_NOT_ACCEPTABLE:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    pr.hr.ec = TALER_JSON_get_error_code (json);
+    pr.hr.hint = TALER_JSON_get_error_hint (json);
     break;
   case MHD_HTTP_BAD_REQUEST:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    pr.hr.ec = TALER_JSON_get_error_code (json);
+    pr.hr.hint = TALER_JSON_get_error_hint (json);
     /* This should never happen, either us
      * or the merchant is buggy (or API version conflict);
      * just pass JSON reply to the application */
     break;
   case MHD_HTTP_PAYMENT_REQUIRED:
     /* was originally paid, but then refunded */
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    pr.hr.ec = TALER_JSON_get_error_code (json);
+    pr.hr.hint = TALER_JSON_get_error_hint (json);
     break;
   case MHD_HTTP_FORBIDDEN:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    pr.hr.ec = TALER_JSON_get_error_code (json);
+    pr.hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, merchant says we tried to abort the payment
      * after it was successful. We should pass the JSON reply to the
      * application */
     break;
   case MHD_HTTP_NOT_FOUND:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    pr.hr.ec = TALER_JSON_get_error_code (json);
+    pr.hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the
        application */
     break;
-  case MHD_HTTP_PRECONDITION_FAILED:
-    TALER_MERCHANT_parse_error_details_ (json,
-                                         response_code,
-                                         &hr);
-    /* Nothing really to verify, the merchant is blaming us for failing to
-       satisfy some constraint (likely it does not like our exchange because
-       of some disagreement on the PKI).  We should pass the JSON reply to the
-       application */
-    break;
   case MHD_HTTP_REQUEST_TIMEOUT:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    pr.hr.ec = TALER_JSON_get_error_code (json);
+    pr.hr.hint = TALER_JSON_get_error_hint (json);
     /* The merchant couldn't generate a timely response, likely because
        it itself waited too long on the exchange.
        Pass on to application. */
     break;
   case MHD_HTTP_CONFLICT:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
-    if (GNUNET_OK != check_conflict (oph,
-                                     json))
     {
-      GNUNET_break_op (0);
-      response_code = 0;
+      enum GNUNET_GenericReturnValue ret;
+
+      ret = parse_conflict (oph,
+                            &pr,
+                            json);
+      switch (ret)
+      {
+      case GNUNET_OK:
+        /* continued below, 'pr' was modified */
+        break;
+      case GNUNET_NO:
+        /* handled asynchronously! */
+        return; /* ! */
+      case GNUNET_SYSERR:
+        GNUNET_break_op (0);
+        response_code = 0;
+        break;
+      }
+      break;
     }
-    break;
   case MHD_HTTP_GONE:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    TALER_MERCHANT_parse_error_details_ (json,
+                                         response_code,
+                                         &pr.hr);
     /* The merchant says we are too late, the offer has expired or some
        denomination key of a coin involved has expired.
        Might be a disagreement in timestamps? Still, pass on to application. */
     break;
+  case MHD_HTTP_PRECONDITION_FAILED:
+    TALER_MERCHANT_parse_error_details_ (json,
+                                         response_code,
+                                         &pr.hr);
+    /* Nothing really to verify, the merchant is blaming us for failing to
+       satisfy some constraint (likely it does not like our exchange because
+       of some disagreement on the PKI).  We should pass the JSON reply to the
+       application */
+    break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    TALER_MERCHANT_parse_error_details_ (json,
+                                         response_code,
+                                         &pr.hr);
     /* Server had an internal issue; we should retry,
        but this API leaves this to the application */
     break;
@@ -378,37 +524,36 @@ handle_pay_finished (void *cls,
        We should pass the JSON reply to the application */
     TALER_MERCHANT_parse_error_details_ (json,
                                          response_code,
-                                         &hr);
+                                         &pr.hr);
     break;
   case MHD_HTTP_SERVICE_UNAVAILABLE:
     TALER_MERCHANT_parse_error_details_ (json,
                                          response_code,
-                                         &hr);
+                                         &pr.hr);
     /* Exchange couldn't respond properly; the retry is
        left to the application */
     break;
   case MHD_HTTP_GATEWAY_TIMEOUT:
     TALER_MERCHANT_parse_error_details_ (json,
                                          response_code,
-                                         &hr);
+                                         &pr.hr);
     /* Exchange couldn't respond in a timely fashion;
        the retry is left to the application */
     break;
   default:
     TALER_MERCHANT_parse_error_details_ (json,
                                          response_code,
-                                         &hr);
+                                         &pr.hr);
     /* unexpected response code */
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d\n",
                 (unsigned int) response_code,
-                (int) hr.ec);
+                (int) pr.hr.ec);
     GNUNET_break_op (0);
     break;
   }
   oph->pay_cb (oph->pay_cb_cls,
-               &hr,
-               merchant_sigp);
+               &pr);
   TALER_MERCHANT_order_pay_cancel (oph);
 }
 
@@ -437,12 +582,13 @@ TALER_MERCHANT_order_pay_frontend (
     return NULL;
   }
   j_coins = json_array ();
+  GNUNET_assert (NULL != j_coins);
   for (unsigned int i = 0; i<num_coins; i++)
   {
     json_t *j_coin;
     const struct TALER_MERCHANT_PaidCoin *pc = &coins[i];
     struct TALER_Amount fee;
-    struct GNUNET_HashCode denom_hash;
+    struct TALER_DenominationHashP denom_hash;
 
     if (0 >
         TALER_amount_subtract (&fee,
@@ -478,8 +624,8 @@ TALER_MERCHANT_order_pay_frontend (
       }
     }
 
-    GNUNET_CRYPTO_rsa_public_key_hash (pc->denom_pub.rsa_public_key,
-                                       &denom_hash);
+    TALER_denom_pub_hash (&pc->denom_pub,
+                          &denom_hash);
     /* create JSON for this coin */
     j_coin = GNUNET_JSON_PACK (
       TALER_JSON_pack_amount ("contribution",
@@ -490,8 +636,8 @@ TALER_MERCHANT_order_pay_frontend (
                                pc->exchange_url),
       GNUNET_JSON_pack_data_auto ("h_denom",
                                   &denom_hash),
-      TALER_JSON_pack_denomination_signature ("ub_sig",
-                                              &pc->denom_sig),
+      TALER_JSON_pack_denom_sig ("ub_sig",
+                                 &pc->denom_sig),
       GNUNET_JSON_pack_data_auto ("coin_sig",
                                   &pc->coin_sig));
     if (0 !=
@@ -541,23 +687,20 @@ TALER_MERCHANT_order_pay_frontend (
           coins,
           num_coins * sizeof (struct TALER_MERCHANT_PaidCoin));
 
-  eh = curl_easy_init ();
+  eh = TALER_MERCHANT_curl_easy_get_ (oph->url);
   if (GNUNET_OK !=
       TALER_curl_easy_post (&oph->post_ctx,
                             eh,
                             pay_obj))
   {
     GNUNET_break (0);
+    curl_easy_cleanup (eh);
     json_decref (pay_obj);
+    GNUNET_free (oph->url);
     GNUNET_free (oph);
     return NULL;
   }
-
   json_decref (pay_obj);
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   oph->url));
   oph->job = GNUNET_CURL_job_add2 (ctx,
                                    eh,
                                    oph->post_ctx.headers,
@@ -568,27 +711,25 @@ TALER_MERCHANT_order_pay_frontend (
 
 
 struct TALER_MERCHANT_OrderPayHandle *
-TALER_MERCHANT_order_pay (struct GNUNET_CURL_Context *ctx,
-                          const char *merchant_url,
-                          const char *session_id,
-                          const struct GNUNET_HashCode *h_contract_terms,
-                          const struct TALER_Amount *amount,
-                          const struct TALER_Amount *max_fee,
-                          const struct TALER_MerchantPublicKeyP *merchant_pub,
-                          const struct TALER_MerchantSignatureP *merchant_sig,
-                          struct GNUNET_TIME_Absolute timestamp,
-                          struct GNUNET_TIME_Absolute refund_deadline,
-                          struct GNUNET_TIME_Absolute pay_deadline,
-                          const struct GNUNET_HashCode *h_wire,
-                          const char *order_id,
-                          unsigned int num_coins,
-                          const struct TALER_MERCHANT_PayCoin coins[],
-                          TALER_MERCHANT_OrderPayCallback pay_cb,
-                          void *pay_cb_cls)
+TALER_MERCHANT_order_pay (
+  struct GNUNET_CURL_Context *ctx,
+  const char *merchant_url,
+  const char *session_id,
+  const struct TALER_PrivateContractHashP *h_contract_terms,
+  const struct TALER_Amount *amount,
+  const struct TALER_Amount *max_fee,
+  const struct TALER_MerchantPublicKeyP *merchant_pub,
+  const struct TALER_MerchantSignatureP *merchant_sig,
+  struct GNUNET_TIME_Timestamp timestamp,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  struct GNUNET_TIME_Timestamp pay_deadline,
+  const struct TALER_MerchantWireHashP *h_wire,
+  const char *order_id,
+  unsigned int num_coins,
+  const struct TALER_MERCHANT_PayCoin coins[],
+  TALER_MERCHANT_OrderPayCallback pay_cb,
+  void *pay_cb_cls)
 {
-  (void) GNUNET_TIME_round_abs (&timestamp);
-  (void) GNUNET_TIME_round_abs (&pay_deadline);
-  (void) GNUNET_TIME_round_abs (&refund_deadline);
   if (GNUNET_YES !=
       TALER_amount_cmp_currency (amount,
                                  max_fee))
@@ -599,27 +740,14 @@ TALER_MERCHANT_order_pay (struct GNUNET_CURL_Context *ctx,
 
   {
     struct TALER_MERCHANT_PaidCoin pc[num_coins];
-    struct TALER_DepositRequestPS dr = {
-      .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT),
-      .purpose.size = htonl (sizeof (dr)),
-      .h_contract_terms = *h_contract_terms,
-      .h_wire = *h_wire,
-      .wallet_timestamp = GNUNET_TIME_absolute_hton (timestamp),
-      .refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline),
-      .merchant = *merchant_pub
-    };
 
     for (unsigned int i = 0; i<num_coins; i++)
     {
       const struct TALER_MERCHANT_PayCoin *coin = &coins[i]; // coin priv.
       struct TALER_MERCHANT_PaidCoin *p = &pc[i]; // coin pub.
       struct TALER_Amount fee;
+      struct TALER_DenominationHashP h_denom_pub;
 
-      /* prepare 'dr' for this coin to generate coin signature */
-      GNUNET_CRYPTO_eddsa_key_get_public (&coin->coin_priv.eddsa_priv,
-                                          &dr.coin_pub.eddsa_pub);
-      TALER_amount_hton (&dr.amount_with_fee,
-                         &coin->amount_with_fee);
       if (0 >
           TALER_amount_subtract (&fee,
                                  &coin->amount_with_fee,
@@ -630,17 +758,25 @@ TALER_MERCHANT_order_pay (struct GNUNET_CURL_Context *ctx,
         GNUNET_break (0);
         return NULL;
       }
-      TALER_amount_hton (&dr.deposit_fee,
-                         &fee);
-      GNUNET_CRYPTO_rsa_public_key_hash (coin->denom_pub.rsa_public_key,
-                                         &dr.h_denom_pub);
-      GNUNET_CRYPTO_eddsa_sign (&coin->coin_priv.eddsa_priv,
-                                &dr,
-                                &p->coin_sig.eddsa_signature);
+      TALER_denom_pub_hash (&coin->denom_pub,
+                            &h_denom_pub);
+      TALER_wallet_deposit_sign (&coin->amount_with_fee,
+                                 &fee,
+                                 h_wire,
+                                 h_contract_terms,
+                                 coin->h_age_commitment,
+                                 NULL /* h_extensions! */,
+                                 &h_denom_pub,
+                                 timestamp,
+                                 merchant_pub,
+                                 refund_deadline,
+                                 &coin->coin_priv,
+                                 &p->coin_sig);
       p->denom_pub = coin->denom_pub;
       p->denom_sig = coin->denom_sig;
       p->denom_value = coin->denom_value;
-      p->coin_pub = dr.coin_pub;
+      GNUNET_CRYPTO_eddsa_key_get_public (&coin->coin_priv.eddsa_priv,
+                                          &p->coin_pub.eddsa_pub);
       p->amount_with_fee = coin->amount_with_fee;
       p->amount_without_fee = coin->amount_without_fee;
       p->exchange_url = coin->exchange_url;
@@ -675,7 +811,14 @@ TALER_MERCHANT_order_pay_cancel (struct TALER_MERCHANT_OrderPayHandle *oph)
     GNUNET_CURL_job_cancel (oph->job);
     oph->job = NULL;
   }
+  if (NULL != oph->exchange)
+  {
+    TALER_EXCHANGE_disconnect (oph->exchange);
+    oph->exchange = NULL;
+  }
   TALER_curl_easy_post_finished (&oph->post_ctx);
+  json_decref (oph->error_history);
+  json_decref (oph->full_reply);
   GNUNET_free (oph->coins);
   GNUNET_free (oph->url);
   GNUNET_free (oph);
