@@ -27,6 +27,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_curl_lib.h>
@@ -101,7 +102,7 @@ handle_refund_finished (void *cls,
   case MHD_HTTP_OK:
     {
       const char *taler_refund_uri;
-      struct GNUNET_HashCode h_contract;
+      struct TALER_PrivateContractHashP h_contract;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_string ("taler_refund_uri",
                                  &taler_refund_uri),
@@ -221,24 +222,20 @@ TALER_MERCHANT_post_order_refund (struct GNUNET_CURL_Context *ctx,
     GNUNET_JSON_pack_string ("reason",
                              reason));
   GNUNET_assert (NULL != req);
-  eh = curl_easy_init ();
-  GNUNET_assert (NULL != eh);
+  eh = TALER_MERCHANT_curl_easy_get_ (orh->url);
   if (GNUNET_OK !=
       TALER_curl_easy_post (&orh->post_ctx,
                             eh,
                             req))
   {
     GNUNET_break (0);
+    curl_easy_cleanup (eh);
     json_decref (req);
     GNUNET_free (orh->url);
     GNUNET_free (orh);
     return NULL;
   }
   json_decref (req);
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   orh->url));
   orh->job = GNUNET_CURL_job_add2 (ctx,
                                    eh,
                                    orh->post_ctx.headers,
