@@ -29,6 +29,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_curl_lib.h>
@@ -91,7 +92,7 @@ handle_post_order_finished (void *cls,
     .hr.http_status = (unsigned int) response_code,
     .hr.reply = json
   };
-  struct TALER_ClaimTokenP token = {0};
+  struct TALER_ClaimTokenP token;
 
   po->job = NULL;
   switch (response_code)
@@ -101,12 +102,14 @@ handle_post_order_finished (void *cls,
     break;
   case MHD_HTTP_OK:
     {
+      bool no_token;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_string ("order_id",
                                  &por.details.ok.order_id),
         GNUNET_JSON_spec_mark_optional (
           GNUNET_JSON_spec_fixed_auto ("token",
-                                       &token)),
+                                       &token),
+          &no_token),
         GNUNET_JSON_spec_end ()
       };
 
@@ -121,7 +124,7 @@ handle_post_order_finished (void *cls,
       }
       else
       {
-        if (! GNUNET_is_zero (&token))
+        if (! no_token)
           por.details.ok.token = &token;
       }
     }
@@ -174,9 +177,10 @@ handle_post_order_finished (void *cls,
           "available_quantity",
           &por.details.gone.available_quantity),
         GNUNET_JSON_spec_mark_optional (
-          GNUNET_JSON_spec_absolute_time (
+          GNUNET_JSON_spec_timestamp (
             "restock_expected",
-            &por.details.gone.restock_expected)),
+            &por.details.gone.restock_expected),
+          NULL),
         GNUNET_JSON_spec_end ()
       };
 
@@ -247,7 +251,7 @@ TALER_MERCHANT_orders_post2 (
   unsigned int inventory_products_length,
   const struct TALER_MERCHANT_InventoryProduct inventory_products[],
   unsigned int uuids_length,
-  const struct GNUNET_Uuid uuids[],
+  const char *uuids[],
   bool create_token,
   TALER_MERCHANT_PostOrdersCallback cb,
   void *cb_cls)
@@ -310,9 +314,7 @@ TALER_MERCHANT_orders_post2 (
     {
       json_t *u;
 
-      u = GNUNET_JSON_PACK (
-        GNUNET_JSON_pack_data_auto ("uuid",
-                                    &uuids[i]));
+      u = json_string (uuids[i]);
       GNUNET_assert (0 ==
                      json_array_append_new (ua,
                                             u));
@@ -329,22 +331,19 @@ TALER_MERCHANT_orders_post2 (
                                         "create_token",
                                         json_boolean (create_token)));
   }
-  eh = curl_easy_init ();
-  GNUNET_assert (NULL != eh);
-  if (GNUNET_OK != TALER_curl_easy_post (&po->post_ctx,
-                                         eh,
-                                         req))
+  eh = TALER_MERCHANT_curl_easy_get_ (po->url);
+  if (GNUNET_OK !=
+      TALER_curl_easy_post (&po->post_ctx,
+                            eh,
+                            req))
   {
     GNUNET_break (0);
+    curl_easy_cleanup (eh);
     json_decref (req);
     GNUNET_free (po);
     return NULL;
   }
   json_decref (req);
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   po->url));
   po->job = GNUNET_CURL_job_add2 (ctx,
                                   eh,
                                   po->post_ctx.headers,
