@@ -28,6 +28,7 @@
 #include <microhttpd.h> /* just for HTTP status codes */
 #include <gnunet/gnunet_util_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_curl_lib.h>
 
@@ -169,14 +170,13 @@ TALER_MERCHANT_products_post (
   const json_t *taxes,
   int64_t total_stock,
   const json_t *address,
-  struct GNUNET_TIME_Absolute next_restock,
+  struct GNUNET_TIME_Timestamp next_restock,
   TALER_MERCHANT_ProductsPostCallback cb,
   void *cb_cls)
 {
   struct TALER_MERCHANT_ProductsPostHandle *pph;
   json_t *req_obj;
 
-  (void) GNUNET_TIME_round_abs (&next_restock);
   req_obj = GNUNET_JSON_PACK (
     GNUNET_JSON_pack_string ("product_id",
                              product_id),
@@ -197,8 +197,8 @@ TALER_MERCHANT_products_post (
     GNUNET_JSON_pack_object_incref ("address",
                                     (json_t *) address),
     GNUNET_JSON_pack_allow_null (
-      GNUNET_JSON_pack_time_abs ("next_restock",
-                                 next_restock)));
+      GNUNET_JSON_pack_timestamp ("next_restock",
+                                  next_restock)));
   pph = GNUNET_new (struct TALER_MERCHANT_ProductsPostHandle);
   pph->ctx = ctx;
   pph->cb = cb;
@@ -217,16 +217,12 @@ TALER_MERCHANT_products_post (
   {
     CURL *eh;
 
-    eh = curl_easy_init ();
-    GNUNET_assert (NULL != eh);
+    eh = TALER_MERCHANT_curl_easy_get_ (pph->url);
     GNUNET_assert (GNUNET_OK ==
                    TALER_curl_easy_post (&pph->post_ctx,
                                          eh,
                                          req_obj));
     json_decref (req_obj);
-    GNUNET_assert (CURLE_OK == curl_easy_setopt (eh,
-                                                 CURLOPT_URL,
-                                                 pph->url));
     pph->job = GNUNET_CURL_job_add2 (ctx,
                                      eh,
                                      pph->post_ctx.headers,
