@@ -26,6 +26,7 @@
 #include <microhttpd.h> /* just for HTTP status codes */
 #include <gnunet/gnunet_util_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_curl_lib.h>
 #include <taler/taler_json_lib.h>
 
@@ -204,22 +205,20 @@ TALER_MERCHANT_reserves_post (
                              wire_method),
     GNUNET_JSON_pack_string ("exchange_url",
                              exchange_url));
-  eh = curl_easy_init ();
-  GNUNET_assert (NULL != eh);
-  if (GNUNET_OK != TALER_curl_easy_post (&prh->post_ctx,
-                                         eh,
-                                         req))
+  eh = TALER_MERCHANT_curl_easy_get_ (prh->url);
+  if (GNUNET_OK !=
+      TALER_curl_easy_post (&prh->post_ctx,
+                            eh,
+                            req))
   {
     GNUNET_break (0);
+    curl_easy_cleanup (eh);
     json_decref (req);
+    GNUNET_free (prh->url);
     GNUNET_free (prh);
     return NULL;
   }
   json_decref (req);
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   prh->url));
   prh->job = GNUNET_CURL_job_add2 (ctx,
                                    eh,
                                    prh->post_ctx.headers,
