@@ -27,6 +27,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_curl_lib.h>
@@ -115,14 +116,14 @@ check_ok (struct TALER_MERCHANT_TipPickup2Handle *tpo,
     return GNUNET_SYSERR;
   }
   {
-    struct TALER_MERCHANT_BlindSignature mblind_sigs[ja_len];
-    struct GNUNET_CRYPTO_RsaSignature *blind_sigs[ja_len];
+    struct TALER_BlindedDenominationSignature mblind_sigs[ja_len];
 
     for (unsigned int i = 0; i<ja_len; i++)
     {
       json_t *pj = json_array_get (ja, i);
       struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_rsa_signature ("blind_sig", &blind_sigs[i]),
+        TALER_JSON_spec_blinded_denom_sig ("blind_sig",
+                                           &mblind_sigs[i]),
         GNUNET_JSON_spec_end ()
       };
 
@@ -135,14 +136,13 @@ check_ok (struct TALER_MERCHANT_TipPickup2Handle *tpo,
         GNUNET_JSON_parse_free (spec);
         return GNUNET_SYSERR;
       }
-      mblind_sigs[i].blind_sig = blind_sigs[i];
     }
     tpo->cb (tpo->cb_cls,
              &hr,
              ja_len,
              mblind_sigs);
     for (unsigned int i = 0; i<ja_len; i++)
-      GNUNET_CRYPTO_rsa_signature_free (blind_sigs[i]);
+      TALER_blinded_denom_sig_free (&mblind_sigs[i]);
     tpo->cb = NULL; /* do not call twice */
   }
   GNUNET_JSON_parse_free (spec);
@@ -232,7 +232,7 @@ handle_tip_pickup_finished (void *cls,
 struct TALER_MERCHANT_TipPickup2Handle *
 TALER_MERCHANT_tip_pickup2 (struct GNUNET_CURL_Context *ctx,
                             const char *backend_url,
-                            const struct GNUNET_HashCode *tip_id,
+                            const struct TALER_TipIdentifierP *tip_id,
                             unsigned int num_planchets,
                             const struct TALER_PlanchetDetail planchets[],
                             TALER_MERCHANT_TipPickup2Callback pickup_cb,
@@ -257,9 +257,8 @@ TALER_MERCHANT_tip_pickup2 (struct GNUNET_CURL_Context *ctx,
     p = GNUNET_JSON_PACK (
       GNUNET_JSON_pack_data_auto ("denom_pub_hash",
                                   &planchet->denom_pub_hash),
-      GNUNET_JSON_pack_data_varsize ("coin_ev",
-                                     planchet->coin_ev,
-                                     planchet->coin_ev_size));
+      TALER_JSON_pack_blinded_planchet ("coin_ev",
+                                        &planchet->blinded_planchet));
     if (0 !=
         json_array_append_new (pa,
                                p))
@@ -304,26 +303,23 @@ TALER_MERCHANT_tip_pickup2 (struct GNUNET_CURL_Context *ctx,
     GNUNET_free (tpo);
     return NULL;
   }
-  eh = curl_easy_init ();
-  if (GNUNET_OK != TALER_curl_easy_post (&tpo->post_ctx,
-                                         eh,
-                                         tp_obj))
+  eh = TALER_MERCHANT_curl_easy_get_ (tpo->url);
+  if (GNUNET_OK !=
+      TALER_curl_easy_post (&tpo->post_ctx,
+                            eh,
+                            tp_obj))
   {
     GNUNET_break (0);
     json_decref (tp_obj);
+    curl_easy_cleanup (eh);
     GNUNET_free (tpo->url);
     GNUNET_free (tpo);
     return NULL;
   }
   json_decref (tp_obj);
-
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Requesting URL '%s'\n",
               tpo->url);
-
-  GNUNET_assert (CURLE_OK == curl_easy_setopt (eh,
-                                               CURLOPT_URL,
-                                               tpo->url));
   tpo->job = GNUNET_CURL_job_add2 (ctx,
                                    eh,
                                    tpo->post_ctx.headers,
