@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2018, 2020 Taler Systems SA
+  Copyright (C) 2018, 2020, 2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -28,6 +28,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 
@@ -78,19 +79,13 @@ cb_failure (struct TALER_MERCHANT_OrderWalletGetHandle *owgh,
             enum TALER_ErrorCode ec,
             const json_t *reply)
 {
-  struct TALER_MERCHANT_HttpResponse hr = {
-    .ec = ec,
-    .reply = reply
+  struct TALER_MERCHANT_OrderWalletGetResponse owgr = {
+    .hr.ec = ec,
+    .hr.reply = reply
   };
 
   owgh->cb (owgh->cb_cls,
-            &hr,
-            GNUNET_SYSERR,
-            GNUNET_SYSERR,
-            GNUNET_SYSERR,
-            NULL,
-            NULL,
-            NULL);
+            &owgr);
 }
 
 
@@ -114,22 +109,19 @@ handle_wallet_get_order_finished (void *cls,
   {
   case MHD_HTTP_OK:
     {
-      struct TALER_Amount refund_amount;
-      bool refunded;
-      bool refund_pending;
+      struct TALER_MERCHANT_OrderWalletGetResponse owgr = {
+        .hr.reply = json,
+        .hr.http_status = MHD_HTTP_OK
+      };
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_bool ("refunded",
-                               &refunded),
+                               &owgr.details.success.refunded),
         GNUNET_JSON_spec_bool ("refund_pending",
-                               &refund_pending),
+                               &owgr.details.success.refund_pending),
         TALER_JSON_spec_amount_any ("refund_amount",
-                                    &refund_amount),
+                                    &owgr.details.success.refund_amount),
         GNUNET_JSON_spec_end ()
       };
-      struct TALER_MERCHANT_HttpResponse hr = {
-        .reply = json,
-        .http_status = MHD_HTTP_OK
-      };
 
       if (GNUNET_OK !=
           GNUNET_JSON_parse (json,
@@ -143,70 +135,54 @@ handle_wallet_get_order_finished (void *cls,
         TALER_MERCHANT_wallet_order_get_cancel (owgh);
         return;
       }
-
       owgh->cb (owgh->cb_cls,
-                &hr,
-                GNUNET_YES,
-                refunded ? GNUNET_YES : GNUNET_NO,
-                refund_pending ? GNUNET_YES : GNUNET_NO,
-                refunded ? &refund_amount : NULL,
-                NULL, /* paid! */
-                NULL);/* paid! */
+                &owgr);
       GNUNET_JSON_parse_free (spec);
       break;
     }
   case MHD_HTTP_PAYMENT_REQUIRED:
     {
+      struct TALER_MERCHANT_OrderWalletGetResponse owgr = {
+        .hr.reply = json,
+        .hr.http_status = MHD_HTTP_PAYMENT_REQUIRED
+      };
+
       /* Status is: unpaid */
-      const char *taler_pay_uri = json_string_value (json_object_get (json,
-                                                                      "taler_pay_uri"));
-      const char *already_paid = json_string_value (json_object_get (json,
-                                                                     "already_paid_order_id"));
-      if (NULL == taler_pay_uri)
+      owgr.details.payment_required.taler_pay_uri
+        = json_string_value (json_object_get (json,
+                                              "taler_pay_uri"));
+      owgr.details.payment_required.already_paid_order_id
+        = json_string_value (json_object_get (json,
+                                              "already_paid_order_id"));
+      if (NULL == owgr.details.payment_required.taler_pay_uri)
       {
         GNUNET_break_op (0);
         cb_failure (owgh,
                     TALER_EC_GENERIC_REPLY_MALFORMED,
                     json);
+        break;
       }
-      else
-      {
-        struct TALER_MERCHANT_HttpResponse hr = {
-          .reply = json,
-          .http_status = MHD_HTTP_OK
-        };
-
-        owgh->cb (owgh->cb_cls,
-                  &hr,
-                  GNUNET_NO,
-                  GNUNET_NO,
-                  GNUNET_NO,
-                  NULL,
-                  taler_pay_uri,
-                  already_paid);
-      }
+      owgh->cb (owgh->cb_cls,
+                &owgr);
       break;
     }
   default:
     {
-      struct TALER_MERCHANT_HttpResponse hr;
+      struct TALER_MERCHANT_OrderWalletGetResponse owgr = {
+        .hr.reply = json,
+        .hr.http_status = response_code
+      };
 
       TALER_MERCHANT_parse_error_details_ (response,
                                            response_code,
-                                           &hr);
+                                           &owgr.hr);
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                   "Checking order status failed with HTTP status code %u/%d\n",
                   (unsigned int) response_code,
-                  (int) hr.ec);
+                  (int) owgr.hr.ec);
       GNUNET_break_op (0);
       owgh->cb (owgh->cb_cls,
-                &hr,
-                GNUNET_SYSERR,
-                GNUNET_SYSERR,
-                GNUNET_SYSERR,
-                NULL,
-                NULL,
-                NULL);
+                &owgr);
       break;
     }
   }
@@ -215,16 +191,17 @@ handle_wallet_get_order_finished (void *cls,
 
 
 struct TALER_MERCHANT_OrderWalletGetHandle *
-TALER_MERCHANT_wallet_order_get (struct GNUNET_CURL_Context *ctx,
-                                 const char *backend_url,
-                                 const char *order_id,
-                                 const struct GNUNET_HashCode *h_contract,
-                                 struct GNUNET_TIME_Relative timeout,
-                                 const char *session_id,
-                                 const struct TALER_Amount *min_refund,
-                                 bool await_refund_obtained,
-                                 TALER_MERCHANT_OrderWalletGetCallback cb,
-                                 void *cb_cls)
+TALER_MERCHANT_wallet_order_get (
+  struct GNUNET_CURL_Context *ctx,
+  const char *backend_url,
+  const char *order_id,
+  const struct TALER_PrivateContractHashP *h_contract,
+  struct GNUNET_TIME_Relative timeout,
+  const char *session_id,
+  const struct TALER_Amount *min_refund,
+  bool await_refund_obtained,
+  TALER_MERCHANT_OrderWalletGetCallback cb,
+  void *cb_cls)
 {
   struct TALER_MERCHANT_OrderWalletGetHandle *owgh;
   unsigned long long tms;
@@ -249,7 +226,7 @@ TALER_MERCHANT_wallet_order_get (struct GNUNET_CURL_Context *ctx,
     struct GNUNET_CRYPTO_HashAsciiEncoded h_contract_s;
     char *path;
 
-    GNUNET_CRYPTO_hash_to_enc (h_contract,
+    GNUNET_CRYPTO_hash_to_enc (&h_contract->hash,
                                &h_contract_s);
     GNUNET_snprintf (timeout_ms,
                      sizeof (timeout_ms),
@@ -290,14 +267,7 @@ TALER_MERCHANT_wallet_order_get (struct GNUNET_CURL_Context *ctx,
   {
     CURL *eh;
 
-    eh = curl_easy_init ();
-    if (NULL == eh)
-    {
-      GNUNET_break (0);
-      GNUNET_free (owgh->url);
-      GNUNET_free (owgh);
-      return NULL;
-    }
+    eh = TALER_MERCHANT_curl_easy_get_ (owgh->url);
     if (0 != tms)
     {
       if (CURLE_OK !=
@@ -312,17 +282,6 @@ TALER_MERCHANT_wallet_order_get (struct GNUNET_CURL_Context *ctx,
         return NULL;
       }
     }
-    if (CURLE_OK !=
-        curl_easy_setopt (eh,
-                          CURLOPT_URL,
-                          owgh->url))
-    {
-      GNUNET_break (0);
-      curl_easy_cleanup (eh);
-      GNUNET_free (owgh->url);
-      GNUNET_free (owgh);
-      return NULL;
-    }
     if (CURLE_OK !=
         curl_easy_setopt (eh,
                           CURLOPT_TIMEOUT_MS,
