@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2021 Taler Systems SA
+  Copyright (C) 2014-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -29,6 +29,7 @@
 #include <taler/taler_exchange_service.h>
 #include <taler/taler_json_lib.h>
 #include <gnunet/gnunet_util_lib.h>
+#include <gnunet/gnunet_testing_lib.h>
 #include <microhttpd.h>
 #include <taler/taler_bank_service.h>
 #include <taler/taler_fakebank_lib.h>
@@ -36,13 +37,29 @@
 #include <taler/taler_error_codes.h>
 #include "taler_merchant_testing_lib.h"
 
+
+/**
+ * The 'poll-orders-conclude-1' and other 'conclude'
+ * commands should NOT wait for this timeout!
+ */
+#define POLL_ORDER_TIMEOUT \
+  GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, 60)
+
+/**
+ * The 'poll-orders-conclude-1x' and other 'conclude'
+ * commands that should (!) wait for this timeout! Hence,
+ * here we use a short value!
+ */
+#define POLL_ORDER_SHORT_TIMEOUT \
+  GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, 2)
+
 /**
  * Configuration file we use.  One (big) configuration is used
  * for the various components for this test.
  */
-#define CONFIG_FILE "test_merchant_api.conf"
+static char *config_file;
 
-#define PAYTO_I1 "payto://x-taler-bank/localhost/3"
+#define PAYTO_I1 "payto://x-taler-bank/localhost/3?receiver-name=3"
 
 /**
  * Exchange base URL.  Could also be taken from config.
@@ -155,7 +172,8 @@ const char *order_1_forgets_3[] = {
 static struct TALER_TESTING_Command
 cmd_exec_wirewatch (const char *label)
 {
-  return TALER_TESTING_cmd_exec_wirewatch (label, CONFIG_FILE);
+  return TALER_TESTING_cmd_exec_wirewatch (label,
+                                           config_file);
 }
 
 
@@ -166,8 +184,8 @@ cmd_exec_wirewatch (const char *label)
  * @param label label to use for the command.
  */
 #define CMD_EXEC_AGGREGATOR(label) \
-  TALER_TESTING_cmd_exec_aggregator (label "-aggregator", CONFIG_FILE), \
-  TALER_TESTING_cmd_exec_transfer (label "-transfer", CONFIG_FILE)
+  TALER_TESTING_cmd_exec_aggregator (label "-aggregator", config_file), \
+  TALER_TESTING_cmd_exec_transfer (label "-transfer", config_file)
 
 
 /**
@@ -206,19 +224,26 @@ run (void *cls,
                                                PAYTO_I1,
                                                "EUR",
                                                MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_kyc_get ("instance-create-kyc-0",
+                                        merchant_url,
+                                        NULL,
+                                        NULL,
+                                        EXCHANGE_URL,
+                                        MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_post_orders_no_claim ("create-proposal-4",
                                                      merchant_url,
                                                      MHD_HTTP_OK,
                                                      "4",
-                                                     GNUNET_TIME_UNIT_ZERO_ABS,
-                                                     GNUNET_TIME_UNIT_FOREVER_ABS,
+                                                     GNUNET_TIME_UNIT_ZERO_TS,
+                                                     GNUNET_TIME_UNIT_FOREVER_TS,
                                                      "EUR:5.0"),
     TALER_TESTING_cmd_merchant_get_order ("get-order-4",
                                           merchant_url,
                                           "create-proposal-4",
                                           TALER_MERCHANT_OSC_UNPAID,
                                           false,
-                                          MHD_HTTP_OK),
+                                          MHD_HTTP_OK,
+                                          NULL),
     TALER_TESTING_cmd_merchant_delete_order ("delete-order-4",
                                              merchant_url,
                                              "4",
@@ -237,10 +262,8 @@ run (void *cls,
     cmd_transfer_to_exchange ("create-reserve-1",
                               "EUR:10.02"),
     /**
-     * Make a reserve exist,
-     * according to the previous
-     * transfer.
-     *///
+     * Make a reserve exist, according to the previous transfer.
+     */
     cmd_exec_wirewatch ("wirewatch-1"),
     TALER_TESTING_cmd_check_bank_admin_transfer ("check_bank_transfer-2",
                                                  "EUR:10.02",
@@ -250,10 +273,12 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-1",
                                        "create-reserve-1",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-2",
                                        "create-reserve-1",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_get_orders ("get-orders-empty",
                                            merchant_url,
@@ -272,7 +297,7 @@ run (void *cls,
                                              MHD_HTTP_NOT_FOUND),
     TALER_TESTING_cmd_poll_orders_start ("poll-orders-1-start",
                                          merchant_url,
-                                         GNUNET_TIME_UNIT_MINUTES),
+                                         POLL_ORDER_TIMEOUT),
     TALER_TESTING_cmd_merchant_claim_order ("claim-order-nx",
                                             merchant_url,
                                             MHD_HTTP_NOT_FOUND,
@@ -282,8 +307,8 @@ run (void *cls,
                                              merchant_url,
                                              MHD_HTTP_OK,
                                              "1", /* order ID */
-                                             GNUNET_TIME_UNIT_ZERO_ABS,
-                                             GNUNET_TIME_UNIT_FOREVER_ABS,
+                                             GNUNET_TIME_UNIT_ZERO_TS,
+                                             GNUNET_TIME_UNIT_FOREVER_TS,
                                              true,
                                              "EUR:5.0",
                                              "x-taler-bank",
@@ -294,8 +319,8 @@ run (void *cls,
                                              merchant_url,
                                              MHD_HTTP_OK,
                                              "1", /* order ID */
-                                             GNUNET_TIME_UNIT_ZERO_ABS,
-                                             GNUNET_TIME_UNIT_FOREVER_ABS,
+                                             GNUNET_TIME_UNIT_ZERO_TS,
+                                             GNUNET_TIME_UNIT_FOREVER_TS,
                                              true,
                                              "EUR:5.0",
                                              "x-taler-bank",
@@ -306,8 +331,8 @@ run (void *cls,
                                              merchant_url,
                                              MHD_HTTP_OK,
                                              "1x", /* order ID */
-                                             GNUNET_TIME_UNIT_ZERO_ABS,
-                                             GNUNET_TIME_UNIT_FOREVER_ABS,
+                                             GNUNET_TIME_UNIT_ZERO_TS,
+                                             GNUNET_TIME_UNIT_FOREVER_TS,
                                              true,
                                              "EUR:5.0",
                                              "x-taler-bank",
@@ -333,8 +358,8 @@ run (void *cls,
                                             merchant_url,
                                             MHD_HTTP_CONFLICT,
                                             "1",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:5.0"),
     TALER_TESTING_cmd_poll_orders_conclude ("poll-orders-1-conclude",
                                             MHD_HTTP_OK,
@@ -351,28 +376,29 @@ run (void *cls,
                                         false,
                                         false,
                                         false,
-                                        MHD_HTTP_OK),
+                                        MHD_HTTP_PAYMENT_REQUIRED),
     TALER_TESTING_cmd_merchant_get_order ("get-order-merchant-1",
                                           merchant_url,
                                           "create-proposal-1",
                                           TALER_MERCHANT_OSC_CLAIMED,
                                           false,
-                                          MHD_HTTP_OK),
+                                          MHD_HTTP_OK,
+                                          NULL),
     TALER_TESTING_cmd_wallet_poll_order_start ("poll-order-wallet-start-1",
                                                merchant_url,
                                                "create-proposal-1",
-                                               GNUNET_TIME_UNIT_MINUTES,
+                                               POLL_ORDER_TIMEOUT,
                                                NULL),
     TALER_TESTING_cmd_wallet_poll_order_start2 ("poll-order-wallet-start-1x",
                                                 merchant_url,
                                                 "create-proposal-1x",
-                                                GNUNET_TIME_UNIT_MINUTES,
-                                                NULL,
+                                                POLL_ORDER_SHORT_TIMEOUT,
+                                                NULL, /* no refund */
                                                 "session-0"),
     TALER_TESTING_cmd_poll_order_start ("poll-order-merchant-1-start",
                                         merchant_url,
                                         "1",
-                                        GNUNET_TIME_UNIT_MINUTES),
+                                        POLL_ORDER_TIMEOUT),
     TALER_TESTING_cmd_merchant_pay_order ("deposit-simple",
                                           merchant_url,
                                           MHD_HTTP_OK,
@@ -388,8 +414,9 @@ run (void *cls,
                                                   MHD_HTTP_OK,
                                                   NULL,
                                                   "poll-order-wallet-start-1"),
+    /* Here we expect to run into a timeout, as we do not pay this one */
     TALER_TESTING_cmd_wallet_poll_order_conclude2 ("poll-order-1x-conclude",
-                                                   MHD_HTTP_OK,
+                                                   MHD_HTTP_PAYMENT_REQUIRED,
                                                    NULL,
                                                    "poll-order-wallet-start-1x",
                                                    "1"),
@@ -410,7 +437,8 @@ run (void *cls,
                                           "create-proposal-1",
                                           TALER_MERCHANT_OSC_PAID,
                                           false,
-                                          MHD_HTTP_OK),
+                                          MHD_HTTP_OK,
+                                          NULL),
     TALER_TESTING_cmd_merchant_get_orders ("get-orders-1-paid",
                                            merchant_url,
                                            MHD_HTTP_OK,
@@ -561,7 +589,7 @@ run (void *cls,
                                               5,
                                               0,
                                               json_object (),
-                                              GNUNET_TIME_relative_to_absolute (
+                                              GNUNET_TIME_relative_to_timestamp (
                                                 GNUNET_TIME_UNIT_MINUTES),
                                               MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_lock_product ("lock-product-p3",
@@ -574,8 +602,8 @@ run (void *cls,
                                              merchant_url,
                                              MHD_HTTP_NOT_FOUND,
                                              "order-p3",
-                                             GNUNET_TIME_UNIT_ZERO_ABS,
-                                             GNUNET_TIME_UNIT_FOREVER_ABS,
+                                             GNUNET_TIME_UNIT_ZERO_TS,
+                                             GNUNET_TIME_UNIT_FOREVER_TS,
                                              true, /* claim token */
                                              "EUR:5.0",
                                              "unsupported-wire-method",
@@ -586,8 +614,8 @@ run (void *cls,
                                              merchant_url,
                                              MHD_HTTP_NOT_FOUND,
                                              "order-p3",
-                                             GNUNET_TIME_UNIT_ZERO_ABS,
-                                             GNUNET_TIME_UNIT_FOREVER_ABS,
+                                             GNUNET_TIME_UNIT_ZERO_TS,
+                                             GNUNET_TIME_UNIT_FOREVER_TS,
                                              true,
                                              "EUR:5.0",
                                              "x-taler-bank",
@@ -599,8 +627,8 @@ run (void *cls,
       merchant_url,
       MHD_HTTP_GONE,
       "order-p3",
-      GNUNET_TIME_UNIT_ZERO_ABS,
-      GNUNET_TIME_UNIT_FOREVER_ABS,
+      GNUNET_TIME_UNIT_ZERO_TS,
+      GNUNET_TIME_UNIT_FOREVER_TS,
       true,
       "EUR:5.0",
       "x-taler-bank",
@@ -611,8 +639,8 @@ run (void *cls,
                                              merchant_url,
                                              MHD_HTTP_OK,
                                              "order-p3",
-                                             GNUNET_TIME_UNIT_ZERO_ABS,
-                                             GNUNET_TIME_UNIT_FOREVER_ABS,
+                                             GNUNET_TIME_UNIT_ZERO_TS,
+                                             GNUNET_TIME_UNIT_FOREVER_TS,
                                              false,
                                              "EUR:5.0",
                                              "x-taler-bank",
@@ -627,8 +655,8 @@ run (void *cls,
                                             merchant_url,
                                             MHD_HTTP_OK,
                                             NULL,
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:5.0"),
     TALER_TESTING_cmd_check_bank_empty ("check_bank_empty-2"),
     TALER_TESTING_cmd_end ()
@@ -639,8 +667,8 @@ run (void *cls,
                                             merchant_url,
                                             MHD_HTTP_OK,
                                             "2",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:5.0"),
     TALER_TESTING_cmd_merchant_claim_order ("fetch-proposal-2",
                                             merchant_url,
@@ -668,7 +696,7 @@ run (void *cls,
                               "EUR:10.02"),
     /**
      * Make a reserve exist, according to the previous transfer.
-     *///
+     */
     cmd_exec_wirewatch ("wirewatch-1r"),
     TALER_TESTING_cmd_check_bank_admin_transfer ("check_bank_transfer-2r",
                                                  "EUR:10.02",
@@ -678,10 +706,12 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-1r",
                                        "create-reserve-1r",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-2r",
                                        "create-reserve-1r",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
     /**
      * Check the reserve is depleted.
@@ -694,19 +724,19 @@ run (void *cls,
                                             merchant_url,
                                             MHD_HTTP_OK,
                                             "1r",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:5.0"),
     TALER_TESTING_cmd_wallet_poll_order_start ("poll-order-wallet-refund-1-low",
                                                merchant_url,
                                                "create-proposal-1r",
-                                               GNUNET_TIME_UNIT_MINUTES,
+                                               POLL_ORDER_TIMEOUT,
                                                "EUR:0.01"),
     TALER_TESTING_cmd_wallet_poll_order_start (
       "poll-order-wallet-refund-1-high",
       merchant_url,
       "create-proposal-1r",
-      GNUNET_TIME_UNIT_MINUTES,
+      POLL_ORDER_TIMEOUT,
       "EUR:0.2"),
     TALER_TESTING_cmd_merchant_pay_order ("pay-for-refund-1r",
                                           merchant_url,
@@ -719,7 +749,7 @@ run (void *cls,
     TALER_TESTING_cmd_poll_order_start ("poll-payment-refund-1",
                                         merchant_url,
                                         "1r", /* proposal name, not cmd ref! */
-                                        GNUNET_TIME_UNIT_MINUTES),
+                                        POLL_ORDER_TIMEOUT),
     TALER_TESTING_cmd_merchant_order_refund ("refund-increase-1r",
                                              merchant_url,
                                              "refund test",
@@ -796,8 +826,8 @@ run (void *cls,
                                             merchant_url,
                                             MHD_HTTP_OK,
                                             "1-unpaid",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:5.0"),
     /* Try to increase an unpaid proposal.  */
     TALER_TESTING_cmd_merchant_order_refund ("refund-increase-unpaid-proposal",
@@ -820,7 +850,7 @@ run (void *cls,
        it, and finally (4) attempt to pick up a refund
        from it without any increasing taking place
        in the first place.
-    *///
+    */
     cmd_transfer_to_exchange ("create-reserve-unincreased-refund",
                               "EUR:5.01"),
     cmd_exec_wirewatch ("wirewatch-unincreased-refund"),
@@ -833,14 +863,15 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-unincreased-refund",
                                        "create-reserve-unincreased-refund",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_post_orders (
       "create-proposal-unincreased-refund",
       merchant_url,
       MHD_HTTP_OK,
       "unincreased-proposal",
-      GNUNET_TIME_UNIT_ZERO_ABS,
-      GNUNET_TIME_UNIT_FOREVER_ABS,
+      GNUNET_TIME_UNIT_ZERO_TS,
+      GNUNET_TIME_UNIT_FOREVER_TS,
       "EUR:5.0"),
     TALER_TESTING_cmd_merchant_pay_order ("pay-unincreased-proposal",
                                           merchant_url,
@@ -854,8 +885,8 @@ run (void *cls,
     TALER_TESTING_cmd_check_bank_transfer (
       "check_bank_transfer-paid-unincreased-refund",
       EXCHANGE_URL,
-      "EUR:8.98", /* '4.98 from above', plus 4.99 from 'pay-for-refund-1r'
-                     and MINUS 0.1 MINUS 0.9 PLUS 0.01 (deposit fee) from
+      "EUR:8.97", /* '4.98 from above', plus 4.99 from 'pay-for-refund-1r'
+                     and MINUS 0.1 MINUS 0.9 from
                      'refund-increase-1r' and 'refund-increase-1r-2' */
       exchange_payto,
       merchant_payto),
@@ -878,12 +909,6 @@ run (void *cls,
                                              merchant_url_i1a,
                                              MHD_HTTP_OK,
                                              NULL),
-    TALER_TESTING_cmd_merchant_post_instance_auth (
-      "instance-create-i1a-auth-fail",
-      merchant_url,
-      "i1a",
-      "my-secret",
-      MHD_HTTP_BAD_REQUEST),
     TALER_TESTING_cmd_merchant_get_product ("get-nx-product-i1a-2",
                                             merchant_url_i1a,
                                             "nx-product",
@@ -1088,7 +1113,7 @@ run (void *cls,
                                                           "EUR:5.01",
                                                           TALER_EC_MERCHANT_PRIVATE_POST_TIP_AUTHORIZE_RESERVE_NOT_FOUND),
 
-    // Test reserve with insufficient funds
+    /* Test reserve with insufficient funds */
     TALER_TESTING_cmd_merchant_post_reserves ("create-reserve-tip-2",
                                               merchant_url,
                                               "EUR:1.04",
@@ -1111,7 +1136,6 @@ run (void *cls,
       "tip 4",
       "EUR:5.01",
       TALER_EC_MERCHANT_PRIVATE_POST_TIP_AUTHORIZE_INSUFFICIENT_FUNDS),
-
     TALER_TESTING_cmd_tip_authorize_fake ("fake-tip-authorization"),
     TALER_TESTING_cmd_tip_pickup_with_ec ("pickup-non-existent-id",
                                           merchant_url,
@@ -1141,32 +1165,44 @@ run (void *cls,
   };
 
   struct TALER_TESTING_Command pay_again[] = {
-    cmd_transfer_to_exchange ("create-reserve-10",
-                              "EUR:10.02"),
-    cmd_exec_wirewatch ("wirewatch-10"),
+    cmd_transfer_to_exchange ("create-reserve-20",
+                              "EUR:20.04"),
+    cmd_exec_wirewatch ("wirewatch-20"),
     TALER_TESTING_cmd_check_bank_admin_transfer ("check_bank_transfer-10",
-                                                 "EUR:10.02",
+                                                 "EUR:20.04",
                                                  payer_payto,
                                                  exchange_payto,
-                                                 "create-reserve-10"),
+                                                 "create-reserve-20"),
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-10a",
-                                       "create-reserve-10",
+                                       "create-reserve-20",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-10b",
-                                       "create-reserve-10",
+                                       "create-reserve-20",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
-    TALER_TESTING_cmd_status ("withdraw-status-10",
-                              "create-reserve-10",
+    TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-10c",
+                                       "create-reserve-20",
+                                       "EUR:5",
+                                       0,
+                                       MHD_HTTP_OK),
+    TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-10d",
+                                       "create-reserve-20",
+                                       "EUR:5",
+                                       0,
+                                       MHD_HTTP_OK),
+    TALER_TESTING_cmd_status ("withdraw-status-20",
+                              "create-reserve-20",
                               "EUR:0",
                               MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_post_orders ("create-proposal-10",
                                             merchant_url,
                                             MHD_HTTP_OK,
                                             "10",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:10.0"),
     TALER_TESTING_cmd_merchant_pay_order ("pay-fail-partial-double-10",
                                           merchant_url,
@@ -1184,6 +1220,14 @@ run (void *cls,
                                           "EUR:5",
                                           "EUR:4.99",
                                           NULL),
+    TALER_TESTING_cmd_merchant_pay_order ("pay-too-much-10",
+                                          merchant_url,
+                                          MHD_HTTP_CONFLICT,
+                                          "create-proposal-10",
+                                          "withdraw-coin-10c;withdraw-coin-10d",
+                                          "EUR:5",
+                                          "EUR:4.99",
+                                          NULL),
     CMD_EXEC_AGGREGATOR ("run-aggregator-10"),
     TALER_TESTING_cmd_check_bank_transfer ("check_bank_transfer-9.97-10",
                                            EXCHANGE_URL,
@@ -1206,10 +1250,12 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-11a",
                                        "create-reserve-11",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-11b",
                                        "create-reserve-11",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_status ("withdraw-status-11",
                               "create-reserve-11",
@@ -1219,8 +1265,8 @@ run (void *cls,
                                             merchant_url,
                                             MHD_HTTP_OK,
                                             "11",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:10.0"),
     TALER_TESTING_cmd_merchant_pay_order ("pay-fail-partial-double-11-good",
                                           merchant_url,
@@ -1253,13 +1299,13 @@ run (void *cls,
                                    MHD_HTTP_NO_CONTENT,
                                    false),
     TALER_TESTING_cmd_wire_add ("add-wire-account",
-                                "payto://x-taler-bank/localhost/2",
+                                "payto://x-taler-bank/localhost/2?receiver-name=2",
                                 MHD_HTTP_NO_CONTENT,
                                 false),
     TALER_TESTING_cmd_exec_offline_sign_keys ("offline-sign-future-keys",
-                                              CONFIG_FILE),
+                                              config_file),
     TALER_TESTING_cmd_exec_offline_sign_fees ("offline-sign-fees",
-                                              CONFIG_FILE,
+                                              config_file,
                                               "EUR:0.01",
                                               "EUR:0.01"),
     TALER_TESTING_cmd_check_keys_pull_all_keys ("refetch /keys",
@@ -1432,7 +1478,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_post_instances ("instance-create-i2-non-idem",
                                                merchant_url,
                                                "i2",
-                                               "payto://other-method/",
+                                               "payto://other-method/?receiver-name=X",
                                                "EUR",
                                                MHD_HTTP_CONFLICT),
     TALER_TESTING_cmd_merchant_delete_instance ("instance-delete-i2",
@@ -1523,7 +1569,7 @@ run (void *cls,
                                               json_pack ("{s:s}",
                                                          "street",
                                                          "pstreet"),
-                                              GNUNET_TIME_relative_to_absolute (
+                                              GNUNET_TIME_relative_to_timestamp (
                                                 GNUNET_TIME_UNIT_MINUTES),
                                               MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_get_product ("get-product-p2",
@@ -1550,7 +1596,7 @@ run (void *cls,
                                               json_pack ("{s:s}",
                                                          "street",
                                                          "pstreet"),
-                                              GNUNET_TIME_relative_to_absolute (
+                                              GNUNET_TIME_relative_to_timestamp (
                                                 GNUNET_TIME_UNIT_MINUTES),
                                               MHD_HTTP_NOT_FOUND),
     TALER_TESTING_cmd_merchant_delete_product ("get-products-empty",
@@ -1613,33 +1659,45 @@ int
 main (int argc,
       char *const *argv)
 {
-  unsigned int ret;
+  char *cipher;
+  enum GNUNET_GenericReturnValue ret;
+
   /* These environment variables get in the way... */
   unsetenv ("XDG_DATA_HOME");
   unsetenv ("XDG_CONFIG_HOME");
-
-  GNUNET_log_setup ("test-merchant-api",
-                    "DEBUG",
+  GNUNET_log_setup (argv[0],
+                    "INFO",
                     NULL);
-  if (GNUNET_OK != TALER_TESTING_prepare_fakebank (CONFIG_FILE,
-                                                   "exchange-account-exchange",
-                                                   &bc))
+  cipher = GNUNET_TESTING_get_testname_from_underscore (argv[0]);
+  GNUNET_assert (NULL != cipher);
+  GNUNET_asprintf (&config_file,
+                   "test_merchant_api-%s.conf",
+                   cipher);
+  GNUNET_free (cipher);
+  if (GNUNET_OK !=
+      TALER_TESTING_prepare_fakebank (config_file,
+                                      "exchange-account-exchange",
+                                      &bc))
     return 77;
 
-  payer_payto = ("payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME);
-  exchange_payto = ("payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME);
-  merchant_payto = ("payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME);
+  payer_payto =
+    ("payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME "?receiver-name="
+     USER_ACCOUNT_NAME);
+  exchange_payto =
+    ("payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME "?receiver-name="
+     EXCHANGE_ACCOUNT_NAME);
+  merchant_payto =
+    ("payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME "?receiver-name="
+     MERCHANT_ACCOUNT_NAME);
 
   if (NULL ==
-      (merchant_url = TALER_TESTING_prepare_merchant (CONFIG_FILE)))
+      (merchant_url = TALER_TESTING_prepare_merchant (config_file)))
     return 77;
-
   GNUNET_asprintf (&merchant_url_i1a,
                    "%sinstances/i1a/",
                    merchant_url);
-  TALER_TESTING_cleanup_files (CONFIG_FILE);
-
-  switch (TALER_TESTING_prepare_exchange (CONFIG_FILE,
+  TALER_TESTING_cleanup_files (config_file);
+  switch (TALER_TESTING_prepare_exchange (config_file,
                                           GNUNET_YES,
                                           &ec))
   {
@@ -1648,23 +1706,20 @@ main (int argc,
     return 1;
   case GNUNET_NO:
     return 77;
-
   case GNUNET_OK:
-
     if (NULL == (merchantd =
-                   TALER_TESTING_run_merchant (CONFIG_FILE,
+                   TALER_TESTING_run_merchant (config_file,
                                                merchant_url)))
       return 1;
 
     ret = TALER_TESTING_setup_with_exchange (&run,
                                              NULL,
-                                             CONFIG_FILE);
+                                             config_file);
 
     GNUNET_OS_process_kill (merchantd, SIGTERM);
     GNUNET_OS_process_wait (merchantd);
     GNUNET_OS_process_destroy (merchantd);
     GNUNET_free (merchant_url);
-
     if (GNUNET_OK != ret)
       return 1;
     break;
