@@ -32,6 +32,7 @@
 #include <taler/taler_exchange_service.h>
 #include <taler/taler_json_lib.h>
 #include <gnunet/gnunet_util_lib.h>
+#include <gnunet/gnunet_testing_lib.h>
 #include <microhttpd.h>
 #include <taler/taler_bank_service.h>
 #include <taler/taler_fakebank_lib.h>
@@ -44,7 +45,7 @@
  * Configuration file we use.  One (big) configuration is used
  * for the various components for this test.
  */
-#define CONFIG_FILE "test_merchant_api_twisted.conf"
+static char *config_file;
 
 /**
  * Account number of the exchange at the bank.
@@ -70,13 +71,13 @@
  * exchange.  Not used directly here in the code (instead
  * used in the merchant config), but kept around for consistency.
  */
-#define PROXY_EXCHANGE_CONFIG_FILE \
+#define PROXY_EXCHANGE_config_file \
   "test_merchant_api_proxy_exchange.conf"
 
 /**
  * Configuration file for the proxy between "lib" and merchant.
  */
-#define PROXY_MERCHANT_CONFIG_FILE \
+#define PROXY_MERCHANT_config_file \
   "test_merchant_api_proxy_merchant.conf"
 
 /**
@@ -150,7 +151,7 @@ static struct TALER_TESTING_ExchangeConfiguration ec;
 static struct TALER_TESTING_Command
 CMD_EXEC_WIREWATCH (const char *label)
 {
-  return TALER_TESTING_cmd_exec_wirewatch (label, CONFIG_FILE);
+  return TALER_TESTING_cmd_exec_wirewatch (label, config_file);
 }
 
 
@@ -161,8 +162,8 @@ CMD_EXEC_WIREWATCH (const char *label)
  * @param label label to use for the command.
  */
 #define CMD_EXEC_AGGREGATOR(label) \
-  TALER_TESTING_cmd_exec_aggregator (label "-aggregator", CONFIG_FILE), \
-  TALER_TESTING_cmd_exec_transfer (label "-transfer", CONFIG_FILE)
+  TALER_TESTING_cmd_exec_aggregator (label "-aggregator", config_file), \
+  TALER_TESTING_cmd_exec_transfer (label "-transfer", config_file)
 
 
 /**
@@ -214,6 +215,7 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-abort-1",
                                        "create-reserve-abort-1",
                                        "EUR:1",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_status ("withdraw-status-abort-1",
                               "create-reserve-abort-1",
@@ -223,8 +225,8 @@ run (void *cls,
                                             twister_merchant_url,
                                             MHD_HTTP_OK,
                                             "abort-one",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:3.0"),
     /* Will only pay _half_ the supposed price,
      * so we'll then have the right to abort.  */
@@ -237,21 +239,21 @@ run (void *cls,
                                           "EUR:1.00",
                                           NULL), // no sense now
     TALER_TESTING_cmd_delete_object ("hack-abort-1",
-                                     PROXY_MERCHANT_CONFIG_FILE,
+                                     PROXY_MERCHANT_config_file,
                                      "merchant_pub"),
     TALER_TESTING_cmd_merchant_order_abort ("pay-abort-1",
                                             twister_merchant_url,
                                             "deposit-simple-for-abort",
                                             MHD_HTTP_OK),
     TALER_TESTING_cmd_delete_object ("hack-abort-2",
-                                     PROXY_MERCHANT_CONFIG_FILE,
+                                     PROXY_MERCHANT_config_file,
                                      "refund_permissions.0.rtransaction_id"),
     TALER_TESTING_cmd_merchant_order_abort ("pay-abort-2",
                                             twister_merchant_url,
                                             "deposit-simple-for-abort",
                                             MHD_HTTP_OK),
     TALER_TESTING_cmd_modify_object_dl ("hack-abort-3",
-                                        PROXY_MERCHANT_CONFIG_FILE,
+                                        PROXY_MERCHANT_config_file,
                                         "refund_permissions.0.coin_pub",
                                         /* dummy coin.  */
                                         "8YX10E41ZWHX0X2RK4XFAXB2D3M05M1HNG14ZFZZB8M7SA4QCKCG"),
@@ -260,7 +262,7 @@ run (void *cls,
                                             "deposit-simple-for-abort",
                                             MHD_HTTP_OK),
     TALER_TESTING_cmd_flip_download ("hack-abort-4",
-                                     PROXY_MERCHANT_CONFIG_FILE,
+                                     PROXY_MERCHANT_config_file,
                                      "refund_permissions.0.merchant_sig"),
     TALER_TESTING_cmd_merchant_order_abort ("pay-abort-4",
                                             twister_merchant_url,
@@ -268,7 +270,7 @@ run (void *cls,
                                             MHD_HTTP_OK),
     /* just malforming the response.  */
     TALER_TESTING_cmd_malform_response ("malform-abortion",
-                                        PROXY_MERCHANT_CONFIG_FILE),
+                                        PROXY_MERCHANT_config_file),
     TALER_TESTING_cmd_merchant_order_abort ("pay-abort-5",
                                             twister_merchant_url,
                                             "deposit-simple-for-abort",
@@ -280,20 +282,21 @@ run (void *cls,
                                             twister_merchant_url,
                                             MHD_HTTP_OK,
                                             "DS-1",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:1.0"),
     TALER_TESTING_cmd_merchant_post_orders ("create-proposal-double-spend-1",
                                             twister_merchant_url,
                                             MHD_HTTP_OK,
                                             "DS-2",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:3.0"),
 
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-double-spend",
                                        "create-reserve-double-spend",
                                        "EUR:1",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_pay_order ("deposit-simple-ok",
                                           twister_merchant_url,
@@ -304,7 +307,7 @@ run (void *cls,
                                           "EUR:1.00",
                                           NULL), // no sense now
     TALER_TESTING_cmd_flip_download ("hack-coin-history",
-                                     PROXY_MERCHANT_CONFIG_FILE,
+                                     PROXY_MERCHANT_config_file,
                                      "history.0.coin_sig"),
     /* Coin history check will fail,
      * due to coin's bad signature.  */
@@ -330,9 +333,9 @@ run (void *cls,
                                 MHD_HTTP_NO_CONTENT,
                                 false),
     TALER_TESTING_cmd_exec_offline_sign_keys ("offline-sign-future-keys",
-                                              CONFIG_FILE),
+                                              config_file),
     TALER_TESTING_cmd_exec_offline_sign_fees ("offline-sign-fees",
-                                              CONFIG_FILE,
+                                              config_file,
                                               "EUR:0.01",
                                               "EUR:0.01"),
     TALER_TESTING_cmd_check_keys_pull_all_keys ("refetch /keys",
@@ -365,10 +368,12 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-1",
                                        "create-reserve-1",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-2",
                                        "create-reserve-1",
                                        "EUR:5",
+                                       0,
                                        MHD_HTTP_OK),
 
     TALER_TESTING_cmd_merchant_post_orders_no_claim (
@@ -376,11 +381,11 @@ run (void *cls,
       merchant_url,
       MHD_HTTP_OK,
       "1",
-      GNUNET_TIME_UNIT_ZERO_ABS,
-      GNUNET_TIME_UNIT_FOREVER_ABS,
+      GNUNET_TIME_UNIT_ZERO_TS,
+      GNUNET_TIME_UNIT_FOREVER_TS,
       "EUR:6.0"),
     TALER_TESTING_cmd_flip_upload ("hack-claim-token",
-                                   PROXY_MERCHANT_CONFIG_FILE,
+                                   PROXY_MERCHANT_config_file,
                                    "token"),
     TALER_TESTING_cmd_merchant_claim_order (
       "claim-1-incorrect-claim-token",
@@ -392,8 +397,8 @@ run (void *cls,
                                             merchant_url,
                                             MHD_HTTP_OK,
                                             "2",
-                                            GNUNET_TIME_UNIT_ZERO_ABS,
-                                            GNUNET_TIME_UNIT_FOREVER_ABS,
+                                            GNUNET_TIME_UNIT_ZERO_TS,
+                                            GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:6.0"),
     TALER_TESTING_cmd_merchant_pay_order ("deposit-2",
                                           merchant_url,
@@ -404,7 +409,7 @@ run (void *cls,
                                           "EUR:4.99",
                                           NULL),
     TALER_TESTING_cmd_malform_response ("malform-abort-merchant-exchange",
-                                        PROXY_EXCHANGE_CONFIG_FILE),
+                                        PROXY_EXCHANGE_config_file),
     TALER_TESTING_cmd_merchant_order_abort ("pay-abort-1",
                                             merchant_url,
                                             "deposit-2",
@@ -436,16 +441,25 @@ int
 main (int argc,
       char *const *argv)
 {
+  char *cipher;
   unsigned int ret;
+
   /* These environment variables get in the way... */
   unsetenv ("XDG_DATA_HOME");
   unsetenv ("XDG_CONFIG_HOME");
-  GNUNET_log_setup ("test-merchant-api-twisted",
-                    "DEBUG", NULL);
-
-  if (GNUNET_OK != TALER_TESTING_prepare_fakebank (CONFIG_FILE,
-                                                   "exchange-account-exchange",
-                                                   &bc))
+  GNUNET_log_setup (argv[0],
+                    "INFO",
+                    NULL);
+  cipher = GNUNET_TESTING_get_testname_from_underscore (argv[0]);
+  GNUNET_assert (NULL != cipher);
+  GNUNET_asprintf (&config_file,
+                   "test_merchant_api_twisted-%s.conf",
+                   cipher);
+  GNUNET_free (cipher);
+  if (GNUNET_OK !=
+      TALER_TESTING_prepare_fakebank (config_file,
+                                      "exchange-account-exchange",
+                                      &bc))
     return 77;
 
 
@@ -454,15 +468,15 @@ main (int argc,
   merchant_payto = ("payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME);
 
   if (NULL == (merchant_url = TALER_TESTING_prepare_merchant
-                                (CONFIG_FILE)))
+                                (config_file)))
     return 77;
 
   if (NULL == (twister_exchange_url = TALER_TWISTER_prepare_twister
-                                        (PROXY_EXCHANGE_CONFIG_FILE)))
+                                        (PROXY_EXCHANGE_config_file)))
     return 77;
 
   if (NULL == (twister_merchant_url = TALER_TWISTER_prepare_twister
-                                        (PROXY_MERCHANT_CONFIG_FILE)))
+                                        (PROXY_MERCHANT_config_file)))
     return 77;
 
   twister_merchant_url_instance_nonexistent = TALER_url_join (
@@ -470,9 +484,9 @@ main (int argc,
   twister_merchant_url_instance_tor = TALER_url_join (
     twister_merchant_url, "instances/tor/", NULL);
 
-  TALER_TESTING_cleanup_files (CONFIG_FILE);
+  TALER_TESTING_cleanup_files (config_file);
 
-  switch (TALER_TESTING_prepare_exchange (CONFIG_FILE,
+  switch (TALER_TESTING_prepare_exchange (config_file,
                                           GNUNET_YES,
                                           &ec))
   {
@@ -481,25 +495,24 @@ main (int argc,
     return 1;
   case GNUNET_NO:
     return 77;
-
   case GNUNET_OK:
 
     if (NULL == (merchantd = TALER_TESTING_run_merchant
-                               (CONFIG_FILE, merchant_url)))
+                               (config_file, merchant_url)))
       // 1 is fine; after all this is merchant test cases.
       return 1;
 
     if (NULL == (twisterexchanged = TALER_TWISTER_run_twister
-                                      (PROXY_EXCHANGE_CONFIG_FILE)))
+                                      (PROXY_EXCHANGE_config_file)))
       return 77;
 
     if (NULL == (twistermerchantd = TALER_TWISTER_run_twister
-                                      (PROXY_MERCHANT_CONFIG_FILE)))
+                                      (PROXY_MERCHANT_config_file)))
       return 77;
 
     /* Run the exchange and schedule 'run()' */
     ret = TALER_TESTING_setup_with_exchange (&run, NULL,
-                                             CONFIG_FILE);
+                                             config_file);
     purge_process (merchantd);
     purge_process (twisterexchanged);
     purge_process (twistermerchantd);
