@@ -113,10 +113,10 @@ get_orders_cb (void *cls,
         gos->orders[i]);
 
       {
-        const char *order_id;
+        const char **order_id;
+
         if (GNUNET_OK !=
             TALER_TESTING_get_trait_order_id (order_cmd,
-                                              0,
                                               &order_id))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -125,7 +125,7 @@ get_orders_cb (void *cls,
           return;
         }
         if (0 != strcmp (orders[i].order_id,
-                         order_id))
+                         *order_id))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Order id does not match\n");
@@ -147,7 +147,6 @@ get_orders_cb (void *cls,
 
         if (GNUNET_OK !=
             TALER_TESTING_get_trait_contract_terms (order_cmd,
-                                                    0,
                                                     &contract_terms))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -180,6 +179,9 @@ get_orders_cb (void *cls,
       }
     }
     break;
+  case MHD_HTTP_ACCEPTED:
+    /* FIXME: do more checks here (new KYC logic!) */
+    break;
   default:
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Unhandled HTTP status.\n");
@@ -191,7 +193,6 @@ get_orders_cb (void *cls,
 /**
  * Run the "GET /orders" CMD.
  *
- *
  * @param cls closure.
  * @param cmd command being run now.
  * @param is interpreter state.
@@ -375,9 +376,11 @@ conclude_task (void *cls)
     TALER_TESTING_FAIL (poc->is);
   }
   now = GNUNET_TIME_absolute_get ();
-  if ((GNUNET_TIME_absolute_add (pos->deadline,
-                                 GNUNET_TIME_UNIT_SECONDS).abs_value_us <
-       now.abs_value_us) )
+  if (GNUNET_TIME_absolute_cmp (GNUNET_TIME_absolute_add (
+                                  pos->deadline,
+                                  GNUNET_TIME_UNIT_SECONDS),
+                                <,
+                                now))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Expected answer to be delayed until %llu, but got response at %llu\n",
@@ -404,7 +407,6 @@ merchant_poll_orders_cb (
   unsigned int orders_length,
   const struct TALER_MERCHANT_OrderEntry orders[])
 {
-  /* FIXME, deeper checks should be implemented here. */
   struct MerchantPollOrdersStartState *pos = cls;
 
   pos->ogh = NULL;
@@ -421,8 +423,7 @@ merchant_poll_orders_cb (
   switch (hr->http_status)
   {
   case MHD_HTTP_OK:
-    // FIXME: use order references
-    // check if the data returned matches that from the POST / PATCH
+    // FIXME: use order references to check if the data returned matches that from the POST / PATCH
     break;
   default:
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
@@ -454,15 +455,16 @@ merchant_poll_orders_start_run (void *cls,
 
   /* add 1s grace time to timeout */
   pos->deadline
-    = GNUNET_TIME_absolute_add (GNUNET_TIME_relative_to_absolute (pos->timeout),
-                                GNUNET_TIME_UNIT_SECONDS);
+    = GNUNET_TIME_relative_to_absolute (
+        GNUNET_TIME_relative_add (pos->timeout,
+                                  GNUNET_TIME_UNIT_SECONDS));
   pos->is = is;
   pos->ogh = TALER_MERCHANT_orders_get2 (is->ctx,
                                          pos->merchant_url,
                                          TALER_EXCHANGE_YNA_ALL,
                                          TALER_EXCHANGE_YNA_ALL,
                                          TALER_EXCHANGE_YNA_ALL,
-                                         GNUNET_TIME_UNIT_ZERO_ABS,
+                                         GNUNET_TIME_UNIT_ZERO_TS,
                                          1,
                                          2,
                                          pos->timeout,
@@ -500,9 +502,6 @@ merchant_poll_orders_start_cleanup (void *cls,
 }
 
 
-/**
- * Start a long poll for GET /private/orders.
- */
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_poll_orders_start (const char *label,
                                      const char *merchant_url,
@@ -527,7 +526,7 @@ TALER_TESTING_cmd_poll_orders_start (const char *label,
 
 
 /**
- * Run the "GET orders" CMD.
+ * Wait for the "GET orders" CMD to complete.
  *
  * @param cls closure.
  * @param cmd command being run now.
@@ -587,9 +586,6 @@ merchant_poll_orders_conclude_cleanup (void *cls,
 }
 
 
-/**
- * Complete a long poll for GET /private/orders.
- */
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_poll_orders_conclude (const char *label,
                                         unsigned int http_status,
