@@ -72,7 +72,6 @@ get_reserves_cb (void *cls,
                  unsigned int reserves_length,
                  const struct TALER_MERCHANT_ReserveSummary reserves[])
 {
-  /* FIXME, deeper checks should be implemented here. */
   struct GetReservesState *grs = cls;
   bool matched[reserves_length];
   bool fail = false;
@@ -98,7 +97,7 @@ get_reserves_cb (void *cls,
       TALER_TESTING_interpreter_fail (grs->is);
       return;
     }
-    // FIXME: check if the data returned matches that from the POST / PATCH
+    /* check if the data returned matches that from the POST / PATCH */
     memset (matched, 0, sizeof (matched));
     for (unsigned int i = 0; i < reserves_length; ++i)
       for (unsigned int j = 0; j < reserves_length; ++j)
@@ -114,7 +113,6 @@ get_reserves_cb (void *cls,
 
           if (GNUNET_OK !=
               TALER_TESTING_get_trait_reserve_pub (reserve_cmd,
-                                                   0,
                                                    &reserve_pub))
           {
             GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -138,9 +136,8 @@ get_reserves_cb (void *cls,
           const struct TALER_Amount *initial;
 
           if (GNUNET_OK !=
-              TALER_TESTING_get_trait_amount_obj (reserve_cmd,
-                                                  0,
-                                                  &initial))
+              TALER_TESTING_get_trait_amount (reserve_cmd,
+                                              &initial))
           {
             GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                         "Could not fetch reserve initial balance\n");
@@ -199,7 +196,7 @@ get_reserves_run (void *cls,
   grs->is = is;
   grs->rgh = TALER_MERCHANT_reserves_get (is->ctx,
                                           grs->merchant_url,
-                                          GNUNET_TIME_UNIT_ZERO_ABS,
+                                          GNUNET_TIME_UNIT_ZERO_TS,
                                           TALER_EXCHANGE_YNA_ALL,
                                           TALER_EXCHANGE_YNA_ALL,
                                           &get_reserves_cb,
