@@ -137,7 +137,6 @@ get_transfers_cb (
 
         if (GNUNET_OK !=
             TALER_TESTING_get_trait_wtid (transfer_cmd,
-                                          0,
                                           &wtid))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -158,24 +157,23 @@ get_transfers_cb (
           transfers[i].credit_serial);
       }
       {
-        const char *payto_uri;
+        const char **payto_uri;
 
         if (GNUNET_OK !=
-            TALER_TESTING_get_trait_string (transfer_cmd,
-                                            0,
-                                            &payto_uri))
+            TALER_TESTING_get_trait_credit_payto_uri (transfer_cmd,
+                                                      &payto_uri))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Could not fetch wire transfer payto uri\n");
           TALER_TESTING_interpreter_fail (gts->is);
           return;
         }
-        if (0 != strcmp (payto_uri,
+        if (0 != strcmp (*payto_uri,
                          transfers[i].payto_uri))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Wire transfer payto uri does not match: %s != %s\n",
-                      payto_uri,
+                      *payto_uri,
                       transfers[i].payto_uri);
           TALER_TESTING_interpreter_fail (gts->is);
           return;
@@ -185,20 +183,19 @@ get_transfers_cb (
         const struct TALER_Amount *credit_amount;
 
         if (GNUNET_OK !=
-            TALER_TESTING_get_trait_amount_obj (transfer_cmd,
-                                                0,
-                                                &credit_amount))
+            TALER_TESTING_get_trait_amount (transfer_cmd,
+                                            &credit_amount))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Could not fetch wire transfer credit amount\n");
           TALER_TESTING_interpreter_fail (gts->is);
           return;
         }
-        if ((GNUNET_OK != TALER_amount_cmp_currency (credit_amount,
-                                                     &transfers[i].credit_amount))
-            ||
-            (0 != TALER_amount_cmp (credit_amount,
-                                    &transfers[i].credit_amount)))
+        if ( (GNUNET_OK !=
+              TALER_amount_cmp_currency (credit_amount,
+                                         &transfers[i].credit_amount)) ||
+             (0 != TALER_amount_cmp (credit_amount,
+                                     &transfers[i].credit_amount)))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Wire transfer credit amount does not match\n");
@@ -207,19 +204,18 @@ get_transfers_cb (
         }
       }
       {
-        const char *exchange_url;
+        const char **exchange_url;
 
         if (GNUNET_OK !=
-            TALER_TESTING_get_trait_string (transfer_cmd,
-                                            1,
-                                            &exchange_url))
+            TALER_TESTING_get_trait_exchange_url (transfer_cmd,
+                                                  &exchange_url))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Could not fetch wire transfer exchange url\n");
           TALER_TESTING_interpreter_fail (gts->is);
           return;
         }
-        if (0 != strcmp (exchange_url,
+        if (0 != strcmp (*exchange_url,
                          transfers[i].exchange_url))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -229,20 +225,21 @@ get_transfers_cb (
         }
       }
       {
-        const struct GNUNET_TIME_Absolute *execution_time;
+        const struct GNUNET_TIME_Timestamp *execution_time;
 
         if (GNUNET_OK !=
-            TALER_TESTING_get_trait_absolute_time (transfer_cmd,
-                                                   0,
-                                                   &execution_time))
+            TALER_TESTING_get_trait_timestamp (transfer_cmd,
+                                               0,
+                                               &execution_time))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Could not fetch wire transfer execution time\n");
           TALER_TESTING_interpreter_fail (gts->is);
           return;
         }
-        if (execution_time->abs_value_us !=
-            transfers[i].execution_time.abs_value_us)
+        if (GNUNET_TIME_timestamp_cmp (*execution_time,
+                                       !=,
+                                       transfers[i].execution_time))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Wire transfer execution time does not match\n");
@@ -278,8 +275,8 @@ get_transfers_run (void *cls,
   gts->gth = TALER_MERCHANT_transfers_get (is->ctx,
                                            gts->merchant_url,
                                            gts->payto_uri,
-                                           GNUNET_TIME_UNIT_FOREVER_ABS,
-                                           GNUNET_TIME_UNIT_ZERO_ABS,
+                                           GNUNET_TIME_UNIT_FOREVER_TS,
+                                           GNUNET_TIME_UNIT_ZERO_TS,
                                            INT64_MAX,
                                            0,
                                            TALER_EXCHANGE_YNA_ALL,
