@@ -130,42 +130,40 @@ apply_forget (void *cls,
  * Callback to process a GET /orders/$ID request
  *
  * @param cls closure
- * @param hr HTTP response details
- * @param osr order status response details (on success)
+ * @param osr order status response details
  */
 static void
 merchant_get_order_cb (
   void *cls,
-  const struct TALER_MERCHANT_HttpResponse *hr,
   const struct TALER_MERCHANT_OrderStatusResponse *osr)
 {
-  /* FIXME, deeper checks should be implemented here. */
   struct MerchantGetOrderState *gos = cls;
 
   gos->ogh = NULL;
-  if (gos->http_status != hr->http_status)
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "GET /private/orders/$ID completed with status %u\n",
+              osr->hr.http_status);
+  if (gos->http_status != osr->hr.http_status)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u (%d) to command %s\n",
-                hr->http_status,
-                (int) hr->ec,
+                osr->hr.http_status,
+                (int) osr->hr.ec,
                 TALER_TESTING_interpreter_get_current_label (gos->is));
     TALER_TESTING_interpreter_fail (gos->is);
     return;
   }
-  switch (hr->http_status)
+  switch (osr->hr.http_status)
   {
   case MHD_HTTP_OK:
-    // FIXME: use gts->tip_reference here to
-    // check if the data returned matches that from the POST / PATCH
-    if (gos->osc != osr->status)
+    if (gos->osc != osr->details.success.status)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Order paid does not match\n");
       TALER_TESTING_interpreter_fail (gos->is);
       return;
     }
-    switch (osr->status)
+    switch (osr->details.success.status)
     {
     case TALER_MERCHANT_OSC_PAID:
       {
@@ -182,7 +180,6 @@ merchant_get_order_cb (
 
           if (GNUNET_OK !=
               TALER_TESTING_get_trait_contract_terms (order_cmd,
-                                                      0,
                                                       &expected_contract_terms))
           {
             GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -197,16 +194,15 @@ merchant_get_order_cb (
           for (unsigned int i = 0; i < gos->forgets_length; ++i)
           {
             const struct TALER_TESTING_Command *forget_cmd;
-            const unsigned int *paths_length;
+            const uint32_t *paths_length;
 
             forget_cmd = TALER_TESTING_interpreter_lookup_command (
               gos->is,
               gos->forgets[i]);
 
             if (GNUNET_OK !=
-                TALER_TESTING_get_trait_uint32 (forget_cmd,
-                                                0,
-                                                &paths_length))
+                TALER_TESTING_get_trait_paths_length (forget_cmd,
+                                                      &paths_length))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Couldn't fetch forget paths length\n");
@@ -216,13 +212,13 @@ merchant_get_order_cb (
 
             for (unsigned int j = 0; j < *paths_length; ++j)
             {
-              const char *path;
+              const char **path;
               int res = GNUNET_OK;
 
               if (GNUNET_OK !=
-                  TALER_TESTING_get_trait_string (forget_cmd,
-                                                  j,
-                                                  &path))
+                  TALER_TESTING_get_trait_paths (forget_cmd,
+                                                 j,
+                                                 &path))
               {
                 GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                             "Couldn't fetch forget path\n");
@@ -232,7 +228,7 @@ merchant_get_order_cb (
 
               GNUNET_assert (GNUNET_OK ==
                              TALER_JSON_expand_path (ct,
-                                                     path,
+                                                     *path,
                                                      &apply_forget,
                                                      &res));
               GNUNET_assert (GNUNET_OK == res);
@@ -240,7 +236,7 @@ merchant_get_order_cb (
           }
 
           if (1 != json_equal (ct,
-                               osr->details.paid.contract_terms))
+                               osr->details.success.details.paid.contract_terms))
           {
             GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                         "Order contract terms do not match\n");
@@ -250,14 +246,14 @@ merchant_get_order_cb (
 
           json_decref (ct);
         }
-        if (gos->wired != osr->details.paid.wired)
+        if (gos->wired != osr->details.success.details.paid.wired)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Order wired does not match\n");
           TALER_TESTING_interpreter_fail (gos->is);
           return;
         }
-        if (gos->transfers_length != osr->details.paid.wts_len)
+        if (gos->transfers_length != osr->details.success.details.paid.wts_len)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Number of transfers found does not match\n");
@@ -276,7 +272,6 @@ merchant_get_order_cb (
 
             if (GNUNET_OK !=
                 TALER_TESTING_get_trait_wtid (transfer_cmd,
-                                              0,
                                               &wtid))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -285,7 +280,8 @@ merchant_get_order_cb (
               return;
             }
             if (0 != GNUNET_memcmp (wtid,
-                                    &osr->details.paid.wts[i].wtid))
+                                    &osr->details.success.details.paid.wts[i].
+                                    wtid))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Wire transfer id does not match\n");
@@ -294,20 +290,20 @@ merchant_get_order_cb (
             }
           }
           {
-            const char *exchange_url;
+            const char **exchange_url;
 
             if (GNUNET_OK !=
-                TALER_TESTING_get_trait_string (transfer_cmd,
-                                                1,
-                                                &exchange_url))
+                TALER_TESTING_get_trait_exchange_url (transfer_cmd,
+                                                      &exchange_url))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Could not fetch wire transfer exchange url\n");
               TALER_TESTING_interpreter_fail (gos->is);
               return;
             }
-            if (0 != strcmp (exchange_url,
-                             osr->details.paid.wts[i].exchange_url))
+            if (0 != strcmp (*exchange_url,
+                             osr->details.success.details.paid.wts[i].
+                             exchange_url))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Wire transfer exchange url does not match\n");
@@ -321,13 +317,11 @@ merchant_get_order_cb (
             const struct TALER_Amount *transfer_fee;
 
             if ((GNUNET_OK !=
-                 TALER_TESTING_get_trait_amount_obj (transfer_cmd,
-                                                     0,
-                                                     &transfer_amount)) ||
+                 TALER_TESTING_get_trait_amount (transfer_cmd,
+                                                 &transfer_amount)) ||
                 (GNUNET_OK !=
-                 TALER_TESTING_get_trait_amount_obj (transfer_cmd,
-                                                     1,
-                                                     &transfer_fee)))
+                 TALER_TESTING_get_trait_fee (transfer_cmd,
+                                              &transfer_fee)))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Could not fetch wire transfer amount/fee\n");
@@ -343,12 +337,13 @@ merchant_get_order_cb (
               TALER_TESTING_interpreter_fail (gos->is);
               return;
             }
-            if ((GNUNET_OK != TALER_amount_cmp_currency (&transfer_total,
-                                                         &osr->details.paid.wts[
-                                                           i]
-                                                         .total_amount)) ||
-                (0 != TALER_amount_cmp (&transfer_total,
-                                        &osr->details.paid.wts[i].total_amount)))
+            if ((GNUNET_OK !=
+                 TALER_amount_cmp_currency (
+                   &transfer_total,
+                   &osr->details.success.details.paid.wts[i].total_amount)) ||
+                (0 != TALER_amount_cmp (
+                   &transfer_total,
+                   &osr->details.success.details.paid.wts[i].total_amount)))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Wire transfer total does not match\n");
@@ -357,14 +352,15 @@ merchant_get_order_cb (
             }
           }
         }
-        if (gos->refunded != osr->details.paid.refunded)
+        if (gos->refunded != osr->details.success.details.paid.refunded)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Order refunded does not match\n");
           TALER_TESTING_interpreter_fail (gos->is);
           return;
         }
-        if (gos->refunds_length != osr->details.paid.refunds_len)
+        if (gos->refunds_length !=
+            osr->details.success.details.paid.refunds_len)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Number of refunds found does not match\n");
@@ -372,10 +368,11 @@ merchant_get_order_cb (
           return;
         }
         if (0 < gos->refunds_length)
-          GNUNET_assert (GNUNET_OK ==
-                         TALER_amount_set_zero (
-                           osr->details.paid.refund_amount.currency,
-                           &refunded_total));
+          GNUNET_assert (
+            GNUNET_OK ==
+            TALER_amount_set_zero (
+              osr->details.success.details.paid.refund_amount.currency,
+              &refunded_total));
         for (unsigned int i = 0; i < gos->refunds_length; ++i)
         {
           const struct TALER_TESTING_Command *refund_cmd;
@@ -384,31 +381,26 @@ merchant_get_order_cb (
             gos->is,
             gos->refunds[i]);
           {
-            const char *expected_amount_str;
-            struct TALER_Amount expected_amount;
+            const struct TALER_Amount *expected_amount;
             struct TALER_Amount *amount_found =
-              &osr->details.paid.refunds[i].refund_amount;
+              &osr->details.success.details.paid.refunds[i].refund_amount;
 
             if (GNUNET_OK !=
-                TALER_TESTING_get_trait_string (refund_cmd,
-                                                0,
-                                                &expected_amount_str))
+                TALER_TESTING_get_trait_amount (refund_cmd,
+                                                &expected_amount))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Could not fetch refund amount\n");
               TALER_TESTING_interpreter_fail (gos->is);
               return;
             }
-            GNUNET_assert (GNUNET_OK ==
-                           TALER_string_to_amount (expected_amount_str,
-                                                   &expected_amount));
             GNUNET_assert (0 <= TALER_amount_add (&refunded_total,
                                                   &refunded_total,
                                                   amount_found));
             if ((GNUNET_OK !=
-                 TALER_amount_cmp_currency (&expected_amount,
+                 TALER_amount_cmp_currency (expected_amount,
                                             &refunded_total)) ||
-                (0 != TALER_amount_cmp (&expected_amount,
+                (0 != TALER_amount_cmp (expected_amount,
                                         &refunded_total)))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -418,11 +410,10 @@ merchant_get_order_cb (
             }
           }
           {
-            const char *expected_reason;
+            const char **expected_reason;
 
             if (GNUNET_OK !=
-                TALER_TESTING_get_trait_string (refund_cmd,
-                                                1,
+                TALER_TESTING_get_trait_reason (refund_cmd,
                                                 &expected_reason))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -430,8 +421,9 @@ merchant_get_order_cb (
               TALER_TESTING_interpreter_fail (gos->is);
               return;
             }
-            if (0 != strcmp (expected_reason,
-                             osr->details.paid.refunds[i].reason))
+            if (0 != strcmp (
+                  *expected_reason,
+                  osr->details.success.details.paid.refunds[i].reason))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Refund reason does not match\n");
@@ -441,7 +433,7 @@ merchant_get_order_cb (
           }
         }
 
-        if (gos->wired != osr->details.paid.wired)
+        if (gos->wired != osr->details.success.details.paid.wired)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Order wired does not match\n");
@@ -455,19 +447,19 @@ merchant_get_order_cb (
       break;
     case TALER_MERCHANT_OSC_UNPAID:
       {
-        /* FIXME: Check all of the members of `pud` */
         struct TALER_MERCHANT_PayUriData pud;
         const struct TALER_TESTING_Command *order_cmd;
-        const char *order_id;
+        const char **order_id;
         const struct TALER_ClaimTokenP *claim_token;
 
         if (GNUNET_OK !=
-            TALER_MERCHANT_parse_pay_uri (osr->details.unpaid.taler_pay_uri,
-                                          &pud))
+            TALER_MERCHANT_parse_pay_uri (
+              osr->details.success.details.unpaid.taler_pay_uri,
+              &pud))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Taler pay uri `%s' is malformed\n",
-                      osr->details.unpaid.taler_pay_uri);
+                      osr->details.success.details.unpaid.taler_pay_uri);
           TALER_TESTING_interpreter_fail (gos->is);
           return;
         }
@@ -478,7 +470,6 @@ merchant_get_order_cb (
 
         if (GNUNET_OK !=
             TALER_TESTING_get_trait_order_id (order_cmd,
-                                              0,
                                               &order_id))
         {
           TALER_MERCHANT_parse_pay_uri_free (&pud);
@@ -487,7 +478,6 @@ merchant_get_order_cb (
 
         if (GNUNET_OK !=
             TALER_TESTING_get_trait_claim_token (order_cmd,
-                                                 0,
                                                  &claim_token))
         {
           TALER_MERCHANT_parse_pay_uri_free (&pud);
@@ -516,15 +506,15 @@ merchant_get_order_cb (
           if ((0 != strcmp (host,
                             pud.merchant_host)) ||
               (NULL != pud.merchant_prefix_path) ||
-              (0 != strcmp (order_id,
+              (0 != strcmp (*order_id,
                             pud.order_id)) ||
               (NULL != pud.ssid))
           {
             GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                         "Order pay uri `%s' does not match, wanted %s/%s\n",
-                        osr->details.unpaid.taler_pay_uri,
+                        osr->details.success.details.unpaid.taler_pay_uri,
                         host,
-                        order_id);
+                        *order_id);
             TALER_TESTING_interpreter_fail (gos->is);
             TALER_MERCHANT_parse_pay_uri_free (&pud);
             GNUNET_free (host);
@@ -552,8 +542,8 @@ merchant_get_order_cb (
           return;
         }
         TALER_MERCHANT_parse_pay_uri_free (&pud);
+        break;
       }
-      break;
     }
     break;
   default:
@@ -578,8 +568,8 @@ merchant_get_order_run (void *cls,
 {
   struct MerchantGetOrderState *gos = cls;
   const struct TALER_TESTING_Command *order_cmd;
-  const char *order_id;
-  const struct GNUNET_HashCode *h_contract;
+  const char **order_id;
+  const struct TALER_PrivateContractHashP *h_contract;
 
   order_cmd = TALER_TESTING_interpreter_lookup_command (
     is,
@@ -587,20 +577,18 @@ merchant_get_order_run (void *cls,
 
   if (GNUNET_OK !=
       TALER_TESTING_get_trait_order_id (order_cmd,
-                                        0,
                                         &order_id))
     TALER_TESTING_FAIL (is);
 
   if (GNUNET_OK !=
       TALER_TESTING_get_trait_h_contract_terms (order_cmd,
-                                                0,
                                                 &h_contract))
     TALER_TESTING_FAIL (is);
 
   gos->is = is;
   gos->ogh = TALER_MERCHANT_merchant_order_get (is->ctx,
                                                 gos->merchant_url,
-                                                order_id,
+                                                *order_id,
                                                 NULL,
                                                 true,
                                                 GNUNET_TIME_UNIT_ZERO,
@@ -705,12 +693,6 @@ TALER_TESTING_cmd_merchant_get_order2 (const char *label,
   gos->wired = wired;
   gos->refunded = refunded;
   gos->http_status = http_status;
-  gos->transfers = NULL;
-  gos->transfers_length = 0;
-  gos->refunds = NULL;
-  gos->refunds_length = 0;
-  gos->forgets = NULL;
-  gos->forgets_length = 0;
   if (wired)
   {
     for (const char **clabel = transfers; *clabel != NULL; ++clabel)
@@ -870,22 +852,23 @@ conclude_task (void *cls)
 
 
 /**
- * Callback to process a GET /orders/$ID request
+ * Callback to process a GET /private/orders/$ID request
  *
  * @param cls closure
- * @param hr HTTP response details
- * @param osr order status response details (on success)
+ * @param osr order status response details
  */
 static void
 merchant_poll_order_cb (
   void *cls,
-  const struct TALER_MERCHANT_HttpResponse *hr,
   const struct TALER_MERCHANT_OrderStatusResponse *osr)
 {
-  /* FIXME, deeper checks should be implemented here. */
   struct MerchantPollOrderStartState *pos = cls;
+  const struct TALER_MERCHANT_HttpResponse *hr = &osr->hr;
 
   pos->ogh = NULL;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "GET /private/orders/$ID finished with status %u.\n",
+              hr->http_status);
   pos->http_status = hr->http_status;
   switch (hr->http_status)
   {
@@ -1017,6 +1000,12 @@ merchant_poll_order_conclude_run (void *cls,
   GNUNET_assert (poll_cmd->run == &merchant_poll_order_start_run);
   pos = poll_cmd->cls;
   pos->cs = poc;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Waiting on GET /private/orders/$ID of %s (%s)\n",
+              poc->start_reference,
+              (NULL == pos->ogh)
+              ? "finished"
+              : "active");
   if (NULL == pos->ogh)
     poc->task = GNUNET_SCHEDULER_add_now (&conclude_task,
                                           poc);
