@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 Taler Systems SA
+  Copyright (C) 2014-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -35,11 +35,6 @@
 struct OrdersState
 {
 
-  /**
-   * The order.
-   */
-  char *order;
-
   /**
    * Expected status code.
    */
@@ -60,10 +55,15 @@ struct OrdersState
    */
   json_t *contract_terms;
 
+  /**
+   * Order submitted to the backend.
+   */
+  json_t *order_terms;
+
   /**
    * Contract terms hash code.
    */
-  struct GNUNET_HashCode h_contract_terms;
+  struct TALER_PrivateContractHashP h_contract_terms;
 
   /**
    * The /orders operation handle.
@@ -149,23 +149,22 @@ struct OrdersState
  * @param index index number of the object to extract.
  * @return #GNUNET_OK on success
  */
-static int
+static enum GNUNET_GenericReturnValue
 orders_traits (void *cls,
                const void **ret,
                const char *trait,
                unsigned int index)
 {
   struct OrdersState *ps = cls;
-
   struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_order_id (0, ps->order_id),
-    TALER_TESTING_make_trait_contract_terms (0, ps->contract_terms),
-    TALER_TESTING_make_trait_h_contract_terms (0, &ps->h_contract_terms),
-    TALER_TESTING_make_trait_merchant_sig (0, &ps->merchant_sig),
-    TALER_TESTING_make_trait_merchant_pub (0, &ps->merchant_pub),
-    TALER_TESTING_make_trait_claim_nonce (0, &ps->nonce),
-    TALER_TESTING_make_trait_claim_token (0, &ps->claim_token),
-    TALER_TESTING_make_trait_string (0, ps->order),
+    TALER_TESTING_make_trait_order_id (&ps->order_id),
+    TALER_TESTING_make_trait_contract_terms (ps->contract_terms),
+    TALER_TESTING_make_trait_order_terms (ps->order_terms),
+    TALER_TESTING_make_trait_h_contract_terms (&ps->h_contract_terms),
+    TALER_TESTING_make_trait_merchant_sig (&ps->merchant_sig),
+    TALER_TESTING_make_trait_merchant_pub (&ps->merchant_pub),
+    TALER_TESTING_make_trait_claim_nonce (&ps->nonce),
+    TALER_TESTING_make_trait_claim_token (&ps->claim_token),
     TALER_TESTING_trait_end ()
   };
 
@@ -192,7 +191,7 @@ orders_claim_cb (void *cls,
                  const struct TALER_MERCHANT_HttpResponse *hr,
                  const json_t *contract_terms,
                  const struct TALER_MerchantSignatureP *sig,
-                 const struct GNUNET_HashCode *hash)
+                 const struct TALER_PrivateContractHashP *hash)
 {
   struct OrdersState *ps = cls;
   struct TALER_MerchantPublicKeyP merchant_pub;
@@ -206,7 +205,13 @@ orders_claim_cb (void *cls,
 
   ps->och = NULL;
   if (ps->http_status != hr->http_status)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Expected status %u, got %u\n",
+                ps->http_status,
+                hr->http_status);
     TALER_TESTING_FAIL (ps->is);
+  }
 
   ps->contract_terms = json_deep_copy (contract_terms);
   ps->h_contract_terms = *hash;
@@ -292,7 +297,6 @@ order_cb (void *cls,
         ps->duplicate_of);
       if (GNUNET_OK !=
           TALER_TESTING_get_trait_claim_token (order_cmd,
-                                               0,
                                                &prev_token))
       {
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -371,35 +375,20 @@ orders_run (void *cls,
             struct TALER_TESTING_Interpreter *is)
 {
   struct OrdersState *ps = cls;
-  json_t *order;
-  json_error_t error;
 
   ps->is = is;
-  order = json_loads (ps->order,
-                      JSON_REJECT_DUPLICATES,
-                      &error);
-  if (NULL == order)
-  {
-    // human error here.
-    GNUNET_break (0);
-    fprintf (stderr, "%s\n", error.text);
-    TALER_TESTING_interpreter_fail (is);
-    return;
-  }
-
-  if (NULL == json_object_get (order,
+  if (NULL == json_object_get (ps->order_terms,
                                "order_id"))
   {
     struct GNUNET_TIME_Absolute now;
     char *order_id;
 
-    // FIXME: should probably use get_monotone() to ensure uniqueness!
-    now = GNUNET_TIME_absolute_get ();
-    order_id = GNUNET_STRINGS_data_to_string_alloc
-                 (&now.abs_value_us,
-                 sizeof (now.abs_value_us));
+    now = GNUNET_TIME_absolute_get_monotonic (is->cfg);
+    order_id = GNUNET_STRINGS_data_to_string_alloc (
+      &now,
+      sizeof (now));
     GNUNET_assert (0 ==
-                   json_object_set_new (order,
+                   json_object_set_new (ps->order_terms,
                                         "order_id",
                                         json_string (order_id)));
     GNUNET_free (order_id);
@@ -409,11 +398,10 @@ orders_run (void *cls,
                               sizeof (struct GNUNET_CRYPTO_EddsaPublicKey));
   ps->po = TALER_MERCHANT_orders_post (is->ctx,
                                        ps->merchant_url,
-                                       order,
+                                       ps->order_terms,
                                        GNUNET_TIME_UNIT_ZERO,
                                        &order_cb,
                                        ps);
-  json_decref (order);
   GNUNET_assert (NULL != ps->po);
 }
 
@@ -431,63 +419,62 @@ orders_run2 (void *cls,
              struct TALER_TESTING_Interpreter *is)
 {
   struct OrdersState *ps = cls;
-  const char *order_str = ps->order;
-  json_t *order;
-  json_error_t error;
+  const json_t *order;
   char *products_string = GNUNET_strdup (ps->products);
   char *locks_string = GNUNET_strdup (ps->locks);
   char *token;
   struct TALER_MERCHANT_InventoryProduct *products = NULL;
   unsigned int products_length = 0;
-  struct GNUNET_Uuid *locks = NULL;
+  const char **locks = NULL;
   unsigned int locks_length = 0;
 
   ps->is = is;
   if (NULL != ps->duplicate_of)
   {
     const struct TALER_TESTING_Command *order_cmd;
+    const json_t *ct;
+
     order_cmd = TALER_TESTING_interpreter_lookup_command (
       is,
       ps->duplicate_of);
     if (GNUNET_OK !=
-        TALER_TESTING_get_trait_string (order_cmd,
-                                        0,
-                                        &order_str))
+        TALER_TESTING_get_trait_order_terms (order_cmd,
+                                             &ct))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Could not fetch previous order string\n");
       TALER_TESTING_interpreter_fail (is);
       return;
     }
+    order = (json_t *) ct;
+  }
+  else
+  {
+    if (NULL == json_object_get (ps->order_terms,
+                                 "order_id"))
+    {
+      struct GNUNET_TIME_Absolute now;
+      char *order_id;
+
+      now = GNUNET_TIME_absolute_get_monotonic (is->cfg);
+      order_id = GNUNET_STRINGS_data_to_string_alloc (
+        &now.abs_value_us,
+        sizeof (now.abs_value_us));
+      GNUNET_assert (0 ==
+                     json_object_set_new (ps->order_terms,
+                                          "order_id",
+                                          json_string (order_id)));
+      GNUNET_free (order_id);
+    }
+    order = ps->order_terms;
   }
-  order = json_loads (order_str,
-                      JSON_REJECT_DUPLICATES,
-                      &error);
   if (NULL == order)
   {
-    // human error here.
     GNUNET_break (0);
-    fprintf (stderr, "%s\n", error.text);
     TALER_TESTING_interpreter_fail (is);
     return;
   }
 
-  if (NULL == json_object_get (order,
-                               "order_id"))
-  {
-    struct GNUNET_TIME_Absolute now;
-    char *order_id;
-
-    // FIXME: should probably use get_monotone() to ensure uniqueness!
-    now = GNUNET_TIME_absolute_get ();
-    order_id = GNUNET_STRINGS_data_to_string_alloc
-                 (&now.abs_value_us,
-                 sizeof (now.abs_value_us));
-    json_object_set_new (order,
-                         "order_id",
-                         json_string (order_id));
-    GNUNET_free (order_id);
-  }
   GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                               &ps->nonce,
                               sizeof (struct GNUNET_CRYPTO_EddsaPublicKey));
@@ -527,16 +514,15 @@ orders_run2 (void *cls,
        token = strtok (NULL, ";"))
   {
     const struct TALER_TESTING_Command *lock_cmd;
-    struct GNUNET_Uuid *uuid;
+    const char **uuid;
 
     lock_cmd = TALER_TESTING_interpreter_lookup_command (
       is,
       token);
 
     if (GNUNET_OK !=
-        TALER_TESTING_get_trait_uuid (lock_cmd,
-                                      0,
-                                      &uuid))
+        TALER_TESTING_get_trait_lock_uuid (lock_cmd,
+                                           &uuid))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Could not fetch lock uuid\n");
@@ -560,7 +546,6 @@ orders_run2 (void *cls,
                                         ps->make_claim_token,
                                         &order_cb,
                                         ps);
-  json_decref (order);
   GNUNET_free (products_string);
   GNUNET_free (locks_string);
   GNUNET_array_grow (products,
@@ -605,7 +590,7 @@ orders_cleanup (void *cls,
   }
 
   json_decref (ps->contract_terms);
-  GNUNET_free (ps->order);
+  json_decref (ps->order_terms);
   GNUNET_free_nz ((void *) ps->order_id);
   GNUNET_free (ps);
 }
@@ -640,18 +625,15 @@ mark_forgettable (void *cls,
  */
 static void
 make_order_json (const char *order_id,
-                 struct GNUNET_TIME_Absolute refund_deadline,
-                 struct GNUNET_TIME_Absolute pay_deadline,
+                 struct GNUNET_TIME_Timestamp refund_deadline,
+                 struct GNUNET_TIME_Timestamp pay_deadline,
                  const char *amount,
-                 char **order)
+                 json_t **order)
 {
-  struct GNUNET_TIME_Absolute refund = refund_deadline;
-  struct GNUNET_TIME_Absolute pay = pay_deadline;
+  struct GNUNET_TIME_Timestamp refund = refund_deadline;
+  struct GNUNET_TIME_Timestamp pay = pay_deadline;
   json_t *contract_terms;
 
-  GNUNET_TIME_round_abs (&refund);
-  GNUNET_TIME_round_abs (&pay);
-
   /* Include required fields and some dummy objects to test forgetting. */
   contract_terms = json_pack (
     "{s:s, s:s?, s:s, s:s, s:o, s:o, s:s, s:[{s:s}, {s:s}, {s:s}]}",
@@ -659,8 +641,8 @@ make_order_json (const char *order_id,
     "order_id", order_id,
     "amount", amount,
     "fulfillment_url", "https://example.com",
-    "refund_deadline", GNUNET_JSON_from_time_abs (refund),
-    "pay_deadline", GNUNET_JSON_from_time_abs (pay),
+    "refund_deadline", GNUNET_JSON_from_timestamp (refund),
+    "pay_deadline", GNUNET_JSON_from_timestamp (pay),
     "dummy_obj", "EUR:1.0",
     "dummy_array", /* For testing forgetting parts of arrays */
     "item", "speakers",
@@ -677,8 +659,7 @@ make_order_json (const char *order_id,
                                          "$.dummy_array[*].item",
                                          &mark_forgettable,
                                          NULL));
-  *order = json_dumps (contract_terms, 0);
-  json_decref (contract_terms);
+  *order = contract_terms;
 }
 
 
@@ -688,8 +669,8 @@ TALER_TESTING_cmd_merchant_post_orders_no_claim (
   const char *merchant_url,
   unsigned int http_status,
   const char *order_id,
-  struct GNUNET_TIME_Absolute refund_deadline,
-  struct GNUNET_TIME_Absolute pay_deadline,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  struct GNUNET_TIME_Timestamp pay_deadline,
   const char *amount)
 {
   struct OrdersState *ps;
@@ -699,7 +680,7 @@ TALER_TESTING_cmd_merchant_post_orders_no_claim (
                    refund_deadline,
                    pay_deadline,
                    amount,
-                   &ps->order);
+                   &ps->order_terms);
   ps->http_status = http_status;
   ps->expected_order_id = order_id;
   ps->merchant_url = merchant_url;
@@ -724,8 +705,8 @@ TALER_TESTING_cmd_merchant_post_orders (
   const char *merchant_url,
   unsigned int http_status,
   const char *order_id,
-  struct GNUNET_TIME_Absolute refund_deadline,
-  struct GNUNET_TIME_Absolute pay_deadline,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  struct GNUNET_TIME_Timestamp pay_deadline,
   const char *amount)
 {
   struct OrdersState *ps;
@@ -735,7 +716,7 @@ TALER_TESTING_cmd_merchant_post_orders (
                    refund_deadline,
                    pay_deadline,
                    amount,
-                   &ps->order);
+                   &ps->order_terms);
   ps->http_status = http_status;
   ps->expected_order_id = order_id;
   ps->merchant_url = merchant_url;
@@ -760,8 +741,8 @@ TALER_TESTING_cmd_merchant_post_orders2 (
   const char *merchant_url,
   unsigned int http_status,
   const char *order_id,
-  struct GNUNET_TIME_Absolute refund_deadline,
-  struct GNUNET_TIME_Absolute pay_deadline,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  struct GNUNET_TIME_Timestamp pay_deadline,
   bool claim_token,
   const char *amount,
   const char *payment_target,
@@ -776,8 +757,7 @@ TALER_TESTING_cmd_merchant_post_orders2 (
                    refund_deadline,
                    pay_deadline,
                    amount,
-                   &ps->order);
-
+                   &ps->order_terms);
   ps->http_status = http_status;
   ps->expected_order_id = order_id;
   ps->merchant_url = merchant_url;
