@@ -118,7 +118,7 @@ struct PostTransfersState
   /**
    * When the exchange executed the transfer.
    */
-  struct GNUNET_TIME_Absolute execution_time;
+  struct GNUNET_TIME_Timestamp execution_time;
 };
 
 
@@ -140,7 +140,7 @@ struct PostTransfersState
 static void
 transfers_cb (void *cls,
               const struct TALER_MERCHANT_HttpResponse *hr,
-              struct GNUNET_TIME_Absolute execution_time,
+              struct GNUNET_TIME_Timestamp execution_time,
               const struct TALER_Amount *total_amount,
               const struct TALER_Amount *wire_fee,
               unsigned int details_length,
@@ -332,7 +332,7 @@ transfers_cb (void *cls,
  * @param index index number of the object to extract.
  * @return #GNUNET_OK on success
  */
-static int
+static enum GNUNET_GenericReturnValue
 post_transfers_traits (void *cls,
                        const void **ret,
                        const char *trait,
@@ -340,12 +340,15 @@ post_transfers_traits (void *cls,
 {
   struct PostTransfersState *pts = cls;
   struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_wtid (0, &pts->wtid),
-    TALER_TESTING_make_trait_string (0, pts->credit_account),
-    TALER_TESTING_make_trait_amount_obj (0, &pts->credit_amount),
-    TALER_TESTING_make_trait_amount_obj (1, &pts->wire_fee),
-    TALER_TESTING_make_trait_string (1, pts->exchange_url),
-    TALER_TESTING_make_trait_absolute_time (0, &pts->execution_time),
+    TALER_TESTING_make_trait_wtid (&pts->wtid),
+    TALER_TESTING_make_trait_credit_payto_uri (
+      (const char **) &pts->credit_account),
+    TALER_TESTING_make_trait_amount (&pts->credit_amount),
+    TALER_TESTING_make_trait_fee (&pts->wire_fee),
+    TALER_TESTING_make_trait_exchange_url (
+      (const char **) &pts->exchange_url),
+    TALER_TESTING_make_trait_timestamp (0,
+                                        &pts->execution_time),
     TALER_TESTING_make_trait_bank_row (&pts->serial),
     TALER_TESTING_trait_end (),
   };
